/* Copyright (c) 1997-1999 Miller Puckette.
 * For information on usage and redistribution, and for a DISCLAIMER OF ALL
 * WARRANTIES, see the file, "LICENSE.txt," in this distribution. */

/* g_7_guis.c written by Thomas Musil (c) IEM KUG Graz Austria 2000-2001 */
/* thanks to Miller Puckette, Guenther Geiger and Krzystof Czaja */


#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "m_imp.h"
#include "g_canvas.h"
#include "t_tk.h"
#include "g_7_guis.h"

#ifdef NT
#include <io.h>
#else
#include <unistd.h>
#endif

/*  #define GGEE_HSLIDER_COMPATIBLE  */

/*------------------ global varaibles -------------------------*/

t_symbol *iemgui_key_sym=0;

int iemgui_color_hex[]=
{
    16579836, 10526880, 4210752, 16572640, 16572608,
    16579784, 14220504, 14220540, 14476540, 16308476,
    14737632, 8158332, 2105376, 16525352, 16559172,
    15263784, 1370132, 2684148, 3952892, 16003312,
    12369084, 6316128, 0, 9177096, 5779456,
    7874580, 2641940, 17488, 5256, 5767248
};

int iemgui_vu_db2i[]=
{
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
    3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
    4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
    5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
    6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
    7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
    8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
    9, 9, 9, 9, 9,10,10,10,10,10,
    11,11,11,11,11,12,12,12,12,12,
    13,13,13,13,14,14,14,14,15,15,
    15,15,16,16,16,16,17,17,17,18,
    18,18,19,19,19,20,20,20,21,21,
    22,22,23,23,24,24,25,26,27,28,
    29,30,31,32,33,33,34,34,35,35,
    36,36,37,37,37,38,38,38,39,39,
    39,39,39,39,40,40
};

int iemgui_vu_col[]=
{
    0,17,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
    15,15,15,15,15,15,15,15,15,15,14,14,13,13,13,13,13,13,13,13,13,13,13,19,19,19
};

char *iemgui_vu_scale_str[]=
{
    "",
    "<-99",
    "",
    "",
    "",
    "-50",
    "",
    "",
    "",
    "-30",
    "",
    "",
    "",
    "-20",
    "",
    "",
    "",
    "-12",
    "",
    "",
    "",
    "-6",
    "",
    "",
    "",
    "-2",
    "",
    "",
    "",
    "-0dB",
    "",
    "",
    "",
    "+2",
    "",
    "",
    "",
    "+6",
    "",
    "",
    "",
    ">+12",
    "",
    "",
    "",
    "",
    "",
};


/*------------------ global functions -------------------------*/


int iemgui_clip_size(int size)
{
    if(size < IEM_GUI_MINSIZE)
	size = IEM_GUI_MINSIZE;
    return(size);
}

int iemgui_clip_font(int size)
{
    if(size < IEM_FONT_MINSIZE)
	size = IEM_FONT_MINSIZE;
    return(size);
}

int iemgui_modulo_color(int col)
{
    while(col >= IEM_GUI_MAX_COLOR)
	col -= IEM_GUI_MAX_COLOR;
    while(col < 0)
	col += IEM_GUI_MAX_COLOR;
    return(col);
}

void iemgui_raute2dollar(t_symbol *s)
{
    if(s->s_name[0] == '#')
	s->s_name[0] = '$';
}

void iemgui_dollar2raute(t_symbol *s)
{
    if(s->s_name[0] == '$')
	s->s_name[0] = '#';
}

t_symbol *iemgui_unique2dollarzero(t_symbol *s, int unique_num, int and_unique_flag)
{
    if(and_unique_flag)
    {
	int l=0;
	char *b, str[144];

	sprintf(str, "%d", unique_num);
	while(str[l])
	{
	    if(s->s_name[l] != str[l])
		return(s);
	    else
		l++;
	}
	str[0] = '$';
	str[1] = '0';
	str[2] = 0;
	b = s->s_name + l;
	if(strlen(b) >= IEM_MAX_SYM_LEN)
	    strncat(str, b, IEM_MAX_SYM_LEN-1);
	else
	    strcat(str, b);
	return(gensym(str));
    }
    else
	return(s);
}

t_symbol *iemgui_sym2dollararg(t_symbol *s, int nth_arg, int tail_len)
{
    if(nth_arg)
    {
	char *b, str[144];
	int i=(int)strlen(s->s_name) - tail_len;

	sprintf(str, "_%d", nth_arg);
	str[0] = '$';
	if(i < 0) i = 0;
	b = s->s_name + i;
	strcat(str, b);
	return(gensym(str));
    }
    else
	return(s);
}

t_symbol *iemgui_dollarzero2unique(t_symbol *s, int unique_num)
{
    int l;
    char *b, str[144];

    sprintf(str, "%d", unique_num);
    l = (int)strlen(s->s_name);
    b = s->s_name + 2;
    if(l < 2)
	strcat(str, "shorty");
    else if(l >= IEM_MAX_SYM_LEN)
	strncat(str, b, IEM_MAX_SYM_LEN-1);
    else
	strcat(str, b);
    return(gensym(str));
}

t_symbol *iemgui_dollararg2sym(t_symbol *s, int nth_arg, int tail_len, int pargc, t_atom *pargv)
{
    int l=(int)strlen(s->s_name);
    char *b, str[288]="0";
    t_symbol *s2;

    if(pargc <= 0){}
    else if(nth_arg < 1){}
    else if(nth_arg > pargc){}
    else if(IS_A_FLOAT(pargv, nth_arg-1))
	sprintf(str, "%d", atom_getintarg(nth_arg-1, pargc, pargv));
    else if(IS_A_SYMBOL(pargv, nth_arg-1))
    {
	s2 = atom_getsymbolarg(nth_arg-1, pargc, pargv);
	strcpy(str, s2->s_name);
    }
    b = s->s_name + (l - tail_len);
    if(l <= tail_len)
	strcat(str, "shorty");
    else if(l >= IEM_MAX_SYM_LEN)
	strncat(str, b, IEM_MAX_SYM_LEN-1);
    else
	strcat(str, b);
    return(gensym(str));
}

int iemgui_is_dollarzero(t_symbol *s)
{
    char *name=s->s_name;

    if((int)strlen(name) >= 2)
    {
	if((name[0] == '$') && (name[1] == '0') && ((name[2] < '0') || (name[2] > '9')))
	    return(1);
    }
    return(0);
}

int iemgui_is_dollararg(t_symbol *s, int *tail_len)
{
    char *name=s->s_name;

    *tail_len = (int)strlen(name);
    if(*tail_len >= 2)
    {
	if((name[0] == '$') && (name[1] >= '1') && (name[1] <= '9'))
	{
	    int i=2, arg=(int)(name[1]-'0');

	    (*tail_len) -= 2;
	    while(name[i] && (name[i] >= '0') && (name[i] <= '9'))
	    {
		arg *= 10;
		arg += (int)(name[i]-'0');
		i++;
		(*tail_len)--;
	    }
	    return(arg);
	}
    }
    return(0);
}

void iemgui_fetch_unique(t_iemgui *iemgui)
{
    if(!iemgui->x_unique_num)
    {
	pd_bind(&iemgui->x_glist->gl_gobj.g_pd, gensym("#X"));
	iemgui->x_unique_num = canvas_getdollarzero();
	pd_unbind(&iemgui->x_glist->gl_gobj.g_pd, gensym("#X"));
    }
}

void iemgui_fetch_parent_args(t_iemgui *iemgui, int *pargc, t_atom **pargv)
{
    t_canvas *canvas=glist_getcanvas(iemgui->x_glist);

    canvas_setcurrent(canvas);
    canvas_getargs(pargc, pargv);
    canvas_unsetcurrent(canvas);
}

void iemgui_verify_snd_ne_rcv(t_iemgui *iemgui)
{
    iemgui->x_fsf.x_put_in2out = 1;
    if(iemgui->x_fsf.x_snd_able && iemgui->x_fsf.x_rcv_able)
    {
	if(!strcmp(iemgui->x_snd->s_name, iemgui->x_rcv->s_name))
	    iemgui->x_fsf.x_put_in2out = 0;
    }
}

void iemgui_all_unique2dollarzero(t_iemgui *iemgui, t_symbol **srlsym)
{
    iemgui_fetch_unique(iemgui);
    srlsym[0] = iemgui_unique2dollarzero(srlsym[0], iemgui->x_unique_num,
					  iemgui->x_fsf.x_snd_is_unique);
    srlsym[1] = iemgui_unique2dollarzero(srlsym[1], iemgui->x_unique_num,
					  iemgui->x_fsf.x_rcv_is_unique);
    srlsym[2] = iemgui_unique2dollarzero(srlsym[2], iemgui->x_unique_num,
					  iemgui->x_fsf.x_lab_is_unique);
}

void iemgui_all_sym2dollararg(t_iemgui *iemgui, t_symbol **srlsym)
{
    srlsym[0] = iemgui_sym2dollararg(srlsym[0], iemgui->x_isa.x_snd_is_arg_num,
				      iemgui->x_isa.x_snd_arg_tail_len);
    srlsym[1] = iemgui_sym2dollararg(srlsym[1], iemgui->x_isa.x_rcv_is_arg_num,
				      iemgui->x_isa.x_rcv_arg_tail_len);
    srlsym[2] = iemgui_sym2dollararg(srlsym[2], iemgui->x_fsf.x_lab_is_arg_num,
				      iemgui->x_fsf.x_lab_arg_tail_len);
}

void iemgui_all_dollarzero2unique(t_iemgui *iemgui, t_symbol **srlsym)
{
    iemgui_fetch_unique(iemgui);
    if(iemgui_is_dollarzero(srlsym[0]))
    {
	iemgui->x_fsf.x_snd_is_unique = 1;
	iemgui->x_isa.x_snd_is_arg_num = 0;
	iemgui->x_isa.x_snd_arg_tail_len = 0;
	srlsym[0] = iemgui_dollarzero2unique(srlsym[0], iemgui->x_unique_num);
    }
    else
	iemgui->x_fsf.x_snd_is_unique = 0;
    if(iemgui_is_dollarzero(srlsym[1]))
    {
	iemgui->x_fsf.x_rcv_is_unique = 1;
	iemgui->x_isa.x_rcv_is_arg_num = 0;
	iemgui->x_isa.x_rcv_arg_tail_len = 0;
	srlsym[1] = iemgui_dollarzero2unique(srlsym[1], iemgui->x_unique_num);
    }
    else
	iemgui->x_fsf.x_rcv_is_unique = 0;
    if(iemgui_is_dollarzero(srlsym[2]))
    {
	iemgui->x_fsf.x_lab_is_unique = 1;
	iemgui->x_fsf.x_lab_is_arg_num = 0;
	iemgui->x_fsf.x_lab_arg_tail_len = 0;
	srlsym[2] = iemgui_dollarzero2unique(srlsym[2], iemgui->x_unique_num);
    }
    else
	iemgui->x_fsf.x_lab_is_unique = 0;
}

void iemgui_all_dollararg2sym(t_iemgui *iemgui, t_symbol **srlsym)
{
    int pargc, tail_len, nth_arg;
    t_atom *pargv;

    iemgui_fetch_parent_args(iemgui, &pargc, &pargv);
    if(nth_arg = iemgui_is_dollararg(srlsym[0], &tail_len))
    {
	iemgui->x_isa.x_snd_is_arg_num = nth_arg;
	iemgui->x_isa.x_snd_arg_tail_len = tail_len;
	iemgui->x_fsf.x_snd_is_unique = 0;
	srlsym[0] = iemgui_dollararg2sym(srlsym[0], nth_arg, tail_len, pargc, pargv);
    }
    else
    {
	iemgui->x_isa.x_snd_is_arg_num = 0;
	iemgui->x_isa.x_snd_arg_tail_len = 0;
    }
    if(nth_arg = iemgui_is_dollararg(srlsym[1], &tail_len))
    {
	iemgui->x_isa.x_rcv_is_arg_num = nth_arg;
	iemgui->x_isa.x_rcv_arg_tail_len = tail_len;
	iemgui->x_fsf.x_rcv_is_unique = 0;
	srlsym[1] = iemgui_dollararg2sym(srlsym[1], nth_arg, tail_len, pargc, pargv);
    }
    else
    {
	iemgui->x_isa.x_rcv_is_arg_num = 0;
	iemgui->x_isa.x_rcv_arg_tail_len = 0;
    }
    if(nth_arg = iemgui_is_dollararg(srlsym[2], &tail_len))
    {
	iemgui->x_fsf.x_lab_is_arg_num = nth_arg;
	iemgui->x_fsf.x_lab_arg_tail_len = tail_len;
	iemgui->x_fsf.x_lab_is_unique = 0;
	srlsym[2] = iemgui_dollararg2sym(srlsym[2], nth_arg, tail_len, pargc, pargv);
    }
    else
    {
	iemgui->x_fsf.x_lab_is_arg_num = 0;
	iemgui->x_fsf.x_lab_arg_tail_len = 0;
    }
}

void iemgui_first_dollararg2sym(t_iemgui *iemgui, t_symbol **srlsym)
{
    int pargc=0, tail_len, nth_arg;
    t_atom pargv;
    char *name;

    SETFLOAT(&pargv, 0.0);
    name = srlsym[0]->s_name;
    if(iemgui->x_isa.x_snd_is_arg_num && (name[0] == '$')
       && (name[1] >= '1') && (name[1] <= '9'))
    {
	srlsym[0] = iemgui_dollararg2sym(srlsym[0], iemgui->x_isa.x_snd_is_arg_num,
					  iemgui->x_isa.x_snd_arg_tail_len, pargc, &pargv);
    }
    name = srlsym[1]->s_name;
    if(iemgui->x_isa.x_rcv_is_arg_num && (name[0] == '$')
       && (name[1] >= '1') && (name[1] <= '9'))
    {
	srlsym[1] = iemgui_dollararg2sym(srlsym[1], iemgui->x_isa.x_rcv_is_arg_num,
					  iemgui->x_isa.x_rcv_arg_tail_len, pargc, &pargv);
    }
    name = srlsym[2]->s_name;
    if(iemgui->x_fsf.x_lab_is_arg_num && (name[0] == '$')
       && (name[1] >= '1') && (name[1] <= '9'))
    {
	srlsym[2] = iemgui_dollararg2sym(srlsym[2], iemgui->x_fsf.x_lab_is_arg_num,
					  iemgui->x_fsf.x_lab_arg_tail_len, pargc, &pargv);
    }
}

void iemgui_all_col2save(t_iemgui *iemgui, int *bflcol)
{
    bflcol[0] = -1 - (((0xfc0000 & iemgui->x_bcol) >> 6)|
		      ((0xfc00 & iemgui->x_bcol) >> 4)|((0xfc & iemgui->x_bcol) >> 2));
    bflcol[1] = -1 - (((0xfc0000 & iemgui->x_fcol) >> 6)|
		      ((0xfc00 & iemgui->x_fcol) >> 4)|((0xfc & iemgui->x_fcol) >> 2));
    bflcol[2] = -1 - (((0xfc0000 & iemgui->x_lcol) >> 6)|
		      ((0xfc00 & iemgui->x_lcol) >> 4)|((0xfc & iemgui->x_lcol) >> 2));
}

void iemgui_all_colfromload(t_iemgui *iemgui, int *bflcol)
{
    if(bflcol[0] < 0)
    {
	bflcol[0] = -1 - bflcol[0];
	iemgui->x_bcol = ((bflcol[0] & 0x3f000) << 6)|((bflcol[0] & 0xfc0) << 4)|
	    ((bflcol[0] & 0x3f) << 2);
    }
    else
    {
	bflcol[0] = iemgui_modulo_color(bflcol[0]);
	iemgui->x_bcol = iemgui_color_hex[bflcol[0]];
    }
    if(bflcol[1] < 0)
    {
	bflcol[1] = -1 - bflcol[1];
	iemgui->x_fcol = ((bflcol[1] & 0x3f000) << 6)|((bflcol[1] & 0xfc0) << 4)|
	    ((bflcol[1] & 0x3f) << 2);
    }
    else
    {
	bflcol[1] = iemgui_modulo_color(bflcol[1]);
	iemgui->x_fcol = iemgui_color_hex[bflcol[1]];
    }
    if(bflcol[2] < 0)
    {
	bflcol[2] = -1 - bflcol[2];
	iemgui->x_lcol = ((bflcol[2] & 0x3f000) << 6)|((bflcol[2] & 0xfc0) << 4)|
	    ((bflcol[2] & 0x3f) << 2);
    }
    else
    {
	bflcol[2] = iemgui_modulo_color(bflcol[2]);
	iemgui->x_lcol = iemgui_color_hex[bflcol[2]];
    }
}

int iemgui_compatible_col(int i)
{
    int j;

    if(i >= 0)
    {
	j = iemgui_modulo_color(i);
	return(iemgui_color_hex[(j)]);
    }
    else
	return((-1 -i)&0xffffff);
}

void iemgui_all_dollar2raute(t_symbol **srlsym)
{
    if(srlsym[0]->s_name[0] == '$')
	srlsym[0]->s_name[0] = '#';
    if(srlsym[1]->s_name[0] == '$')
	srlsym[1]->s_name[0] = '#';
    if(srlsym[2]->s_name[0] == '$')
	srlsym[2]->s_name[0] = '#';
}

void iemgui_all_raute2dollar(t_symbol **srlsym)
{
    if(srlsym[0]->s_name[0] == '#')
	srlsym[0]->s_name[0] = '$';
    if(srlsym[1]->s_name[0] == '#')
	srlsym[1]->s_name[0] = '$';
    if(srlsym[2]->s_name[0] == '#')
	srlsym[2]->s_name[0] = '$';
}

void iemgui_send(void *x, t_iemgui *iemgui, t_symbol *s)
{
    t_symbol *snd;
    int pargc, tail_len, nth_arg, sndable=1, oldsndrcvable=0;
    t_atom *pargv;

    if(iemgui->x_fsf.x_rcv_able)
	oldsndrcvable += IEM_GUI_OLD_RCV_FLAG;
    if(iemgui->x_fsf.x_snd_able)
        oldsndrcvable += IEM_GUI_OLD_SND_FLAG;

    if(!strcmp(s->s_name, "empty")) sndable = 0;
    iemgui_raute2dollar(s);
    iemgui_fetch_unique(iemgui);
    snd = s;
    if(iemgui_is_dollarzero(snd))
    {
	iemgui->x_fsf.x_snd_is_unique = 1;
	iemgui->x_isa.x_snd_is_arg_num = 0;
	iemgui->x_isa.x_snd_arg_tail_len = 0;
	snd = iemgui_dollarzero2unique(snd, iemgui->x_unique_num);
    }
    else
	iemgui->x_fsf.x_snd_is_unique = 0;
    iemgui_fetch_parent_args(iemgui, &pargc, &pargv);
    if(nth_arg = iemgui_is_dollararg(snd, &tail_len))
    {
	iemgui->x_isa.x_snd_is_arg_num = nth_arg;
	iemgui->x_isa.x_snd_arg_tail_len = tail_len;
	iemgui->x_fsf.x_snd_is_unique = 0;
	snd = iemgui_dollararg2sym(snd, nth_arg, tail_len, pargc, pargv);
    }
    else
    {
	iemgui->x_isa.x_snd_is_arg_num = 0;
	iemgui->x_isa.x_snd_arg_tail_len = 0;
    }
    iemgui->x_snd = snd;
    iemgui->x_fsf.x_snd_able = sndable;
    iemgui_verify_snd_ne_rcv(iemgui);
    (*iemgui->x_draw)(x, iemgui->x_glist, IEM_GUI_DRAW_MODE_IO + oldsndrcvable);
}

void iemgui_receive(void *x, t_iemgui *iemgui, t_symbol *s)
{
    t_symbol *rcv;
    int pargc, tail_len, nth_arg, rcvable=1, oldsndrcvable=0;
    t_atom *pargv;

    if(iemgui->x_fsf.x_rcv_able)
	oldsndrcvable += IEM_GUI_OLD_RCV_FLAG;
    if(iemgui->x_fsf.x_snd_able)
        oldsndrcvable += IEM_GUI_OLD_SND_FLAG;

    if(!strcmp(s->s_name, "empty")) rcvable = 0;
    iemgui_raute2dollar(s);
    rcv = s;
    iemgui_fetch_unique(iemgui);
    if(iemgui_is_dollarzero(rcv))
    {
	iemgui->x_fsf.x_rcv_is_unique = 1;
	iemgui->x_isa.x_rcv_is_arg_num = 0;
	iemgui->x_isa.x_rcv_arg_tail_len = 0;
	rcv = iemgui_dollarzero2unique(rcv, iemgui->x_unique_num);
    }
    else
	iemgui->x_fsf.x_rcv_is_unique = 0;
    iemgui_fetch_parent_args(iemgui, &pargc, &pargv);
    if(nth_arg = iemgui_is_dollararg(rcv, &tail_len))
    {
	iemgui->x_isa.x_rcv_is_arg_num = nth_arg;
	iemgui->x_isa.x_rcv_arg_tail_len = tail_len;
	iemgui->x_fsf.x_rcv_is_unique = 0;
	rcv = iemgui_dollararg2sym(rcv, nth_arg, tail_len, pargc, pargv);
    }
    else
    {
	iemgui->x_isa.x_rcv_is_arg_num = 0;
	iemgui->x_isa.x_rcv_arg_tail_len = 0;
    }
    if(rcvable)
    {
	if(strcmp(rcv->s_name, iemgui->x_rcv->s_name))
	{
	    if(iemgui->x_fsf.x_rcv_able)
		pd_unbind(&iemgui->x_obj.ob_pd, iemgui->x_rcv);
	    iemgui->x_rcv = rcv;
	    pd_bind(&iemgui->x_obj.ob_pd, iemgui->x_rcv);
	}
    }
    else if(!rcvable && iemgui->x_fsf.x_rcv_able)
    {
	pd_unbind(&iemgui->x_obj.ob_pd, iemgui->x_rcv);
	iemgui->x_rcv = rcv;
    }
    iemgui->x_fsf.x_rcv_able = rcvable;
    iemgui_verify_snd_ne_rcv(iemgui);
    (*iemgui->x_draw)(x, iemgui->x_glist, IEM_GUI_DRAW_MODE_IO + oldsndrcvable);
}

void iemgui_label(void *x, t_iemgui *iemgui, t_symbol *s)
{
    t_symbol *lab;
    int pargc, tail_len, nth_arg;
    t_atom *pargv;

    iemgui_raute2dollar(s);
    lab = s;
    iemgui_fetch_unique(iemgui);

    if(iemgui_is_dollarzero(lab))
    {
	iemgui->x_fsf.x_lab_is_unique = 1;
	iemgui->x_fsf.x_lab_is_arg_num = 0;
	iemgui->x_fsf.x_lab_arg_tail_len = 0;
	lab = iemgui_dollarzero2unique(lab, iemgui->x_unique_num);
    }
    else
	iemgui->x_fsf.x_lab_is_unique = 0;

    iemgui_fetch_parent_args(iemgui, &pargc, &pargv);
    if(nth_arg = iemgui_is_dollararg(lab, &tail_len))
    {
	iemgui->x_fsf.x_lab_is_arg_num = nth_arg;
	iemgui->x_fsf.x_lab_arg_tail_len = tail_len;
	iemgui->x_fsf.x_lab_is_unique = 0;
	lab = iemgui_dollararg2sym(lab, nth_arg, tail_len, pargc, pargv);
    }
    else
    {
	iemgui->x_fsf.x_lab_is_arg_num = 0;
	iemgui->x_fsf.x_lab_arg_tail_len = 0;
    }
    iemgui->x_lab = lab;
    if(glist_isvisible(iemgui->x_glist))
	sys_vgui(".x%x.c itemconfigure %xLABEL -text {%s} \n",
		 glist_getcanvas(iemgui->x_glist), x,
		 strcmp(s->s_name, "empty")?iemgui->x_lab->s_name:"");
}

void iemgui_label_pos(void *x, t_iemgui *iemgui, t_symbol *s, int ac, t_atom *av)
{
    iemgui->x_ldx = (int)atom_getintarg(0, ac, av);
    iemgui->x_ldy = (int)atom_getintarg(1, ac, av);
    if(glist_isvisible(iemgui->x_glist))
	sys_vgui(".x%x.c coords %xLABEL %d %d\n",
		 glist_getcanvas(iemgui->x_glist), x,
		 iemgui->x_obj.te_xpos+iemgui->x_ldx,
		 iemgui->x_obj.te_ypos+iemgui->x_ldy);
}

void iemgui_label_font(void *x, t_iemgui *iemgui, t_symbol *s, int ac, t_atom *av)
{
    int f = (int)atom_getintarg(0, ac, av);

    if(f == 1) strcpy(iemgui->x_font, "helvetica");
    else if(f == 2) strcpy(iemgui->x_font, "times");
    else
    {
	f = 0;
	strcpy(iemgui->x_font, "courier");
    }
    iemgui->x_fsf.x_font_style = f;
    f = (int)atom_getintarg(1, ac, av);
    if(f < 4)
	f = 4;
    iemgui->x_fontsize = f;
    if(glist_isvisible(iemgui->x_glist))
	sys_vgui(".x%x.c itemconfigure %xLABEL -font {%s %d bold}\n",
		 glist_getcanvas(iemgui->x_glist), x, iemgui->x_font, iemgui->x_fontsize);
}

void iemgui_size(void *x, t_iemgui *iemgui)
{
    if(glist_isvisible(iemgui->x_glist))
    {
	(*iemgui->x_draw)(x, iemgui->x_glist, IEM_GUI_DRAW_MODE_MOVE);
	canvas_fixlinesfor(glist_getcanvas(iemgui->x_glist), (t_text*)x);
    }
}

void iemgui_delta(void *x, t_iemgui *iemgui, t_symbol *s, int ac, t_atom *av)
{
    iemgui->x_obj.te_xpos += (int)atom_getintarg(0, ac, av);
    iemgui->x_obj.te_ypos += (int)atom_getintarg(1, ac, av);
    if(glist_isvisible(iemgui->x_glist))
    {
	(*iemgui->x_draw)(x, iemgui->x_glist, IEM_GUI_DRAW_MODE_MOVE);
	canvas_fixlinesfor(glist_getcanvas(iemgui->x_glist), (t_text*)x);
    }
}

void iemgui_pos(void *x, t_iemgui *iemgui, t_symbol *s, int ac, t_atom *av)
{
    iemgui->x_obj.te_xpos = (int)atom_getintarg(0, ac, av);
    iemgui->x_obj.te_ypos = (int)atom_getintarg(1, ac, av);
    if(glist_isvisible(iemgui->x_glist))
    {
	(*iemgui->x_draw)(x, iemgui->x_glist, IEM_GUI_DRAW_MODE_MOVE);
	canvas_fixlinesfor(glist_getcanvas(iemgui->x_glist), (t_text*)x);
    }
}

void iemgui_color(void *x, t_iemgui *iemgui, t_symbol *s, int ac, t_atom *av)
{
    iemgui->x_bcol = iemgui_compatible_col(atom_getintarg(0, ac, av));
    if(ac > 2)
    {
	iemgui->x_fcol = iemgui_compatible_col(atom_getintarg(1, ac, av));
	iemgui->x_lcol = iemgui_compatible_col(atom_getintarg(2, ac, av));
    }
    else
	iemgui->x_lcol = iemgui_compatible_col(atom_getintarg(1, ac, av));
    if(glist_isvisible(iemgui->x_glist))
	(*iemgui->x_draw)(x, iemgui->x_glist, IEM_GUI_DRAW_MODE_CONFIG);
}

int iemgui_list(void *x, t_iemgui *iemgui, t_symbol *s, int ac, t_atom *av)
{
    if(iemgui->x_fsf.x_selected)
    {
	if((ac == 2)&&IS_A_FLOAT(av,0)&&IS_A_SYMBOL(av,1))
	{
	    t_symbol *key = atom_getsymbolarg(1, ac, av);
	    int keydown = atom_getintarg(0, ac, av);

	    if(keydown)
	    {
		int refresh = 1,i,d=1;
		static char buf[20];

		buf[0] = 0;
		if(!strcmp(key->s_name, "Shift_L")||!strcmp(key->s_name, "Shift_R"))
		    iemgui->x_fsf.x_shiftdown = 1;
		else
		{
		    if(iemgui->x_fsf.x_shiftdown)
			d = 10;
		    if(!strcmp(key->s_name, "Up"))
			iemgui->x_obj.te_ypos -= d;
		    else if(!strcmp(key->s_name, "Down"))
			iemgui->x_obj.te_ypos += d;
		    else if(!strcmp(key->s_name, "Left"))
			iemgui->x_obj.te_xpos -= d;
		    else if(!strcmp(key->s_name, "Right"))
			iemgui->x_obj.te_xpos += d;
		    else
			refresh = 0;
		    if(refresh)
			return(1);
		}
		return(0);
	    }
	    else
	    {
		if(!strcmp(key->s_name, "Shift_L")||!strcmp(key->s_name, "Shift_R"))
		    iemgui->x_fsf.x_shiftdown = 0;
		return(0);
	    }
	}
	else
	    return(-1);
    }
    else
	return(-1);
}

void iemgui_displace(t_gobj *z, t_glist *glist, int dx, int dy)
{
    t_iemguidummy *x = (t_iemguidummy *)z;

    x->x_gui.x_obj.te_xpos += dx;
    x->x_gui.x_obj.te_ypos += dy;
    (*x->x_gui.x_draw)((void *)z, glist, IEM_GUI_DRAW_MODE_MOVE);
    canvas_fixlinesfor(glist_getcanvas(glist), (t_text *)z);
}

void iemgui_select(t_gobj *z, t_glist *glist, int selected)
{
    t_iemguidummy *x = (t_iemguidummy *)z;

    x->x_gui.x_fsf.x_selected = selected;
    (*x->x_gui.x_draw)((void *)z, glist, IEM_GUI_DRAW_MODE_SELECT);
}

void iemgui_delete(t_gobj *z, t_glist *glist)
{
    canvas_deletelinesfor(glist_getcanvas(glist), (t_text *)z);
}

void iemgui_vis(t_gobj *z, t_glist *glist, int vis)
{
    t_iemguidummy *x = (t_iemguidummy *)z;
    t_rtext *y;

    if(vis)
    {
	y = rtext_new_without_senditup(glist, (t_text *)z, glist->gl_editor->e_rtext);
	(*x->x_gui.x_draw)((void *)z, glist, IEM_GUI_DRAW_MODE_NEW);
    }
    else
    {
	y = glist_findrtext(glist, (t_text *)z);
	(*x->x_gui.x_draw)((void *)z, glist, IEM_GUI_DRAW_MODE_ERASE);
	rtext_free(y);
    }
}

void iemgui_save(t_iemgui *iemgui, t_symbol **srl, int *bflcol)
{
    srl[0] = iemgui->x_snd;
    srl[1] = iemgui->x_rcv;
    srl[2] = iemgui->x_lab;
    iemgui_all_unique2dollarzero(iemgui, srl);
    iemgui_all_sym2dollararg(iemgui, srl);
    iemgui_all_col2save(iemgui, bflcol);
}

void iemgui_properties(t_iemgui *iemgui, t_symbol **srl)
{
    srl[0] = iemgui->x_snd;
    srl[1] = iemgui->x_rcv;
    srl[2] = iemgui->x_lab;
    iemgui_all_unique2dollarzero(iemgui, srl);
    iemgui_all_sym2dollararg(iemgui, srl);
    iemgui_all_dollar2raute(srl);
}

int iemgui_dialog(t_iemgui *iemgui, t_symbol **srl, int argc, t_atom *argv)
{
    char str[144];
    int init = (int)atom_getintarg(5, argc, argv);
    int ldx = (int)atom_getintarg(10, argc, argv);
    int ldy = (int)atom_getintarg(11, argc, argv);
    int f = (int)atom_getintarg(12, argc, argv);
    int fs = (int)atom_getintarg(13, argc, argv);
    int bcol = (int)atom_getintarg(14, argc, argv);
    int fcol = (int)atom_getintarg(15, argc, argv);
    int lcol = (int)atom_getintarg(16, argc, argv);
    int sndable=1, rcvable=1, oldsndrcvable=0;

    if(iemgui->x_fsf.x_rcv_able)
	oldsndrcvable += IEM_GUI_OLD_RCV_FLAG;
    if(iemgui->x_fsf.x_snd_able)
        oldsndrcvable += IEM_GUI_OLD_SND_FLAG;
    if(IS_A_SYMBOL(argv,7))
	srl[0] = atom_getsymbolarg(7, argc, argv);
    else if(IS_A_FLOAT(argv,7))
    {
	sprintf(str, "%d", (int)atom_getintarg(7, argc, argv));
	srl[0] = gensym(str);
    }
    if(IS_A_SYMBOL(argv,8))
	srl[1] = atom_getsymbolarg(8, argc, argv);
    else if(IS_A_FLOAT(argv,8))
    {
	sprintf(str, "%d", (int)atom_getintarg(8, argc, argv));
	srl[1] = gensym(str);
    }
    if(IS_A_SYMBOL(argv,9))
	srl[2] = atom_getsymbolarg(9, argc, argv);
    else if(IS_A_FLOAT(argv,9))
    {
	sprintf(str, "%d", (int)atom_getintarg(9, argc, argv));
	srl[2] = gensym(str);
    }
    if(init != 0) init = 1;
    iemgui->x_isa.x_loadinit = init;
    if(!strcmp(srl[0]->s_name, "empty")) sndable = 0;
    if(!strcmp(srl[1]->s_name, "empty")) rcvable = 0;
    iemgui_all_raute2dollar(srl);
    iemgui_all_dollarzero2unique(iemgui, srl);
    iemgui_all_dollararg2sym(iemgui, srl);
    if(rcvable)
    {
	if(strcmp(srl[1]->s_name, iemgui->x_rcv->s_name))
	{
	    if(iemgui->x_fsf.x_rcv_able)
		pd_unbind(&iemgui->x_obj.ob_pd, iemgui->x_rcv);
	    iemgui->x_rcv = srl[1];
	    pd_bind(&iemgui->x_obj.ob_pd, iemgui->x_rcv);
	}
    }
    else if(!rcvable && iemgui->x_fsf.x_rcv_able)
    {
	pd_unbind(&iemgui->x_obj.ob_pd, iemgui->x_rcv);
	iemgui->x_rcv = srl[1];
    }
    iemgui->x_snd = srl[0];
    iemgui->x_fsf.x_snd_able = sndable;
    iemgui->x_fsf.x_rcv_able = rcvable;
    iemgui->x_lcol = lcol & 0xffffff;
    iemgui->x_fcol = fcol & 0xffffff;
    iemgui->x_bcol = bcol & 0xffffff;
    iemgui->x_lab = srl[2];
    iemgui->x_ldx = ldx;
    iemgui->x_ldy = ldy;
    if(f == 1) strcpy(iemgui->x_font, "helvetica");
    else if(f == 2) strcpy(iemgui->x_font, "times");
    else
    {
	f = 0;
	strcpy(iemgui->x_font, "courier");
    }
    iemgui->x_fsf.x_font_style = f;
    if(fs < 4)
	fs = 4;
    iemgui->x_fontsize = fs;
    iemgui_verify_snd_ne_rcv(iemgui);
    return(oldsndrcvable);
}







/* --------------- bng     gui-bang ------------------------- */

t_widgetbehavior bng_widgetbehavior;
static t_class *bng_class;

/*  widget helper functions  */


void bng_draw_update(t_bng *x, t_glist *glist)
{
    if(glist_isvisible(glist))
    {
	sys_vgui(".x%x.c itemconfigure %xBUT -fill #%6.6x\n", glist_getcanvas(glist), x,
		 x->x_flashed?x->x_gui.x_fcol:x->x_gui.x_bcol);
    }
}

void bng_draw_new(t_bng *x, t_glist *glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c create rectangle %d %d %d %d -fill #%6.6x -tags %xBASE\n",
	     canvas, x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos,
	     x->x_gui.x_obj.te_xpos + x->x_gui.x_w, x->x_gui.x_obj.te_ypos + x->x_gui.x_h,
	     x->x_gui.x_bcol, x);
    sys_vgui(".x%x.c create oval %d %d %d %d -fill #%6.6x -tags %xBUT\n",
	     canvas, x->x_gui.x_obj.te_xpos+1, x->x_gui.x_obj.te_ypos+1,
	     x->x_gui.x_obj.te_xpos + x->x_gui.x_w-1, x->x_gui.x_obj.te_ypos + x->x_gui.x_h-1,
	     x->x_flashed?x->x_gui.x_fcol:x->x_gui.x_bcol, x);
    sys_vgui(".x%x.c create text %d %d -text {%s} -anchor w \
	     -font {%s %d bold} -fill #%6.6x -tags %xLABEL\n",
	     canvas, x->x_gui.x_obj.te_xpos+x->x_gui.x_ldx,
	     x->x_gui.x_obj.te_ypos+x->x_gui.x_ldy,
	     strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"",
	     x->x_gui.x_font, x->x_gui.x_fontsize, x->x_gui.x_lcol, x);
    if(!x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xOUT%d\n",
	     canvas, x->x_gui.x_obj.te_xpos,
	     x->x_gui.x_obj.te_ypos + x->x_gui.x_h-1, x->x_gui.x_obj.te_xpos + IOWIDTH,
	     x->x_gui.x_obj.te_ypos + x->x_gui.x_h, x, 0);
    if(!x->x_gui.x_fsf.x_rcv_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xIN%d\n",
	     canvas, x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos,
	     x->x_gui.x_obj.te_xpos + IOWIDTH, x->x_gui.x_obj.te_ypos+1, x, 0);
}

void bng_draw_move(t_bng *x, t_glist *glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c coords %xBASE %d %d %d %d\n",
	     canvas, x, x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos,
	     x->x_gui.x_obj.te_xpos + x->x_gui.x_w, x->x_gui.x_obj.te_ypos + x->x_gui.x_h);
    sys_vgui(".x%x.c coords %xBUT %d %d %d %d\n",
	     canvas, x, x->x_gui.x_obj.te_xpos+1,x->x_gui.x_obj.te_ypos+1,
	     x->x_gui.x_obj.te_xpos + x->x_gui.x_w-1, x->x_gui.x_obj.te_ypos + x->x_gui.x_h-1);
    sys_vgui(".x%x.c itemconfigure %xBUT -fill #%6.6x\n", canvas, x,
	     x->x_flashed?x->x_gui.x_fcol:x->x_gui.x_bcol);
    sys_vgui(".x%x.c coords %xLABEL %d %d\n",
	     canvas, x, x->x_gui.x_obj.te_xpos+x->x_gui.x_ldx, x->x_gui.x_obj.te_ypos+x->x_gui.x_ldy);
    if(!x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c coords %xOUT%d %d %d %d %d\n",
	     canvas, x, 0, x->x_gui.x_obj.te_xpos,
	     x->x_gui.x_obj.te_ypos + x->x_gui.x_h-1, x->x_gui.x_obj.te_xpos + IOWIDTH,
	     x->x_gui.x_obj.te_ypos + x->x_gui.x_h);
    if(!x->x_gui.x_fsf.x_rcv_able)
        sys_vgui(".x%x.c coords %xIN%d %d %d %d %d\n",
	     canvas, x, 0, x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos,
	     x->x_gui.x_obj.te_xpos + IOWIDTH, x->x_gui.x_obj.te_ypos+1);
}

void bng_draw_erase(t_bng* x, t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c delete %xBASE\n", canvas, x);
    sys_vgui(".x%x.c delete %xBUT\n", canvas, x);
    sys_vgui(".x%x.c delete %xLABEL\n", canvas, x);
    if(!x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c delete %xOUT%d\n", canvas, x, 0);
    if(!x->x_gui.x_fsf.x_rcv_able)
	sys_vgui(".x%x.c delete %xIN%d\n", canvas, x, 0);
}

void bng_draw_config(t_bng* x, t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c itemconfigure %xLABEL -font {%s %d bold} -fill #%6.6x -text {%s} \n",
	     canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize,
	     x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
	     strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"");
    sys_vgui(".x%x.c itemconfigure %xBASE -fill #%6.6x\n", canvas, x, x->x_gui.x_bcol);
    sys_vgui(".x%x.c itemconfigure %xBUT -fill #%6.6x\n", canvas, x,
	     x->x_flashed?x->x_gui.x_fcol:x->x_gui.x_bcol);
}

void bng_draw_io(t_bng* x, t_glist* glist, int old_snd_rcv_flags)
{
    t_canvas *canvas=glist_getcanvas(glist);

    if((old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && !x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xOUT%d\n",
	     canvas, x->x_gui.x_obj.te_xpos,
	     x->x_gui.x_obj.te_ypos + x->x_gui.x_h-1, x->x_gui.x_obj.te_xpos + IOWIDTH,
	     x->x_gui.x_obj.te_ypos + x->x_gui.x_h, x, 0);
    if(!(old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c delete %xOUT%d\n", canvas, x, 0);
    if((old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && !x->x_gui.x_fsf.x_rcv_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xIN%d\n",
	     canvas, x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos,
	     x->x_gui.x_obj.te_xpos + IOWIDTH, x->x_gui.x_obj.te_ypos+1, x, 0);
    if(!(old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && x->x_gui.x_fsf.x_rcv_able)
        sys_vgui(".x%x.c delete %xIN%d\n", canvas, x, 0);
}

void bng_draw_select(t_bng* x, t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    if(x->x_gui.x_fsf.x_selected)
    {
	pd_bind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
	sys_vgui(".x%x.c itemconfigure %xBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
	sys_vgui(".x%x.c itemconfigure %xBUT -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
	sys_vgui(".x%x.c itemconfigure %xLABEL -fill #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
    }
    else
    {
	pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
	sys_vgui(".x%x.c itemconfigure %xBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
	sys_vgui(".x%x.c itemconfigure %xBUT -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
	sys_vgui(".x%x.c itemconfigure %xLABEL -fill #%6.6x\n", canvas, x, x->x_gui.x_lcol);
    }
}

void bng_draw(t_bng *x, t_glist *glist, int mode)
{
    if(mode == IEM_GUI_DRAW_MODE_UPDATE)
	bng_draw_update(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_MOVE)
	bng_draw_move(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_NEW)
	bng_draw_new(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_SELECT)
	bng_draw_select(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_ERASE)
	bng_draw_erase(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_CONFIG)
	bng_draw_config(x, glist);
    else if(mode >= IEM_GUI_DRAW_MODE_IO)
	bng_draw_io(x, glist, mode - IEM_GUI_DRAW_MODE_IO);
}

/* ------------------------ bng widgetbehaviour----------------------------- */

static void bng_getrect(t_gobj *z, t_glist *owner, int *xp1, int *yp1, int *xp2, int *yp2)
{
    t_bng *x = (t_bng *)z;

    *xp1 = x->x_gui.x_obj.te_xpos;
    *yp1 = x->x_gui.x_obj.te_ypos;
    *xp2 = x->x_gui.x_obj.te_xpos + x->x_gui.x_w;
    *yp2 = x->x_gui.x_obj.te_ypos + x->x_gui.x_h;
}

static void bng_save(t_gobj *z, t_binbuf *b)
{
    t_bng *x = (t_bng *)z;
    int bflcol[3], *ip1, *ip2;
    t_symbol *srl[3];

    iemgui_save(&x->x_gui, srl, bflcol);
    ip1 = (int *)(&x->x_gui.x_isa);
    ip2 = (int *)(&x->x_gui.x_fsf);
    binbuf_addv(b, "ssiisiiiisssiiiiiii", gensym("#X"),gensym("obj"),
		(t_int)x->x_gui.x_obj.te_xpos, (t_int)x->x_gui.x_obj.te_ypos,
		gensym("bng"), x->x_gui.x_w,
		x->x_flashtime_hold, x->x_flashtime_break,
		(*ip1)&IEM_INIT_ARGS_ALL,
		srl[0], srl[1], srl[2],
		x->x_gui.x_ldx, x->x_gui.x_ldy,
		(*ip2)&IEM_FSTYLE_FLAGS_ALL, x->x_gui.x_fontsize,
		bflcol[0], bflcol[1], bflcol[2]);
    binbuf_addv(b, ";");
}

void bng_check_minmax(t_bng *x, int ftbreak, int fthold)
{
    if(ftbreak > fthold)
    {
	int h;

	h = ftbreak;
	ftbreak = fthold;
	fthold = h;
    }
    if(ftbreak < IEM_BNG_MINBREAKFLASHTIME)
	ftbreak = IEM_BNG_MINBREAKFLASHTIME;
    if(fthold < IEM_BNG_MINHOLDFLASHTIME)
	fthold = IEM_BNG_MINHOLDFLASHTIME;
    x->x_flashtime_break = ftbreak;
    x->x_flashtime_hold = fthold;
}

static void bng_properties(t_gobj *z, t_glist *owner)
{
    t_bng *x = (t_bng *)z;
    char buf[800];
    t_symbol *srl[3];

    iemgui_properties(&x->x_gui, srl);
    sprintf(buf, "pdtk_iemgui_dialog %%s BANG \
	    ----------dimensions(pix):----------- %d %d size: 0 0 empty \
	    --------flash-time(ms)(ms):--------- %d intrrpt: %d hold: %d \
	    %d empty empty %d %d \
	    %s %s \
	    %s %d %d \
	    %d %d \
	    %d %d %d\n",
	    x->x_gui.x_w, IEM_GUI_MINSIZE,
	    x->x_flashtime_break, x->x_flashtime_hold, 2,/*min_max_schedule+clip*/
	    -1, x->x_gui.x_isa.x_loadinit, -1,/*no linlog, no multi*/
	    srl[0]->s_name, srl[1]->s_name,
	    srl[2]->s_name, x->x_gui.x_ldx, x->x_gui.x_ldy,
	    x->x_gui.x_fsf.x_font_style, x->x_gui.x_fontsize,
	    0xffffff & x->x_gui.x_bcol, 0xffffff & x->x_gui.x_fcol, 0xffffff & x->x_gui.x_lcol);
    gfxstub_new(&x->x_gui.x_obj.ob_pd, x, buf);
}

static void bng_set(t_bng *x)
{
    if(x->x_flashed)
    {
	x->x_flashed = 0;
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
	clock_delay(x->x_clock_brk, x->x_flashtime_break);
	x->x_flashed = 1;
    }
    else
    {
	x->x_flashed = 1;
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
    }
    clock_delay(x->x_clock_hld, x->x_flashtime_hold);
}

static void bng_bout1(t_bng *x)/*wird nur mehr gesendet, wenn snd != rcv*/
{
    if(!x->x_gui.x_fsf.x_put_in2out)
    {
	x->x_gui.x_isa.x_locked = 1;
	clock_delay(x->x_clock_lck, 2);
    }
    outlet_bang(x->x_gui.x_obj.ob_outlet);
    if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing && x->x_gui.x_fsf.x_put_in2out)
	pd_bang(x->x_gui.x_snd->s_thing);
}

static void bng_bout2(t_bng *x)/*wird immer gesendet, wenn moeglich*/
{
    if(!x->x_gui.x_fsf.x_put_in2out)
    {
	x->x_gui.x_isa.x_locked = 1;
	clock_delay(x->x_clock_lck, 2);
    }
    outlet_bang(x->x_gui.x_obj.ob_outlet);
    if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
	pd_bang(x->x_gui.x_snd->s_thing);
}

static void bng_bang(t_bng *x)/*wird nur mehr gesendet, wenn snd != rcv*/
{
    if(!x->x_gui.x_isa.x_locked)
    {
	bng_set(x);
	bng_bout1(x);
    }
}

static void bng_bang2(t_bng *x)/*wird immer gesendet, wenn moeglich*/
{
    if(!x->x_gui.x_isa.x_locked)
    {
	bng_set(x);
	bng_bout2(x);
    }
}

static void bng_dialog(t_bng *x, t_symbol *s, int argc, t_atom *argv)
{
    t_symbol *srl[3];
    int a = (int)atom_getintarg(0, argc, argv);
    int fthold = (int)atom_getintarg(2, argc, argv);
    int ftbreak = (int)atom_getintarg(3, argc, argv);
    int sr_flags = iemgui_dialog(&x->x_gui, srl, argc, argv);

    x->x_gui.x_w = iemgui_clip_size(a);
    x->x_gui.x_h = x->x_gui.x_w;
    bng_check_minmax(x, ftbreak, fthold);
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_CONFIG);
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_IO + sr_flags);
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
    canvas_fixlinesfor(glist_getcanvas(x->x_gui.x_glist), (t_text*)x);
}

static void bng_click(t_bng *x, t_floatarg xpos, t_floatarg ypos, t_floatarg shift, t_floatarg ctrl, t_floatarg alt)
{
    bng_set(x);
    bng_bout2(x);
}

static int bng_newclick(t_gobj *z, struct _glist *glist, int xpix, int ypix, int shift, int alt, int dbl, int doit)
{
    if(doit)
	bng_click((t_bng *)z, (t_floatarg)xpix, (t_floatarg)ypix, (t_floatarg)shift, 0, (t_floatarg)alt);
    return (1);
}

static void bng_float(t_bng *x, t_floatarg f)
{bng_bang2(x);}

static void bng_symbol(t_bng *x, t_symbol *s)
{bng_bang2(x);}

static void bng_pointer(t_bng *x, t_gpointer *gp)
{bng_bang2(x);}

static void bng_list(t_bng *x, t_symbol *s, int ac, t_atom *av)
{
    int l=iemgui_list((void *)x, &x->x_gui, s, ac, av);

    if(l < 0)
    {
	bng_bang2(x);
    }
    else if(l > 0)
    {
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
	canvas_fixlinesfor(glist_getcanvas(x->x_gui.x_glist), (t_text*)x);
    }
}

static void bng_anything(t_bng *x, t_symbol *s, int argc, t_atom *argv)
{bng_bang2(x);}

static void bng_loadbang(t_bng *x)
{
    if(!sys_noloadbang && x->x_gui.x_isa.x_loadinit)
    {
        bng_set(x);
	bng_bout2(x);
    }
}

static void bng_size(t_bng *x, t_symbol *s, int ac, t_atom *av)
{
    x->x_gui.x_w = iemgui_clip_size((int)atom_getintarg(0, ac, av));
    x->x_gui.x_h = x->x_gui.x_w;
    iemgui_size((void *)x, &x->x_gui);
}

static void bng_delta(t_bng *x, t_symbol *s, int ac, t_atom *av)
{iemgui_delta((void *)x, &x->x_gui, s, ac, av);}

static void bng_pos(t_bng *x, t_symbol *s, int ac, t_atom *av)
{iemgui_pos((void *)x, &x->x_gui, s, ac, av);}

static void bng_flashtime(t_bng *x, t_symbol *s, int ac, t_atom *av)
{
    bng_check_minmax(x, (int)atom_getintarg(0, ac, av),
		     (int)atom_getintarg(1, ac, av));
}

static void bng_color(t_bng *x, t_symbol *s, int ac, t_atom *av)
{iemgui_color((void *)x, &x->x_gui, s, ac, av);}

static void bng_send(t_bng *x, t_symbol *s)
{iemgui_send(x, &x->x_gui, s);}

static void bng_receive(t_bng *x, t_symbol *s)
{iemgui_receive(x, &x->x_gui, s);}

static void bng_label(t_bng *x, t_symbol *s)
{iemgui_label((void *)x, &x->x_gui, s);}

static void bng_label_pos(t_bng *x, t_symbol *s, int ac, t_atom *av)
{iemgui_label_pos((void *)x, &x->x_gui, s, ac, av);}

static void bng_label_font(t_bng *x, t_symbol *s, int ac, t_atom *av)
{iemgui_label_font((void *)x, &x->x_gui, s, ac, av);}

static void bng_init(t_bng *x, t_floatarg f)
{
    x->x_gui.x_isa.x_loadinit = (f==0.0)?0:1;
}

static void bng_tick_hld(t_bng *x)
{
    x->x_flashed = 0;
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
}

static void bng_tick_brk(t_bng *x)
{
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
}

static void bng_tick_lck(t_bng *x)
{
    x->x_gui.x_isa.x_locked = 0;
}

static void *bng_new(t_symbol *s, int argc, t_atom *argv)
{
    t_bng *x = (t_bng *)pd_new(bng_class);
    int bflcol[]={-262144, -1, -1};
    t_symbol *srl[3];
    int a=IEM_GUI_DEFAULTSIZE;
    int ldx=20, ldy=8;
    int fs=8, iinit=0, ifstyle=0;
    int ftbreak=IEM_BNG_DEFAULTBREAKFLASHTIME, fthold=IEM_BNG_DEFAULTHOLDFLASHTIME;
    t_iem_init_symargs *init=(t_iem_init_symargs *)(&iinit);
    t_iem_fstyle_flags *fstyle=(t_iem_fstyle_flags *)(&ifstyle);
    char str[144];

    srl[0] = gensym("empty");
    srl[1] = gensym("empty");
    srl[2] = gensym("empty");

    if((argc == 14)&&IS_A_FLOAT(argv,0)
       &&IS_A_FLOAT(argv,1)&&IS_A_FLOAT(argv,2)
       &&IS_A_FLOAT(argv,3)
       &&(IS_A_SYMBOL(argv,4)||IS_A_FLOAT(argv,4))
       &&(IS_A_SYMBOL(argv,5)||IS_A_FLOAT(argv,5))
       &&(IS_A_SYMBOL(argv,6)||IS_A_FLOAT(argv,6))
       &&IS_A_FLOAT(argv,7)&&IS_A_FLOAT(argv,8)
       &&IS_A_FLOAT(argv,9)&&IS_A_FLOAT(argv,10)&&IS_A_FLOAT(argv,11)
       &&IS_A_FLOAT(argv,12)&&IS_A_FLOAT(argv,13))
    {

	a = (int)atom_getintarg(0, argc, argv);
	fthold = (int)atom_getintarg(1, argc, argv);
	ftbreak = (int)atom_getintarg(2, argc, argv);
	iinit = (int)(atom_getintarg(3, argc, argv));
	if(IS_A_SYMBOL(argv,4))
	    srl[0] = atom_getsymbolarg(4, argc, argv);
	else if(IS_A_FLOAT(argv,4))
	{
	    sprintf(str, "%d", (int)atom_getintarg(4, argc, argv));
	    srl[0] = gensym(str);
	}
	if(IS_A_SYMBOL(argv,5))
	    srl[1] = atom_getsymbolarg(5, argc, argv);
	else if(IS_A_FLOAT(argv,5))
	{
	    sprintf(str, "%d", (int)atom_getintarg(5, argc, argv));
	    srl[1] = gensym(str);
	}
	if(IS_A_SYMBOL(argv,6))
	    srl[2] = atom_getsymbolarg(6, argc, argv);
	else if(IS_A_FLOAT(argv,6))
	{
	    sprintf(str, "%d", (int)atom_getintarg(6, argc, argv));
	    srl[2] = gensym(str);
	}
	ldx = (int)atom_getintarg(7, argc, argv);
	ldy = (int)atom_getintarg(8, argc, argv);
	ifstyle = (int)(atom_getintarg(9, argc, argv));
	fs = (int)atom_getintarg(10, argc, argv);
	bflcol[0] = (int)atom_getintarg(11, argc, argv);
	bflcol[1] = (int)atom_getintarg(12, argc, argv);
	bflcol[2] = (int)atom_getintarg(13, argc, argv);
    }

    x->x_gui.x_draw = (t_iemfunptr)bng_draw;
    iinit &= IEM_INIT_ARGS_ALL;
    ifstyle &= IEM_FSTYLE_FLAGS_ALL;

    fstyle->x_snd_able = 1;
    fstyle->x_rcv_able = 1;
    x->x_flashed = 0;
    x->x_gui.x_glist = (t_glist *)canvas_getcurrent();
    x->x_gui.x_isa = *init;
    if(!strcmp(srl[0]->s_name, "empty")) fstyle->x_snd_able = 0;
    if(!strcmp(srl[1]->s_name, "empty")) fstyle->x_rcv_able = 0;
    x->x_gui.x_unique_num = 0;
    if(fstyle->x_font_style == 1) strcpy(x->x_gui.x_font, "helvetica");
    else if(fstyle->x_font_style == 2) strcpy(x->x_gui.x_font, "times");
    else { fstyle->x_font_style = 0;
	strcpy(x->x_gui.x_font, "courier"); }
    x->x_gui.x_fsf = *fstyle;
    iemgui_first_dollararg2sym(&x->x_gui, srl);

    if(x->x_gui.x_fsf.x_rcv_able) pd_bind(&x->x_gui.x_obj.ob_pd, srl[1]);
    x->x_gui.x_snd = srl[0];
    x->x_gui.x_rcv = srl[1];
    x->x_gui.x_lab = srl[2];
    x->x_gui.x_ldx = ldx;
    x->x_gui.x_ldy = ldy;

    if(fs < 4)
	fs = 4;
    x->x_gui.x_fontsize = fs;
    x->x_gui.x_w = iemgui_clip_size(a);
    x->x_gui.x_h = x->x_gui.x_w;
    bng_check_minmax(x, ftbreak, fthold);
    iemgui_all_colfromload(&x->x_gui, bflcol);
    x->x_gui.x_isa.x_locked = 0;
    iemgui_verify_snd_ne_rcv(&x->x_gui);
    x->x_clock_hld = clock_new(x, (t_method)bng_tick_hld);
    x->x_clock_brk = clock_new(x, (t_method)bng_tick_brk);
    x->x_clock_lck = clock_new(x, (t_method)bng_tick_lck);
    outlet_new(&x->x_gui.x_obj, &s_bang);
    return (x);
}

static void bng_ff(t_bng *x)
{
    if(x->x_gui.x_fsf.x_selected)
	pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
    if(x->x_gui.x_fsf.x_rcv_able)
	pd_unbind(&x->x_gui.x_obj.ob_pd, x->x_gui.x_rcv);
    clock_free(x->x_clock_lck);
    clock_free(x->x_clock_brk);
    clock_free(x->x_clock_hld);
}

void bng_setup(void)
{
    bng_class = class_new(gensym("bng"), (t_newmethod)bng_new,
			  (t_method)bng_ff, sizeof(t_bng), 0, A_GIMME, 0);
    class_addbang(bng_class, bng_bang);
    class_addfloat(bng_class, bng_float);
    class_addsymbol(bng_class, bng_symbol);
    class_addpointer(bng_class, bng_pointer);
    class_addlist(bng_class, bng_list);
    class_addanything(bng_class, bng_anything);
    class_addmethod(bng_class, (t_method)bng_click, gensym("click"),
		    A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
    class_addmethod(bng_class, (t_method)bng_dialog, gensym("dialog"),
		    A_GIMME, 0);
    class_addmethod(bng_class, (t_method)bng_loadbang, gensym("loadbang"), 0);
    class_addmethod(bng_class, (t_method)bng_size, gensym("size"), A_GIMME, 0);
    class_addmethod(bng_class, (t_method)bng_delta, gensym("delta"), A_GIMME, 0);
    class_addmethod(bng_class, (t_method)bng_pos, gensym("pos"), A_GIMME, 0);
    class_addmethod(bng_class, (t_method)bng_flashtime, gensym("flashtime"), A_GIMME, 0);
    class_addmethod(bng_class, (t_method)bng_color, gensym("color"), A_GIMME, 0);
    class_addmethod(bng_class, (t_method)bng_send, gensym("send"), A_DEFSYM, 0);
    class_addmethod(bng_class, (t_method)bng_receive, gensym("receive"), A_DEFSYM, 0);
    class_addmethod(bng_class, (t_method)bng_label, gensym("label"), A_DEFSYM, 0);
    class_addmethod(bng_class, (t_method)bng_label_pos, gensym("label_pos"), A_GIMME, 0);
    class_addmethod(bng_class, (t_method)bng_label_font, gensym("label_font"), A_GIMME, 0);
    class_addmethod(bng_class, (t_method)bng_init, gensym("init"), A_FLOAT, 0);
    if(!iemgui_key_sym)
	iemgui_key_sym = gensym("#keyname");
    bng_widgetbehavior.w_getrectfn = bng_getrect;
    bng_widgetbehavior.w_displacefn = iemgui_displace;
    bng_widgetbehavior.w_selectfn = iemgui_select;
    bng_widgetbehavior.w_activatefn = NULL;
    bng_widgetbehavior.w_deletefn = iemgui_delete;
    bng_widgetbehavior.w_visfn = iemgui_vis;
    bng_widgetbehavior.w_clickfn = bng_newclick;
    bng_widgetbehavior.w_propertiesfn = bng_properties;
    bng_widgetbehavior.w_savefn = bng_save;
    class_setwidget(bng_class, &bng_widgetbehavior);
    class_sethelpsymbol(bng_class, gensym("bng"));
}






/* ------------ hsl    gui-horicontal  slider ----------------------- */

t_widgetbehavior hslider_widgetbehavior;
static t_class *hslider_class;

/* widget helper functions */

static void hslider_draw_update(t_hslider *x, t_glist *glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    if (glist_isvisible(glist))
    {
	int r = x->x_gui.x_obj.te_xpos + (x->x_val + 50)/100;
	sys_vgui(".x%x.c coords %xKNOB %d %d %d %d\n",
		 canvas, x, r, x->x_gui.x_obj.te_ypos+1,
		 r, x->x_gui.x_obj.te_ypos + x->x_gui.x_h);
	if(x->x_val == x->x_center)
	{
	    if(!x->x_thick)
	    {
		sys_vgui(".x%x.c itemconfigure %xKNOB -width 7\n", canvas, x);
		x->x_thick = 1;
	    }
	}
	else
	{
	    if(x->x_thick)
	    {
		sys_vgui(".x%x.c itemconfigure %xKNOB -width 3\n", canvas, x);
		x->x_thick = 0;
	    }
	}
    }
}

static void hslider_draw_new(t_hslider *x, t_glist *glist)
{
    int r = x->x_gui.x_obj.te_xpos + (x->x_val + 50)/100;
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c create rectangle %d %d %d %d -fill #%6.6x -tags %xBASE\n",
	     canvas, x->x_gui.x_obj.te_xpos-3, x->x_gui.x_obj.te_ypos,
	     x->x_gui.x_obj.te_xpos + x->x_gui.x_w+2, x->x_gui.x_obj.te_ypos + x->x_gui.x_h,
	     x->x_gui.x_bcol, x);
    sys_vgui(".x%x.c create line %d %d %d %d -width 3 -fill #%6.6x -tags %xKNOB\n",
	     canvas, r, x->x_gui.x_obj.te_ypos+1, r,
	     x->x_gui.x_obj.te_ypos + x->x_gui.x_h, x->x_gui.x_fcol, x);
    sys_vgui(".x%x.c create text %d %d -text {%s} -anchor w \
	     -font {%s %d bold} -fill #%6.6x -tags %xLABEL\n",
	     canvas, x->x_gui.x_obj.te_xpos+x->x_gui.x_ldx,
	     x->x_gui.x_obj.te_ypos+x->x_gui.x_ldy,
	     strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"",
	     x->x_gui.x_font, x->x_gui.x_fontsize, x->x_gui.x_lcol, x);
    if(!x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xOUT%d\n",
	     canvas, x->x_gui.x_obj.te_xpos-3, x->x_gui.x_obj.te_ypos + x->x_gui.x_h-1,
	     x->x_gui.x_obj.te_xpos+4, x->x_gui.x_obj.te_ypos + x->x_gui.x_h, x, 0);
    if(!x->x_gui.x_fsf.x_rcv_able)
	sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xIN%d\n",
	     canvas, x->x_gui.x_obj.te_xpos-3, x->x_gui.x_obj.te_ypos,
	     x->x_gui.x_obj.te_xpos+4, x->x_gui.x_obj.te_ypos+1, x, 0);
}

static void hslider_draw_move(t_hslider *x, t_glist *glist)
{
    int r = x->x_gui.x_obj.te_xpos + (x->x_val + 50)/100;
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c coords %xBASE %d %d %d %d\n",
	     canvas, x,
	     x->x_gui.x_obj.te_xpos-3, x->x_gui.x_obj.te_ypos,
	     x->x_gui.x_obj.te_xpos + x->x_gui.x_w+2, x->x_gui.x_obj.te_ypos + x->x_gui.x_h);
    sys_vgui(".x%x.c coords %xKNOB %d %d %d %d\n",
	     canvas, x, r, x->x_gui.x_obj.te_ypos+1,
	     r, x->x_gui.x_obj.te_ypos + x->x_gui.x_h);
    sys_vgui(".x%x.c coords %xLABEL %d %d\n",
	     canvas, x, x->x_gui.x_obj.te_xpos+x->x_gui.x_ldx, x->x_gui.x_obj.te_ypos+x->x_gui.x_ldy);
    if(!x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c coords %xOUT%d %d %d %d %d\n",
	     canvas, x, 0,
	     x->x_gui.x_obj.te_xpos-3, x->x_gui.x_obj.te_ypos + x->x_gui.x_h-1,
	     x->x_gui.x_obj.te_xpos+4, x->x_gui.x_obj.te_ypos + x->x_gui.x_h);
    if(!x->x_gui.x_fsf.x_rcv_able)
	sys_vgui(".x%x.c coords %xIN%d %d %d %d %d\n",
	     canvas, x, 0,
	     x->x_gui.x_obj.te_xpos-3, x->x_gui.x_obj.te_ypos,
	     x->x_gui.x_obj.te_xpos+4, x->x_gui.x_obj.te_ypos+1);
}

static void hslider_draw_erase(t_hslider* x,t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c delete %xBASE\n", canvas, x);
    sys_vgui(".x%x.c delete %xKNOB\n", canvas, x);
    sys_vgui(".x%x.c delete %xLABEL\n", canvas, x);
    if(!x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c delete %xOUT%d\n", canvas, x, 0);
    if(!x->x_gui.x_fsf.x_rcv_able)
        sys_vgui(".x%x.c delete %xIN%d\n", canvas, x, 0);
}

static void hslider_draw_config(t_hslider* x,t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c itemconfigure %xLABEL -font {%s %d bold} -fill #%6.6x -text {%s} \n",
	     canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize,
	     x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
	     strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"");
    sys_vgui(".x%x.c itemconfigure %xKNOB -fill #%6.6x\n", canvas, x, x->x_gui.x_fcol);
    sys_vgui(".x%x.c itemconfigure %xBASE -fill #%6.6x\n", canvas, x, x->x_gui.x_bcol);
}

static void hslider_draw_io(t_hslider* x,t_glist* glist, int old_snd_rcv_flags)
{
    t_canvas *canvas=glist_getcanvas(glist);

    if((old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && !x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xOUT%d\n",
	     canvas, x->x_gui.x_obj.te_xpos-3, x->x_gui.x_obj.te_ypos + x->x_gui.x_h-1,
	     x->x_gui.x_obj.te_xpos+4, x->x_gui.x_obj.te_ypos + x->x_gui.x_h, x, 0);
    if(!(old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c delete %xOUT%d\n", canvas, x, 0);
    if((old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && !x->x_gui.x_fsf.x_rcv_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xIN%d\n",
	     canvas, x->x_gui.x_obj.te_xpos-3, x->x_gui.x_obj.te_ypos,
	     x->x_gui.x_obj.te_xpos+4, x->x_gui.x_obj.te_ypos+1, x, 0);
    if(!(old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && x->x_gui.x_fsf.x_rcv_able)
	sys_vgui(".x%x.c delete %xIN%d\n", canvas, x, 0);
}

static void hslider_draw_select(t_hslider* x,t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    if(x->x_gui.x_fsf.x_selected)
    {
	pd_bind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
	sys_vgui(".x%x.c itemconfigure %xBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
	sys_vgui(".x%x.c itemconfigure %xLABEL -fill #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
    }
    else
    {
	pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
	sys_vgui(".x%x.c itemconfigure %xBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
	sys_vgui(".x%x.c itemconfigure %xLABEL -fill #%6.6x\n", canvas, x, x->x_gui.x_lcol);
    }
}

void hslider_draw(t_hslider *x, t_glist *glist, int mode)
{
    if(mode == IEM_GUI_DRAW_MODE_UPDATE)
	hslider_draw_update(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_MOVE)
	hslider_draw_move(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_NEW)
	hslider_draw_new(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_SELECT)
	hslider_draw_select(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_ERASE)
	hslider_draw_erase(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_CONFIG)
	hslider_draw_config(x, glist);
    else if(mode >= IEM_GUI_DRAW_MODE_IO)
	hslider_draw_io(x, glist, mode - IEM_GUI_DRAW_MODE_IO);
}

/* ------------------------ hsl widgetbehaviour----------------------------- */


static void hslider_getrect(t_gobj *z, t_glist *owner,
			    int *xp1, int *yp1, int *xp2, int *yp2)
{
    t_hslider* x = (t_hslider*)z;

    *xp1 = x->x_gui.x_obj.te_xpos - 3;
    *yp1 = x->x_gui.x_obj.te_ypos;
    *xp2 = x->x_gui.x_obj.te_xpos + x->x_gui.x_w + 2;
    *yp2 = x->x_gui.x_obj.te_ypos + x->x_gui.x_h;
}

static void hslider_save(t_gobj *z, t_binbuf *b)
{
    t_hslider *x = (t_hslider *)z;
    int bflcol[3], *ip1, *ip2;
    t_symbol *srl[3];

    iemgui_save(&x->x_gui, srl, bflcol);
    ip1 = (int *)(&x->x_gui.x_isa);
    ip2 = (int *)(&x->x_gui.x_fsf);
    binbuf_addv(b, "ssiisiiffiisssiiiiiiiii", gensym("#X"),gensym("obj"),
		(t_int)x->x_gui.x_obj.te_xpos, (t_int)x->x_gui.x_obj.te_ypos,
		gensym("hsl"), x->x_gui.x_w, x->x_gui.x_h,
		(float)x->x_min, (float)x->x_max,
		x->x_lin0_log1, (*ip1)&IEM_INIT_ARGS_ALL,
		srl[0], srl[1], srl[2],
		x->x_gui.x_ldx, x->x_gui.x_ldy,
		(*ip2)&IEM_FSTYLE_FLAGS_ALL, x->x_gui.x_fontsize,
		bflcol[0], bflcol[1], bflcol[2],
		x->x_val, x->x_steady);
    binbuf_addv(b, ";");
}

void hslider_check_width(t_hslider *x, int w)
{
    if(w < IEM_SL_MINSIZE)
	w = IEM_SL_MINSIZE;
    x->x_gui.x_w = w;
    x->x_center = (x->x_gui.x_w-1)*50;
    if(x->x_val > (x->x_gui.x_w*100 - 100))
    {
	x->x_pos = x->x_gui.x_w*100 - 100;
	x->x_val = x->x_pos;
    }
    if(x->x_lin0_log1)
	x->x_k = log(x->x_max/x->x_min)/(double)(x->x_gui.x_w - 1);
    else
	x->x_k = (x->x_max - x->x_min)/(double)(x->x_gui.x_w - 1);
}

void hslider_check_minmax(t_hslider *x, double min, double max)
{
    if(x->x_lin0_log1)
    {
	if((min == 0.0)&&(max == 0.0))
	    max = 1.0;
	if(max > 0.0)
	{
	    if(min <= 0.0)
		min = 0.01*max;
	}
	else
	{
	    if(min > 0.0)
		max = 0.01*min;
	}
    }
    x->x_min = min;
    x->x_max = max;
    if(x->x_lin0_log1)
	x->x_k = log(x->x_max/x->x_min)/(double)(x->x_gui.x_w - 1);
    else
	x->x_k = (x->x_max - x->x_min)/(double)(x->x_gui.x_w - 1);
}

static void hslider_properties(t_gobj *z, t_glist *owner)
{
    t_hslider *x = (t_hslider *)z;
    char buf[800];
    int steady=-23;
    t_symbol *srl[3];

    iemgui_properties(&x->x_gui, srl);
    if(x->x_steady)
	steady = -22;
    sprintf(buf, "pdtk_iemgui_dialog %%s HSLIDER \
	    --------dimensions(pix)(pix):-------- %d %d width: %d %d height: \
	    -----------output-range:----------- %g left: %g right: %g \
	    %d lin log %d %d \
	    %s %s \
	    %s %d %d \
	    %d %d \
	    %d %d %d\n",
	    x->x_gui.x_w, IEM_SL_MINSIZE, x->x_gui.x_h, IEM_GUI_MINSIZE,
	    x->x_min, x->x_max, 0.0,/*no_schedule*/
	    x->x_lin0_log1, x->x_gui.x_isa.x_loadinit, steady,/*no multi, but iem-characteristic*/
	    srl[0]->s_name, srl[1]->s_name,
	    srl[2]->s_name, x->x_gui.x_ldx, x->x_gui.x_ldy,
	    x->x_gui.x_fsf.x_font_style, x->x_gui.x_fontsize,
	    0xffffff & x->x_gui.x_bcol, 0xffffff & x->x_gui.x_fcol, 0xffffff & x->x_gui.x_lcol);
    gfxstub_new(&x->x_gui.x_obj.ob_pd, x, buf);
}

static void hslider_set(t_hslider *x, t_floatarg f)
{
    double g;

    if(f > x->x_max)
	f = x->x_max;
    if(f < x->x_min)
	f = x->x_min;
    if(x->x_lin0_log1)
	g = log(f/x->x_min)/x->x_k;
    else
	g = (f - x->x_min) / x->x_k;
    x->x_val = (int)(100.0*g + 0.49999);
    x->x_pos = x->x_val;
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
}

static void hslider_bang(t_hslider *x)
{
    double out;

    if(x->x_lin0_log1)
	out = x->x_min*exp(x->x_k*(double)(x->x_val)*0.01);
    else
	out = (double)(x->x_val)*0.01*x->x_k + x->x_min;
    if((out < 1.0e-10)&&(out > -1.0e-10))
	out = 0.0;
    outlet_float(x->x_gui.x_obj.ob_outlet, out);
    if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
        pd_float(x->x_gui.x_snd->s_thing, out);
}

static void hslider_dialog(t_hslider *x, t_symbol *s, int argc, t_atom *argv)
{
    t_symbol *srl[3];
    int w = (int)atom_getintarg(0, argc, argv);
    int h = (int)atom_getintarg(1, argc, argv);
    double min = (double)atom_getfloatarg(2, argc, argv);
    double max = (double)atom_getfloatarg(3, argc, argv);
    int lilo = (int)atom_getintarg(4, argc, argv);
    int steady = (int)atom_getintarg(6, argc, argv);
    int sr_flags;

    if(lilo != 0) lilo = 1;
    x->x_lin0_log1 = lilo;
    if(steady == 3)
	x->x_steady = 0;
    else
	x->x_steady = 1;
    sr_flags = iemgui_dialog(&x->x_gui, srl, argc, argv);
    x->x_gui.x_h = iemgui_clip_size(h);
    hslider_check_width(x, w);
    hslider_check_minmax(x, min, max);
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_CONFIG);
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_IO + sr_flags);
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
    canvas_fixlinesfor(glist_getcanvas(x->x_gui.x_glist), (t_text*)x);
}

static void hslider_motion(t_hslider *x, t_floatarg dx, t_floatarg dy)
{
    int old = x->x_val;

    if(x->x_gui.x_fsf.x_finemoved)
	x->x_pos += (int)dx;
    else
	x->x_pos += 100*(int)dx;
    x->x_val = x->x_pos;
    if(x->x_val > (100*x->x_gui.x_w - 100))
    {
	x->x_val = 100*x->x_gui.x_w - 100;
	x->x_pos += 50;
	x->x_pos -= x->x_pos%100;
    }
    if(x->x_val < 0)
    {
	x->x_val = 0;
	x->x_pos -= 50;
	x->x_pos -= x->x_pos%100;
    }
    if(old != x->x_val)
    {
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
	hslider_bang(x);
    }
}

static void hslider_click(t_hslider *x, t_floatarg xpos, t_floatarg ypos,
			  t_floatarg shift, t_floatarg ctrl, t_floatarg alt)
{
    if(!x->x_steady)
	x->x_val = (int)(100.0 * (xpos - x->x_gui.x_obj.te_xpos));
    if(x->x_val > (100*x->x_gui.x_w - 100))
	x->x_val = 100*x->x_gui.x_w - 100;
    if(x->x_val < 0)
	x->x_val = 0;
    x->x_pos = x->x_val;
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
    hslider_bang(x);
    glist_grab(x->x_gui.x_glist, &x->x_gui.x_obj.te_g, (t_glistmotionfn)hslider_motion,
	       0, xpos, ypos);
}

static int hslider_newclick(t_gobj *z, struct _glist *glist,
			    int xpix, int ypix, int shift, int alt, int dbl, int doit)
{
    t_hslider* x = (t_hslider *)z;

    if(doit)
    {
	hslider_click( x, (t_floatarg)xpix, (t_floatarg)ypix, (t_floatarg)shift,
		       0, (t_floatarg)alt);
	if(shift)
	    x->x_gui.x_fsf.x_finemoved = 1;
	else
	    x->x_gui.x_fsf.x_finemoved = 0;
    }
    return (1);
}

static void hslider_size(t_hslider *x, t_symbol *s, int ac, t_atom *av)
{
    hslider_check_width(x, (int)atom_getintarg(0, ac, av));
    if(ac > 1)
	x->x_gui.x_h = iemgui_clip_size((int)atom_getintarg(1, ac, av));
    iemgui_size((void *)x, &x->x_gui);
}

static void hslider_delta(t_hslider *x, t_symbol *s, int ac, t_atom *av)
{iemgui_delta((void *)x, &x->x_gui, s, ac, av);}

static void hslider_pos(t_hslider *x, t_symbol *s, int ac, t_atom *av)
{iemgui_pos((void *)x, &x->x_gui, s, ac, av);}

static void hslider_range(t_hslider *x, t_symbol *s, int ac, t_atom *av)
{
    hslider_check_minmax(x, (double)atom_getfloatarg(0, ac, av),
			 (double)atom_getfloatarg(1, ac, av));
}

static void hslider_color(t_hslider *x, t_symbol *s, int ac, t_atom *av)
{iemgui_color((void *)x, &x->x_gui, s, ac, av);}

static void hslider_send(t_hslider *x, t_symbol *s)
{iemgui_send(x, &x->x_gui, s);}

static void hslider_receive(t_hslider *x, t_symbol *s)
{iemgui_receive(x, &x->x_gui, s);}

static void hslider_label(t_hslider *x, t_symbol *s)
{iemgui_label((void *)x, &x->x_gui, s);}

static void hslider_label_pos(t_hslider *x, t_symbol *s, int ac, t_atom *av)
{iemgui_label_pos((void *)x, &x->x_gui, s, ac, av);}

static void hslider_label_font(t_hslider *x, t_symbol *s, int ac, t_atom *av)
{iemgui_label_font((void *)x, &x->x_gui, s, ac, av);}

static void hslider_log(t_hslider *x)
{
    x->x_lin0_log1 = 1;
    hslider_check_minmax(x, x->x_min, x->x_max);
}

static void hslider_lin(t_hslider *x)
{
    x->x_lin0_log1 = 0;
    x->x_k = (x->x_max - x->x_min)/(double)(x->x_gui.x_w - 1);
}

static void hslider_init(t_hslider *x, t_floatarg f)
{
    x->x_gui.x_isa.x_loadinit = (f==0.0)?0:1;
}

static void hslider_steady(t_hslider *x, t_floatarg f)
{
    x->x_steady = (f==0.0)?0:1;
}

static void hslider_float(t_hslider *x, t_floatarg f)
{
    double out;

    hslider_set(x, f);
    if(x->x_lin0_log1)
	out = x->x_min*exp(x->x_k*(double)(x->x_val)*0.01);
    else
	out = (double)(x->x_val)*0.01*x->x_k + x->x_min;
    if((out < 1.0e-10)&&(out > -1.0e-10))
	out = 0.0;
    if(x->x_gui.x_fsf.x_put_in2out)
    {
	outlet_float(x->x_gui.x_obj.ob_outlet, out);
	if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
	    pd_float(x->x_gui.x_snd->s_thing, out);
    }
}

static void hslider_loadbang(t_hslider *x)
{
    if(!sys_noloadbang && x->x_gui.x_isa.x_loadinit)
    {
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
	hslider_bang(x);
    }
}

static void hslider_list(t_hslider *x, t_symbol *s, int ac, t_atom *av)
{
    int l=iemgui_list((void *)x, &x->x_gui, s, ac, av);

    if(l < 0)
    {
	if(IS_A_FLOAT(av,0))
	    hslider_float(x, atom_getfloatarg(0, ac, av));
    }
    else if(l > 0)
    {
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
	canvas_fixlinesfor(glist_getcanvas(x->x_gui.x_glist), (t_text*)x);
    }
}

static void *hslider_new(t_symbol *s, int argc, t_atom *argv)
{
    t_hslider *x = (t_hslider *)pd_new(hslider_class);
    int bflcol[]={-262144, -1, -1};
    t_symbol *srl[3];
    int w=IEM_SL_DEFAULTSIZE, h=IEM_GUI_DEFAULTSIZE;
    int lilo=0, ldx=20, ldy=8, f=0, v=0, steady=1;
    int fs=8, iinit=0, ifstyle=0;
    double min=0.0, max=(double)(IEM_SL_DEFAULTSIZE-1);
    t_iem_init_symargs *init=(t_iem_init_symargs *)(&iinit);
    t_iem_fstyle_flags *fstyle=(t_iem_fstyle_flags *)(&ifstyle);
    char str[144];

    srl[0] = gensym("empty");
    srl[1] = gensym("empty");
    srl[2] = gensym("empty");

    if(((argc == 17)||(argc == 18))&&IS_A_FLOAT(argv,0)&&IS_A_FLOAT(argv,1)
       &&IS_A_FLOAT(argv,2)&&IS_A_FLOAT(argv,3)
       &&IS_A_FLOAT(argv,4)&&IS_A_FLOAT(argv,5)
       &&(IS_A_SYMBOL(argv,6)||IS_A_FLOAT(argv,6))
       &&(IS_A_SYMBOL(argv,7)||IS_A_FLOAT(argv,7))
       &&(IS_A_SYMBOL(argv,8)||IS_A_FLOAT(argv,8))
       &&IS_A_FLOAT(argv,9)&&IS_A_FLOAT(argv,10)
       &&IS_A_FLOAT(argv,11)&&IS_A_FLOAT(argv,12)&&IS_A_FLOAT(argv,13)
       &&IS_A_FLOAT(argv,14)&&IS_A_FLOAT(argv,15)&&IS_A_FLOAT(argv,16))
    {
	w = (int)atom_getintarg(0, argc, argv);
	h = (int)atom_getintarg(1, argc, argv);
	min = (double)atom_getfloatarg(2, argc, argv);
	max = (double)atom_getfloatarg(3, argc, argv);
	lilo = (int)atom_getintarg(4, argc, argv);
	iinit = (int)atom_getintarg(5, argc, argv);
	if(IS_A_SYMBOL(argv,6))
	    srl[0] = atom_getsymbolarg(6, argc, argv);
	else if(IS_A_FLOAT(argv,6))
	{
	    sprintf(str, "%d", (int)atom_getintarg(6, argc, argv));
	    srl[0] = gensym(str);
	}
	if(IS_A_SYMBOL(argv,7))
	    srl[1] = atom_getsymbolarg(7, argc, argv);
	else if(IS_A_FLOAT(argv,7))
	{
	    sprintf(str, "%d", (int)atom_getintarg(7, argc, argv));
	    srl[1] = gensym(str);
	}
	if(IS_A_SYMBOL(argv,8))
	    srl[2] = atom_getsymbolarg(8, argc, argv);
	else if(IS_A_FLOAT(argv,8))
	{
	    sprintf(str, "%d", (int)atom_getintarg(8, argc, argv));
	    srl[2] = gensym(str);
	}
	ldx = (int)atom_getintarg(9, argc, argv);
	ldy = (int)atom_getintarg(10, argc, argv);
	ifstyle = (int)atom_getintarg(11, argc, argv);
	fs = (int)atom_getintarg(12, argc, argv);
	bflcol[0] = (int)atom_getintarg(13, argc, argv);
	bflcol[1] = (int)atom_getintarg(14, argc, argv);
	bflcol[2] = (int)atom_getintarg(15, argc, argv);
	v = (int)atom_getintarg(16, argc, argv);
    }
    if((argc == 18)&&IS_A_FLOAT(argv,17))
	steady = (int)atom_getintarg(17, argc, argv);

    x->x_gui.x_draw = (t_iemfunptr)hslider_draw;
    iinit &= IEM_INIT_ARGS_ALL;
    ifstyle &= IEM_FSTYLE_FLAGS_ALL;

    fstyle->x_snd_able = 1;
    fstyle->x_rcv_able = 1;

    x->x_gui.x_glist = (t_glist *)canvas_getcurrent();
    x->x_gui.x_isa = *init;
    if(x->x_gui.x_isa.x_loadinit)
	x->x_val = v;
    else
	x->x_val = 0;
    x->x_pos = x->x_val;
    if(lilo != 0) lilo = 1;
    x->x_lin0_log1 = lilo;
    if(steady != 0) steady = 1;
    x->x_steady = steady;
    if(!strcmp(srl[0]->s_name, "empty")) fstyle->x_snd_able = 0;
    if(!strcmp(srl[1]->s_name, "empty")) fstyle->x_rcv_able = 0;
    x->x_gui.x_unique_num = 0;
    if(fstyle->x_font_style == 1) strcpy(x->x_gui.x_font, "helvetica");
    else if(fstyle->x_font_style == 2) strcpy(x->x_gui.x_font, "times");
    else { fstyle->x_font_style = 0;
	strcpy(x->x_gui.x_font, "courier"); }
    x->x_gui.x_fsf = *fstyle;
    iemgui_first_dollararg2sym(&x->x_gui, srl);
    if(x->x_gui.x_fsf.x_rcv_able) pd_bind(&x->x_gui.x_obj.ob_pd, srl[1]);
    x->x_gui.x_snd = srl[0];
    x->x_gui.x_rcv = srl[1];
    x->x_gui.x_lab = srl[2];
    x->x_gui.x_ldx = ldx;
    x->x_gui.x_ldy = ldy;
    if(fs < 4)
	fs = 4;
    x->x_gui.x_fontsize = fs;
    x->x_gui.x_h = iemgui_clip_size(h);
    hslider_check_width(x, w);
    hslider_check_minmax(x, min, max);
    iemgui_all_colfromload(&x->x_gui, bflcol);
    x->x_thick = 0;
    iemgui_verify_snd_ne_rcv(&x->x_gui);
    outlet_new(&x->x_gui.x_obj, &s_float);
    return (x);
}

static void hslider_free(t_hslider *x)
{
    if(x->x_gui.x_fsf.x_selected)
	pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
    if(x->x_gui.x_fsf.x_rcv_able)
	pd_unbind(&x->x_gui.x_obj.ob_pd, x->x_gui.x_rcv);
}

void hslider_setup(void)
{
    hslider_class = class_new(gensym("hsl"), (t_newmethod)hslider_new,
			      (t_method)hslider_free, sizeof(t_hslider), 0, A_GIMME, 0);
#ifndef GGEE_HSLIDER_COMPATIBLE
    class_addcreator((t_newmethod)hslider_new, gensym("hslider"), A_GIMME, 0);
#endif
    class_addbang(hslider_class,hslider_bang);
    class_addfloat(hslider_class,hslider_float);
    class_addlist(hslider_class, hslider_list);
    class_addmethod(hslider_class, (t_method)hslider_click, gensym("click"),
		    A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
    class_addmethod(hslider_class, (t_method)hslider_motion, gensym("motion"),
		    A_FLOAT, A_FLOAT, 0);
    class_addmethod(hslider_class, (t_method)hslider_dialog, gensym("dialog"), A_GIMME, 0);
    class_addmethod(hslider_class, (t_method)hslider_loadbang, gensym("loadbang"), 0);
    class_addmethod(hslider_class, (t_method)hslider_set, gensym("set"), A_FLOAT, 0);
    class_addmethod(hslider_class, (t_method)hslider_size, gensym("size"), A_GIMME, 0);
    class_addmethod(hslider_class, (t_method)hslider_delta, gensym("delta"), A_GIMME, 0);
    class_addmethod(hslider_class, (t_method)hslider_pos, gensym("pos"), A_GIMME, 0);
    class_addmethod(hslider_class, (t_method)hslider_range, gensym("range"), A_GIMME, 0);
    class_addmethod(hslider_class, (t_method)hslider_color, gensym("color"), A_GIMME, 0);
    class_addmethod(hslider_class, (t_method)hslider_send, gensym("send"), A_DEFSYM, 0);
    class_addmethod(hslider_class, (t_method)hslider_receive, gensym("receive"), A_DEFSYM, 0);
    class_addmethod(hslider_class, (t_method)hslider_label, gensym("label"), A_DEFSYM, 0);
    class_addmethod(hslider_class, (t_method)hslider_label_pos, gensym("label_pos"), A_GIMME, 0);
    class_addmethod(hslider_class, (t_method)hslider_label_font, gensym("label_font"), A_GIMME, 0);
    class_addmethod(hslider_class, (t_method)hslider_log, gensym("log"), 0);
    class_addmethod(hslider_class, (t_method)hslider_lin, gensym("lin"), 0);
    class_addmethod(hslider_class, (t_method)hslider_init, gensym("init"), A_FLOAT, 0);
    class_addmethod(hslider_class, (t_method)hslider_steady, gensym("steady"), A_FLOAT, 0);
    if(!iemgui_key_sym)
	iemgui_key_sym = gensym("#keyname");
    hslider_widgetbehavior.w_getrectfn =    hslider_getrect;
    hslider_widgetbehavior.w_displacefn =   iemgui_displace;
    hslider_widgetbehavior.w_selectfn =     iemgui_select;
    hslider_widgetbehavior.w_activatefn =   NULL;
    hslider_widgetbehavior.w_deletefn =     iemgui_delete;
    hslider_widgetbehavior.w_visfn =        iemgui_vis;
    hslider_widgetbehavior.w_clickfn =      hslider_newclick;
    hslider_widgetbehavior.w_propertiesfn = hslider_properties;
    hslider_widgetbehavior.w_savefn =       hslider_save;
    class_setwidget(hslider_class, &hslider_widgetbehavior);
    class_sethelpsymbol(hslider_class, gensym("hslider"));
}









/* ------------- hdl     gui-horicontal dial ---------------------- */

t_widgetbehavior hdial_widgetbehavior;
static t_class *hdial_class;

/* widget helper functions */

void hdial_draw_update(t_hdial *x, t_glist *glist)
{
    if(glist_isvisible(glist))
    {
	t_canvas *canvas=glist_getcanvas(glist);

	sys_vgui(".x%x.c itemconfigure %xBUT%d -fill #%6.6x -outline #%6.6x\n",
		 canvas, x, x->x_on_old,
		 x->x_gui.x_bcol, x->x_gui.x_bcol);
	sys_vgui(".x%x.c itemconfigure %xBUT%d -fill #%6.6x -outline #%6.6x\n",
		 canvas, x, x->x_on,
		 x->x_gui.x_fcol, x->x_gui.x_fcol);
    }
}

void hdial_draw_new(t_hdial *x, t_glist *glist)
{
    t_canvas *canvas=glist_getcanvas(glist);
    int n=x->x_number, i, dx=x->x_gui.x_w, s4=dx/4;
    int yy11=x->x_gui.x_obj.te_ypos, yy12=yy11+dx;
    int yy21=yy11+s4, yy22=yy12-s4;
    int xx11b=x->x_gui.x_obj.te_xpos, xx11=x->x_gui.x_obj.te_xpos, xx21=xx11b+s4;
    int xx22=xx11b+dx-s4;


    for(i=0; i<n; i++)
    {
	sys_vgui(".x%x.c create rectangle %d %d %d %d -fill #%6.6x -tags %xBASE%d\n",
		 canvas, xx11, yy11, xx11+dx, yy12,
		 x->x_gui.x_bcol, x, i);
	sys_vgui(".x%x.c create rectangle %d %d %d %d -fill #%6.6x -outline #%6.6x -tags %xBUT%d\n",
		 canvas, xx21, yy21, xx22, yy22,
		 (x->x_on==i)?x->x_gui.x_fcol:x->x_gui.x_bcol,
		 (x->x_on==i)?x->x_gui.x_fcol:x->x_gui.x_bcol, x, i);
	xx11 += dx;
	xx21 += dx;
	xx22 += dx;
    }
    sys_vgui(".x%x.c create text %d %d -text {%s} -anchor w \
	     -font {%s %d bold} -fill #%6.6x -tags %xLABEL\n",
	     canvas, xx11b+x->x_gui.x_ldx, yy11+x->x_gui.x_ldy,
	     strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"",
	     x->x_gui.x_font, x->x_gui.x_fontsize,
	     x->x_gui.x_lcol, x);
    if(!x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xOUT%d\n",
	     canvas, xx11b, yy12-1, xx11b + IOWIDTH, yy12, x, 0);
    if(!x->x_gui.x_fsf.x_rcv_able)
	sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xIN%d\n",
	     canvas, xx11b, yy11, xx11b + IOWIDTH, yy11+1, x, 0);

}

void hdial_draw_move(t_hdial *x, t_glist *glist)
{
    t_canvas *canvas=glist_getcanvas(glist);
    int n=x->x_number, i, dx=x->x_gui.x_w, s4=dx/4;
    int yy11=x->x_gui.x_obj.te_ypos, yy12=yy11+dx;
    int yy21=yy11+s4, yy22=yy12-s4;
    int xx11b=x->x_gui.x_obj.te_xpos, xx11=x->x_gui.x_obj.te_xpos, xx21=xx11b+s4;
    int xx22=xx11b+dx-s4;

    xx11 = xx11b;
    xx21=xx11b+s4;
    xx22=xx11b+dx-s4;
    for(i=0; i<n; i++)
    {
	sys_vgui(".x%x.c coords %xBASE%d %d %d %d %d\n",
		 canvas, x, i, xx11, yy11, xx11+dx, yy12);
	sys_vgui(".x%x.c coords %xBUT%d %d %d %d %d\n",
		 canvas, x, i, xx21, yy21, xx22, yy22);
	xx11 += dx;
	xx21 += dx;
	xx22 += dx;
    }
    sys_vgui(".x%x.c coords %xLABEL %d %d\n",
	     canvas, x, xx11b+x->x_gui.x_ldx, yy11+x->x_gui.x_ldy);
    if(!x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c coords %xOUT%d %d %d %d %d\n",
	     canvas, x, 0, xx11b, yy12-1, xx11b + IOWIDTH, yy12);
    if(!x->x_gui.x_fsf.x_rcv_able)
	sys_vgui(".x%x.c coords %xIN%d %d %d %d %d\n",
	     canvas, x, 0, xx11b, yy11, xx11b + IOWIDTH, yy11+1);
}

void hdial_draw_erase(t_hdial* x, t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);
    int n=x->x_number, i;

    for(i=0; i<n; i++)
    {
	sys_vgui(".x%x.c delete %xBASE%d\n", canvas, x, i);
	sys_vgui(".x%x.c delete %xBUT%d\n", canvas, x, i);
    }
    sys_vgui(".x%x.c delete %xLABEL\n", canvas, x);
    if(!x->x_gui.x_fsf.x_snd_able)
	sys_vgui(".x%x.c delete %xOUT%d\n", canvas, x, 0);
    if(!x->x_gui.x_fsf.x_rcv_able)
    sys_vgui(".x%x.c delete %xIN%d\n", canvas, x, 0);
}

void hdial_draw_config(t_hdial* x, t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);
    int n=x->x_number, i;

    sys_vgui(".x%x.c itemconfigure %xLABEL -font {%s %d bold} -fill #%6.6x -text {%s} \n",
	     canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize,
	     x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
	     strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"");
    for(i=0; i<n; i++)
    {
	sys_vgui(".x%x.c itemconfigure %xBASE%d -fill #%6.6x\n", canvas, x, i,
		 x->x_gui.x_bcol);
	sys_vgui(".x%x.c itemconfigure %xBUT%d -fill #%6.6x -outline #%6.6x\n", canvas, x, i,
		 (x->x_on==i)?x->x_gui.x_fcol:x->x_gui.x_bcol,
		 (x->x_on==i)?x->x_gui.x_fcol:x->x_gui.x_bcol);
    }
}

void hdial_draw_io(t_hdial* x, t_glist* glist, int old_snd_rcv_flags)
{
    t_canvas *canvas=glist_getcanvas(glist);
    
    if((old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && !x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xOUT%d\n",
		 canvas,
		 x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos + x->x_gui.x_w-1,
		 x->x_gui.x_obj.te_xpos + IOWIDTH, x->x_gui.x_obj.te_ypos + x->x_gui.x_w,
		 x, 0);
    if(!(old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c delete %xOUT%d\n", canvas, x, 0);
    if((old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && !x->x_gui.x_fsf.x_rcv_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xIN%d\n",
		 canvas,
		 x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos,
		 x->x_gui.x_obj.te_xpos + IOWIDTH, x->x_gui.x_obj.te_ypos+1, x, 0);
    if(!(old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && x->x_gui.x_fsf.x_rcv_able)
	sys_vgui(".x%x.c delete %xIN%d\n", canvas, x, 0);
}

void hdial_draw_select(t_hdial* x, t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);
    int n=x->x_number, i;

    if(x->x_gui.x_fsf.x_selected)
    {
	pd_bind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
	for(i=0; i<n; i++)
	{
	    sys_vgui(".x%x.c itemconfigure %xBASE%d -outline #%6.6x\n", canvas, x, i,
		     IEM_GUI_COLOR_SELECTED);
	}
	sys_vgui(".x%x.c itemconfigure %xLABEL -fill #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
    }
    else
    {
	pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
	for(i=0; i<n; i++)
	{
	    sys_vgui(".x%x.c itemconfigure %xBASE%d -outline #%6.6x\n", canvas, x, i,
		     IEM_GUI_COLOR_NORMAL);
	}
	sys_vgui(".x%x.c itemconfigure %xLABEL -fill #%6.6x\n", canvas, x,
		 x->x_gui.x_lcol);
    }
}

void hdial_draw(t_hdial *x, t_glist *glist, int mode)
{
    if(mode == IEM_GUI_DRAW_MODE_UPDATE)
	hdial_draw_update(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_MOVE)
	hdial_draw_move(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_NEW)
	hdial_draw_new(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_SELECT)
	hdial_draw_select(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_ERASE)
	hdial_draw_erase(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_CONFIG)
	hdial_draw_config(x, glist);
    else if(mode >= IEM_GUI_DRAW_MODE_IO)
	hdial_draw_io(x, glist, mode - IEM_GUI_DRAW_MODE_IO);
}

/* ------------------------ hdl widgetbehaviour----------------------------- */

static void hdial_getrect(t_gobj *z, t_glist *owner, int *xp1, int *yp1, int *xp2, int *yp2)
{
    t_hdial *x = (t_hdial *)z;

    *xp1 = x->x_gui.x_obj.te_xpos;
    *yp1 = x->x_gui.x_obj.te_ypos;
    *xp2 = x->x_gui.x_obj.te_xpos + x->x_gui.x_w*x->x_number;
    *yp2 = x->x_gui.x_obj.te_ypos + x->x_gui.x_h;
}

static void hdial_save(t_gobj *z, t_binbuf *b)
{
    t_hdial *x = (t_hdial *)z;
    int bflcol[3], *ip1, *ip2;
    t_symbol *srl[3];

    iemgui_save(&x->x_gui, srl, bflcol);
    ip1 = (int *)(&x->x_gui.x_isa);
    ip2 = (int *)(&x->x_gui.x_fsf);
    binbuf_addv(b, "ssiisiiiisssiiiiiiii", gensym("#X"),gensym("obj"),
		(t_int)x->x_gui.x_obj.te_xpos, (t_int)x->x_gui.x_obj.te_ypos,
		gensym("hdl"), x->x_gui.x_w,
		x->x_change, (*ip1)&IEM_INIT_ARGS_ALL, x->x_number,
		srl[0], srl[1], srl[2],
		x->x_gui.x_ldx, x->x_gui.x_ldy,
		(*ip2)&IEM_FSTYLE_FLAGS_ALL, x->x_gui.x_fontsize,
		bflcol[0], bflcol[1], bflcol[2], x->x_on);
    binbuf_addv(b, ";");
}

static void hdial_properties(t_gobj *z, t_glist *owner)
{
    t_hdial *x = (t_hdial *)z;
    char buf[800];
    t_symbol *srl[3];

    iemgui_properties(&x->x_gui, srl);
    sprintf(buf, "pdtk_iemgui_dialog %%s RADIOBUTTON \
	    ----------dimensions(pix):----------- %d %d size: 0 0 empty \
	    empty 0.0 empty 0.0 empty %d \
	    %d new-only new&old %d %d \
	    %s %s \
	    %s %d %d \
	    %d %d \
	    %d %d %d\n",
	    x->x_gui.x_w, IEM_GUI_MINSIZE,
	    0,/*no_schedule*/
	    x->x_change, x->x_gui.x_isa.x_loadinit, x->x_number,
	    srl[0]->s_name, srl[1]->s_name,
	    srl[2]->s_name, x->x_gui.x_ldx, x->x_gui.x_ldy,
	    x->x_gui.x_fsf.x_font_style, x->x_gui.x_fontsize,
	    0xffffff & x->x_gui.x_bcol, 0xffffff & x->x_gui.x_fcol, 0xffffff & x->x_gui.x_lcol);
    gfxstub_new(&x->x_gui.x_obj.ob_pd, x, buf);
}

static void hdial_dialog(t_hdial *x, t_symbol *s, int argc, t_atom *argv)
{
    t_symbol *srl[3];
    int a = (int)atom_getintarg(0, argc, argv);
    int chg = (int)atom_getintarg(4, argc, argv);
    int num = (int)atom_getintarg(6, argc, argv);
    int sr_flags;

    if(chg != 0) chg = 1;
    x->x_change = chg;
    sr_flags = iemgui_dialog(&x->x_gui, srl, argc, argv);
    x->x_gui.x_w = iemgui_clip_size(a);
    x->x_gui.x_h = x->x_gui.x_w;
    if(x->x_number != num)
    {
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_ERASE);
	x->x_number = num;
	if(x->x_on >= x->x_number)
	{
	    x->x_on = x->x_number - 1;
	    x->x_on_old = x->x_on;
	}
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_NEW);
    }
    else
    {
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_CONFIG);
        (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_IO + sr_flags);
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
	canvas_fixlinesfor(glist_getcanvas(x->x_gui.x_glist), (t_text*)x);
    }

}

static void hdial_set(t_hdial *x, t_floatarg f)
{
    int i=(int)f;
    int old=x->x_on_old;

    if(i < 0)
	i = 0;
    if(i >= x->x_number)
	i = x->x_number-1;
    if(x->x_on != x->x_on_old)
    {
	old = x->x_on_old;
	x->x_on_old = x->x_on;
	x->x_on = i;
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
	x->x_on_old = old;
    }
    else
    {
	x->x_on = i;
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
    }
}

static void hdial_bang(t_hdial *x)
{
    if((x->x_change)&&(x->x_on != x->x_on_old))
    {
        SETFLOAT(x->x_at, (float)x->x_on_old);
	SETFLOAT(x->x_at+1, 0.0);
	outlet_list(x->x_gui.x_obj.ob_outlet, &s_list, 2, x->x_at);
	if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
	    pd_list(x->x_gui.x_snd->s_thing, &s_list, 2, x->x_at);
    }
    x->x_on_old = x->x_on;
    SETFLOAT(x->x_at, (float)x->x_on);
    SETFLOAT(x->x_at+1, 1.0);
    outlet_list(x->x_gui.x_obj.ob_outlet, &s_list, 2, x->x_at);
    if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
	pd_list(x->x_gui.x_snd->s_thing, &s_list, 2, x->x_at);
}

static void hdial_fout(t_hdial *x, t_floatarg f)
{
    int i=(int)f;

    if(i < 0)
	i = 0;
    if(i >= x->x_number)
	i = x->x_number-1;

    if((x->x_change)&&(i != x->x_on_old))
    {
	SETFLOAT(x->x_at, (float)x->x_on_old);
	SETFLOAT(x->x_at+1, 0.0);
	outlet_list(x->x_gui.x_obj.ob_outlet, &s_list, 2, x->x_at);
	if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
	    pd_list(x->x_gui.x_snd->s_thing, &s_list, 2, x->x_at);
    }
    if(x->x_on != x->x_on_old)
	x->x_on_old = x->x_on;
    x->x_on = i;
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
    x->x_on_old = x->x_on;
    SETFLOAT(x->x_at, (float)x->x_on);
    SETFLOAT(x->x_at+1, 1.0);
    outlet_list(x->x_gui.x_obj.ob_outlet, &s_list, 2, x->x_at);
    if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
	pd_list(x->x_gui.x_snd->s_thing, &s_list, 2, x->x_at);
}

static void hdial_float(t_hdial *x, t_floatarg f)
{
    int i=(int)f;

    if(i < 0)
	i = 0;
    if(i >= x->x_number)
	i = x->x_number-1;

    if((x->x_change)&&(i != x->x_on_old))
    {
	if(x->x_gui.x_fsf.x_put_in2out)
	{
            SETFLOAT(x->x_at, (float)x->x_on_old);
	    SETFLOAT(x->x_at+1, 0.0);
	    outlet_list(x->x_gui.x_obj.ob_outlet, &s_list, 2, x->x_at);
	    if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
		pd_list(x->x_gui.x_snd->s_thing, &s_list, 2, x->x_at);
	}
    }
    if(x->x_on != x->x_on_old)
	x->x_on_old = x->x_on;
    x->x_on = i;
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
    x->x_on_old = x->x_on;
    if(x->x_gui.x_fsf.x_put_in2out)
    {
	SETFLOAT(x->x_at, (float)x->x_on);
        SETFLOAT(x->x_at+1, 1.0);
	outlet_list(x->x_gui.x_obj.ob_outlet, &s_list, 2, x->x_at);
	if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
	    pd_list(x->x_gui.x_snd->s_thing, &s_list, 2, x->x_at);
    }
}

static void hdial_click(t_hdial *x, t_floatarg xpos, t_floatarg ypos, t_floatarg shift, t_floatarg ctrl, t_floatarg alt)
{
    int xx = (int)xpos - (int)x->x_gui.x_obj.te_xpos;

    hdial_fout(x, (float)(xx / x->x_gui.x_w));
}

static int hdial_newclick(t_gobj *z, struct _glist *glist, int xpix, int ypix, int shift, int alt, int dbl, int doit)
{
    if(doit)
	hdial_click((t_hdial *)z, (t_floatarg)xpix, (t_floatarg)ypix, (t_floatarg)shift, 0, (t_floatarg)alt);
    return (1);
}

static void hdial_loadbang(t_hdial *x)
{
    if(!sys_noloadbang && x->x_gui.x_isa.x_loadinit)
	hdial_bang(x);
}

static void hdial_number(t_hdial *x, t_floatarg num)
{
    int n=(int)num;

    if(n < 1)
	n = 1;
    if(n > IEM_RADIO_MAX)
	n = IEM_RADIO_MAX;
    if(n != x->x_number)
    {
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_ERASE);
	x->x_number = n;
	if(x->x_on >= x->x_number)
	    x->x_on = x->x_number - 1;
	x->x_on_old = x->x_on;
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_NEW);
    }
}

static void hdial_size(t_hdial *x, t_symbol *s, int ac, t_atom *av)
{
    x->x_gui.x_w = iemgui_clip_size((int)atom_getintarg(0, ac, av));
    x->x_gui.x_h = x->x_gui.x_w;
    iemgui_size((void *)x, &x->x_gui);
}

static void hdial_delta(t_hdial *x, t_symbol *s, int ac, t_atom *av)
{iemgui_delta((void *)x, &x->x_gui, s, ac, av);}

static void hdial_pos(t_hdial *x, t_symbol *s, int ac, t_atom *av)
{iemgui_pos((void *)x, &x->x_gui, s, ac, av);}

static void hdial_color(t_hdial *x, t_symbol *s, int ac, t_atom *av)
{iemgui_color((void *)x, &x->x_gui, s, ac, av);}

static void hdial_send(t_hdial *x, t_symbol *s)
{iemgui_send(x, &x->x_gui, s);}

static void hdial_receive(t_hdial *x, t_symbol *s)
{iemgui_receive(x, &x->x_gui, s);}

static void hdial_label(t_hdial *x, t_symbol *s)
{iemgui_label((void *)x, &x->x_gui, s);}

static void hdial_label_pos(t_hdial *x, t_symbol *s, int ac, t_atom *av)
{iemgui_label_pos((void *)x, &x->x_gui, s, ac, av);}

static void hdial_label_font(t_hdial *x, t_symbol *s, int ac, t_atom *av)
{iemgui_label_font((void *)x, &x->x_gui, s, ac, av);}

static void hdial_init(t_hdial *x, t_floatarg f)
{
    x->x_gui.x_isa.x_loadinit = (f==0.0)?0:1;
}

static void hdial_double_change(t_hdial *x)
{x->x_change = 1;}

static void hdial_single_change(t_hdial *x)
{x->x_change = 0;}

static void hdial_list(t_hdial *x, t_symbol *s, int ac, t_atom *av)
{
    int l=iemgui_list((void *)x, &x->x_gui, s, ac, av);

    if(l < 0)
    {
	if(IS_A_FLOAT(av,0))
	    hdial_float(x, atom_getfloatarg(0, ac, av));
    }
    else if(l > 0)
    {
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
	canvas_fixlinesfor(glist_getcanvas(x->x_gui.x_glist), (t_text*)x);
    }
}

static void *hdial_new(t_symbol *s, int argc, t_atom *argv)
{
    t_hdial *x = (t_hdial *)pd_new(hdial_class);
    int bflcol[]={-262144, -1, -1};
    t_symbol *srl[3];
    int a=IEM_GUI_DEFAULTSIZE, on=0, f=0;
    int ldx=20, ldy=8, chg=1, num=8;
    int fs=8, iinit=0, ifstyle=0;
    int ftbreak=IEM_BNG_DEFAULTBREAKFLASHTIME, fthold=IEM_BNG_DEFAULTHOLDFLASHTIME;
    t_iem_init_symargs *init=(t_iem_init_symargs *)(&iinit);
    t_iem_fstyle_flags *fstyle=(t_iem_fstyle_flags *)(&ifstyle);
    char str[144];

    srl[0] = gensym("empty");
    srl[1] = gensym("empty");
    srl[2] = gensym("empty");

    if((argc == 15)&&IS_A_FLOAT(argv,0)&&IS_A_FLOAT(argv,1)&&IS_A_FLOAT(argv,2)
       &&IS_A_FLOAT(argv,3)
       &&(IS_A_SYMBOL(argv,4)||IS_A_FLOAT(argv,4))
       &&(IS_A_SYMBOL(argv,5)||IS_A_FLOAT(argv,5))
       &&(IS_A_SYMBOL(argv,6)||IS_A_FLOAT(argv,6))
       &&IS_A_FLOAT(argv,7)&&IS_A_FLOAT(argv,8)
       &&IS_A_FLOAT(argv,9)&&IS_A_FLOAT(argv,10)&&IS_A_FLOAT(argv,11)
       &&IS_A_FLOAT(argv,12)&&IS_A_FLOAT(argv,13)&&IS_A_FLOAT(argv,14))
    {
	a = (int)atom_getintarg(0, argc, argv);
	chg = (int)atom_getintarg(1, argc, argv);
	iinit = (int)atom_getintarg(2, argc, argv);
	num = (int)atom_getintarg(3, argc, argv);
	if(IS_A_SYMBOL(argv,4))
	    srl[0] = atom_getsymbolarg(4, argc, argv);
	else if(IS_A_FLOAT(argv,4))
	{
	    sprintf(str, "%d", (int)atom_getintarg(4, argc, argv));
	    srl[0] = gensym(str);
	}
	if(IS_A_SYMBOL(argv,5))
	    srl[1] = atom_getsymbolarg(5, argc, argv);
	else if(IS_A_FLOAT(argv,5))
	{
	    sprintf(str, "%d", (int)atom_getintarg(5, argc, argv));
	    srl[1] = gensym(str);
	}
	if(IS_A_SYMBOL(argv,6))
	    srl[2] = atom_getsymbolarg(6, argc, argv);
	else if(IS_A_FLOAT(argv,6))
	{
	    sprintf(str, "%d", (int)atom_getintarg(6, argc, argv));
	    srl[2] = gensym(str);
	}
	ldx = (int)atom_getintarg(7, argc, argv);
	ldy = (int)atom_getintarg(8, argc, argv);
	ifstyle = (int)atom_getintarg(9, argc, argv);
	fs = (int)atom_getintarg(10, argc, argv);
	bflcol[0] = (int)atom_getintarg(11, argc, argv);
	bflcol[1] = (int)atom_getintarg(12, argc, argv);
	bflcol[2] = (int)atom_getintarg(13, argc, argv);
	on = (int)atom_getintarg(14, argc, argv);
    }
    x->x_gui.x_draw = (t_iemfunptr)hdial_draw;
    iinit &= IEM_INIT_ARGS_ALL;
    ifstyle &= IEM_FSTYLE_FLAGS_ALL;
    fstyle->x_snd_able = 1;
    fstyle->x_rcv_able = 1;
    x->x_gui.x_glist = (t_glist *)canvas_getcurrent();
    x->x_gui.x_isa = *init;
    if(!strcmp(srl[0]->s_name, "empty")) fstyle->x_snd_able = 0;
    if(!strcmp(srl[1]->s_name, "empty")) fstyle->x_rcv_able = 0;
    if(fstyle->x_font_style == 1) strcpy(x->x_gui.x_font, "helvetica");
    else if(fstyle->x_font_style == 2) strcpy(x->x_gui.x_font, "times");
    else { fstyle->x_font_style = 0;
	strcpy(x->x_gui.x_font, "courier"); }
    x->x_gui.x_fsf = *fstyle;
    x->x_gui.x_unique_num = 0;
    if(num < 1)
	num = 1;
    if(num > IEM_RADIO_MAX)
	num = IEM_RADIO_MAX;
    x->x_number = num;
    if(on < 0)
	on = 0;
    if(on >= x->x_number)
	on = x->x_number - 1;
    if(x->x_gui.x_isa.x_loadinit)
	x->x_on = on;
    else
	x->x_on = 0;
    x->x_on_old = x->x_on;
    x->x_change = (chg==0)?0:1;
    iemgui_first_dollararg2sym(&x->x_gui, srl);
    if(x->x_gui.x_fsf.x_rcv_able) pd_bind(&x->x_gui.x_obj.ob_pd, srl[1]);
    x->x_gui.x_snd = srl[0];
    x->x_gui.x_rcv = srl[1];
    x->x_gui.x_lab = srl[2];
    x->x_gui.x_ldx = ldx;
    x->x_gui.x_ldy = ldy;
    if(fs < 4)
	fs = 4;
    x->x_gui.x_fontsize = fs;
    x->x_gui.x_w = iemgui_clip_size(a);
    x->x_gui.x_h = x->x_gui.x_w;
    iemgui_verify_snd_ne_rcv(&x->x_gui);
    iemgui_all_colfromload(&x->x_gui, bflcol);
    outlet_new(&x->x_gui.x_obj, &s_list);
    return (x);
}

static void hdial_ff(t_hdial *x)
{
    if(x->x_gui.x_fsf.x_selected)
	pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
    if(x->x_gui.x_fsf.x_rcv_able)
	pd_unbind(&x->x_gui.x_obj.ob_pd, x->x_gui.x_rcv);
}

void hdial_setup(void)
{
    hdial_class = class_new(gensym("hdl"), (t_newmethod)hdial_new,
			    (t_method)hdial_ff, sizeof(t_hdial), 0, A_GIMME, 0);
    class_addcreator((t_newmethod)hdial_new, gensym("rdb"), A_GIMME, 0);
    class_addcreator((t_newmethod)hdial_new, gensym("radiobut"), A_GIMME, 0);
    class_addcreator((t_newmethod)hdial_new, gensym("radiobutton"), A_GIMME, 0);
    class_addbang(hdial_class, hdial_bang);
    class_addfloat(hdial_class, hdial_float);
    class_addlist(hdial_class, hdial_list);
    class_addmethod(hdial_class, (t_method)hdial_click, gensym("click"),
		    A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
    class_addmethod(hdial_class, (t_method)hdial_dialog, gensym("dialog"),
		    A_GIMME, 0);
    class_addmethod(hdial_class, (t_method)hdial_loadbang, gensym("loadbang"), 0);
    class_addmethod(hdial_class, (t_method)hdial_set, gensym("set"), A_FLOAT, 0);
    class_addmethod(hdial_class, (t_method)hdial_size, gensym("size"), A_GIMME, 0);
    class_addmethod(hdial_class, (t_method)hdial_delta, gensym("delta"), A_GIMME, 0);
    class_addmethod(hdial_class, (t_method)hdial_pos, gensym("pos"), A_GIMME, 0);
    class_addmethod(hdial_class, (t_method)hdial_color, gensym("color"), A_GIMME, 0);
    class_addmethod(hdial_class, (t_method)hdial_send, gensym("send"), A_DEFSYM, 0);
    class_addmethod(hdial_class, (t_method)hdial_receive, gensym("receive"), A_DEFSYM, 0);
    class_addmethod(hdial_class, (t_method)hdial_label, gensym("label"), A_DEFSYM, 0);
    class_addmethod(hdial_class, (t_method)hdial_label_pos, gensym("label_pos"), A_GIMME, 0);
    class_addmethod(hdial_class, (t_method)hdial_label_font, gensym("label_font"), A_GIMME, 0);
    class_addmethod(hdial_class, (t_method)hdial_init, gensym("init"), A_FLOAT, 0);
    class_addmethod(hdial_class, (t_method)hdial_number, gensym("number"), A_FLOAT, 0);
    class_addmethod(hdial_class, (t_method)hdial_single_change, gensym("single_change"), 0);
    class_addmethod(hdial_class, (t_method)hdial_double_change, gensym("double_change"), 0);
    if(!iemgui_key_sym)
	iemgui_key_sym = gensym("#keyname");
    hdial_widgetbehavior.w_getrectfn = hdial_getrect;
    hdial_widgetbehavior.w_displacefn = iemgui_displace;
    hdial_widgetbehavior.w_selectfn = iemgui_select;
    hdial_widgetbehavior.w_activatefn = NULL;
    hdial_widgetbehavior.w_deletefn = iemgui_delete;
    hdial_widgetbehavior.w_visfn = iemgui_vis;
    hdial_widgetbehavior.w_clickfn = hdial_newclick;
    hdial_widgetbehavior.w_propertiesfn = hdial_properties;
    hdial_widgetbehavior.w_savefn = hdial_save;
    class_setwidget(hdial_class, &hdial_widgetbehavior);
    class_sethelpsymbol(hdial_class, gensym("radiobutton"));
}








/* --------------- tgl     gui-toggle ------------------------- */

t_widgetbehavior toggle_widgetbehavior;
static t_class *toggle_class;

/* widget helper functions */

void toggle_draw_update(t_toggle *x, t_glist *glist)
{
    if(glist_isvisible(glist))
    {
	t_canvas *canvas=glist_getcanvas(glist);

	sys_vgui(".x%x.c itemconfigure %xX1 -fill #%6.6x\n", canvas, x,
		 (x->x_on!=0.0)?x->x_gui.x_fcol:x->x_gui.x_bcol);
	sys_vgui(".x%x.c itemconfigure %xX2 -fill #%6.6x\n", canvas, x,
		 (x->x_on!=0.0)?x->x_gui.x_fcol:x->x_gui.x_bcol);
    }
}

void toggle_draw_new(t_toggle *x, t_glist *glist)
{
    t_canvas *canvas=glist_getcanvas(glist);
    int w=1, xx=x->x_gui.x_obj.te_xpos, yy=x->x_gui.x_obj.te_ypos;

    if(x->x_gui.x_w >= 30)
	w = 2;
    if(x->x_gui.x_w >= 60)
	w = 3;
    sys_vgui(".x%x.c create rectangle %d %d %d %d -fill #%6.6x -tags %xBASE\n",
	     canvas, xx, yy, xx + x->x_gui.x_w, yy + x->x_gui.x_h,
	     x->x_gui.x_bcol, x);
    sys_vgui(".x%x.c create line %d %d %d %d -width %d -fill #%6.6x -tags %xX1\n",
	     canvas, xx+w+1, yy+w+1, xx + x->x_gui.x_w-w, yy + x->x_gui.x_h-w, w,
	     (x->x_on!=0.0)?x->x_gui.x_fcol:x->x_gui.x_bcol, x);
    sys_vgui(".x%x.c create line %d %d %d %d -width %d -fill #%6.6x -tags %xX2\n",
	     canvas, xx+w+1, yy + x->x_gui.x_h-w-1, xx + x->x_gui.x_w-w, yy+w, w,
	     (x->x_on!=0.0)?x->x_gui.x_fcol:x->x_gui.x_bcol, x);
    sys_vgui(".x%x.c create text %d %d -text {%s} -anchor w \
	     -font {%s %d bold} -fill #%6.6x -tags %xLABEL\n",
	     canvas, x->x_gui.x_obj.te_xpos+x->x_gui.x_ldx,
	     x->x_gui.x_obj.te_ypos+x->x_gui.x_ldy,
	     strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"",
	     x->x_gui.x_font, x->x_gui.x_fontsize, x->x_gui.x_lcol, x);
    if(!x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xOUT%d\n",
	     canvas, xx, yy + x->x_gui.x_h-1, xx + IOWIDTH, yy + x->x_gui.x_h, x, 0);
    if(!x->x_gui.x_fsf.x_rcv_able)
	sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xIN%d\n",
	     canvas, xx, yy, xx + IOWIDTH, yy+1, x, 0);
}

void toggle_draw_move(t_toggle *x, t_glist *glist)
{
    t_canvas *canvas=glist_getcanvas(glist);
    int w=1, xx=x->x_gui.x_obj.te_xpos, yy=x->x_gui.x_obj.te_ypos;

    if(x->x_gui.x_w >= 30)
	w = 2;
   
    if(x->x_gui.x_w >= 60)
	w = 3;
    sys_vgui(".x%x.c coords %xBASE %d %d %d %d\n",
	     canvas, x, xx, yy, xx + x->x_gui.x_w, yy + x->x_gui.x_h);
    sys_vgui(".x%x.c itemconfigure %xX1 -width %d\n", canvas, x, w);
    sys_vgui(".x%x.c coords %xX1 %d %d %d %d\n",
	     canvas, x, xx+w+1, yy+w+1, xx + x->x_gui.x_w-w, yy + x->x_gui.x_h-w);
    sys_vgui(".x%x.c itemconfigure %xX2 -width %d\n", canvas, x, w);
    sys_vgui(".x%x.c coords %xX2 %d %d %d %d\n",
	     canvas, x, xx+w+1, yy + x->x_gui.x_h-w-1, xx + x->x_gui.x_w-w, yy+w);
    sys_vgui(".x%x.c coords %xLABEL %d %d\n",
	     canvas, x, xx+x->x_gui.x_ldx, yy+x->x_gui.x_ldy);
    if(!x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c coords %xOUT%d %d %d %d %d\n",
	     canvas, x, 0, xx, yy + x->x_gui.x_h-1, xx + IOWIDTH, yy + x->x_gui.x_h);
    if(!x->x_gui.x_fsf.x_rcv_able)
	sys_vgui(".x%x.c coords %xIN%d %d %d %d %d\n",
	     canvas, x, 0, xx, yy, xx + IOWIDTH, yy+1);
}

void toggle_draw_erase(t_toggle* x, t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c delete %xBASE\n", canvas, x);
    sys_vgui(".x%x.c delete %xX1\n", canvas, x);
    sys_vgui(".x%x.c delete %xX2\n", canvas, x);
    sys_vgui(".x%x.c delete %xLABEL\n", canvas, x);
    if(!x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c delete %xOUT%d\n", canvas, x, 0);
    if(!x->x_gui.x_fsf.x_rcv_able)
	sys_vgui(".x%x.c delete %xIN%d\n", canvas, x, 0);
}

void toggle_draw_config(t_toggle* x, t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c itemconfigure %xLABEL -font {%s %d bold} -fill #%6.6x -text {%s} \n",
	     canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize,
	     x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
	     strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"");
    sys_vgui(".x%x.c itemconfigure %xBASE -fill #%6.6x\n", canvas, x,
	     x->x_gui.x_bcol);
    sys_vgui(".x%x.c itemconfigure %xX1 -fill #%6.6x\n", canvas, x,
	     x->x_on?x->x_gui.x_fcol:x->x_gui.x_bcol);
    sys_vgui(".x%x.c itemconfigure %xX2 -fill #%6.6x\n", canvas, x,
	     x->x_on?x->x_gui.x_fcol:x->x_gui.x_bcol);
}

void toggle_draw_io(t_toggle* x, t_glist* glist, int old_snd_rcv_flags)
{
    t_canvas *canvas=glist_getcanvas(glist);

    if((old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && !x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xOUT%d\n",
	     canvas, x->x_gui.x_obj.te_xpos,
	     x->x_gui.x_obj.te_ypos + x->x_gui.x_h-1, x->x_gui.x_obj.te_xpos + IOWIDTH,
	     x->x_gui.x_obj.te_ypos + x->x_gui.x_h, x, 0);
    if(!(old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c delete %xOUT%d\n", canvas, x, 0);
    if((old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && !x->x_gui.x_fsf.x_rcv_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xIN%d\n",
	     canvas, x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos,
	     x->x_gui.x_obj.te_xpos + IOWIDTH, x->x_gui.x_obj.te_ypos+1, x, 0);
    if(!(old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && x->x_gui.x_fsf.x_rcv_able)
	sys_vgui(".x%x.c delete %xIN%d\n", canvas, x, 0);
}

void toggle_draw_select(t_toggle* x, t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    if(x->x_gui.x_fsf.x_selected)
    {
	pd_bind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
	sys_vgui(".x%x.c itemconfigure %xBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
	sys_vgui(".x%x.c itemconfigure %xLABEL -fill #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
    }
    else
    {
	pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
	sys_vgui(".x%x.c itemconfigure %xBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
	sys_vgui(".x%x.c itemconfigure %xLABEL -fill #%6.6x\n", canvas, x, x->x_gui.x_lcol);
    }
}

void toggle_draw(t_toggle *x, t_glist *glist, int mode)
{
    if(mode == IEM_GUI_DRAW_MODE_UPDATE)
	toggle_draw_update(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_MOVE)
	toggle_draw_move(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_NEW)
	toggle_draw_new(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_SELECT)
	toggle_draw_select(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_ERASE)
	toggle_draw_erase(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_CONFIG)
	toggle_draw_config(x, glist);
    else if(mode >= IEM_GUI_DRAW_MODE_IO)
	toggle_draw_io(x, glist, mode - IEM_GUI_DRAW_MODE_IO);
}

/* ------------------------ tgl widgetbehaviour----------------------------- */

static void toggle_getrect(t_gobj *z, t_glist *owner, int *xp1, int *yp1, int *xp2, int *yp2)
{
    t_toggle *x = (t_toggle *)z;

    *xp1 = x->x_gui.x_obj.te_xpos;
    *yp1 = x->x_gui.x_obj.te_ypos;
    *xp2 = x->x_gui.x_obj.te_xpos + x->x_gui.x_w;
    *yp2 = x->x_gui.x_obj.te_ypos + x->x_gui.x_h;
}

static void toggle_save(t_gobj *z, t_binbuf *b)
{
    t_toggle *x = (t_toggle *)z;
    int bflcol[3], *ip1, *ip2;
    t_symbol *srl[3];

    iemgui_save(&x->x_gui, srl, bflcol);
    ip1 = (int *)(&x->x_gui.x_isa);
    ip2 = (int *)(&x->x_gui.x_fsf);
    binbuf_addv(b, "ssiisiisssiiiiiiiff", gensym("#X"),gensym("obj"),
		(t_int)x->x_gui.x_obj.te_xpos, (t_int)x->x_gui.x_obj.te_ypos,
		gensym("tgl"), x->x_gui.x_w,
		(*ip1)&IEM_INIT_ARGS_ALL,
		srl[0], srl[1], srl[2],
		x->x_gui.x_ldx, x->x_gui.x_ldy,
		(*ip2)&IEM_FSTYLE_FLAGS_ALL, x->x_gui.x_fontsize,
		bflcol[0], bflcol[1], bflcol[2], x->x_on, x->x_nonzero);
    binbuf_addv(b, ";");
}

static void toggle_properties(t_gobj *z, t_glist *owner)
{
    t_toggle *x = (t_toggle *)z;
    char buf[800];
    t_symbol *srl[3];

    iemgui_properties(&x->x_gui, srl);
    sprintf(buf, "pdtk_iemgui_dialog %%s TOGGLE \
	    ----------dimensions(pix):----------- %d %d size: 0 0 empty \
	    -----------non-zero-value:----------- %g value: 0.0 empty %g \
	    -1 lin log %d %d \
	    %s %s \
	    %s %d %d \
	    %d %d \
	    %d %d %d\n",
	    x->x_gui.x_w, IEM_GUI_MINSIZE,
	    x->x_nonzero, 1.0,/*non_zero-schedule*/
	    x->x_gui.x_isa.x_loadinit, -1,/*no multi*/
	    srl[0]->s_name, srl[1]->s_name,
	    srl[2]->s_name, x->x_gui.x_ldx, x->x_gui.x_ldy,
	    x->x_gui.x_fsf.x_font_style, x->x_gui.x_fontsize,
	    0xffffff & x->x_gui.x_bcol, 0xffffff & x->x_gui.x_fcol, 0xffffff & x->x_gui.x_lcol);
    gfxstub_new(&x->x_gui.x_obj.ob_pd, x, buf);
}

static void toggle_bang(t_toggle *x)
{
    x->x_on = (x->x_on==0.0)?x->x_nonzero:0.0;
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
    outlet_float(x->x_gui.x_obj.ob_outlet, x->x_on);
    if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
	pd_float(x->x_gui.x_snd->s_thing, x->x_on);
}

static void toggle_dialog(t_toggle *x, t_symbol *s, int argc, t_atom *argv)
{
    t_symbol *srl[3];
    int a = (int)atom_getintarg(0, argc, argv);
    float nonzero = (float)atom_getfloatarg(2, argc, argv);
    int sr_flags;

    if(nonzero == 0.0)
	nonzero = 1.0;
    x->x_nonzero = nonzero;
    if(x->x_on != 0.0)
	x->x_on = x->x_nonzero;
    sr_flags = iemgui_dialog(&x->x_gui, srl, argc, argv);
    x->x_gui.x_w = iemgui_clip_size(a);
    x->x_gui.x_h = x->x_gui.x_w;
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_CONFIG);
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_IO + sr_flags);
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
    canvas_fixlinesfor(glist_getcanvas(x->x_gui.x_glist), (t_text*)x);
}

static void toggle_click(t_toggle *x, t_floatarg xpos, t_floatarg ypos, t_floatarg shift, t_floatarg ctrl, t_floatarg alt)
{toggle_bang(x);}

static int toggle_newclick(t_gobj *z, struct _glist *glist, int xpix, int ypix, int shift, int alt, int dbl, int doit)
{
    if(doit)
	toggle_click((t_toggle *)z, (t_floatarg)xpix, (t_floatarg)ypix, (t_floatarg)shift, 0, (t_floatarg)alt);
    return (1);
}

static void toggle_set(t_toggle *x, t_floatarg f)
{
    x->x_on = f;
    if(f != 0.0)
	x->x_nonzero = f;
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
}

static void toggle_float(t_toggle *x, t_floatarg f)
{
    toggle_set(x, f);
    if(x->x_gui.x_fsf.x_put_in2out)
    {
	outlet_float(x->x_gui.x_obj.ob_outlet, x->x_on);
	if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
	    pd_float(x->x_gui.x_snd->s_thing, x->x_on);
    }
}

static void toggle_fout(t_toggle *x, t_floatarg f)
{
    toggle_set(x, f);
    outlet_float(x->x_gui.x_obj.ob_outlet, x->x_on);
    if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
	pd_float(x->x_gui.x_snd->s_thing, x->x_on);
}

static void toggle_loadbang(t_toggle *x)
{
    if(!sys_noloadbang && x->x_gui.x_isa.x_loadinit)
	toggle_fout(x, (float)x->x_on);
}

static void toggle_size(t_toggle *x, t_symbol *s, int ac, t_atom *av)
{
    x->x_gui.x_w = iemgui_clip_size((int)atom_getintarg(0, ac, av));
    x->x_gui.x_h = x->x_gui.x_w;
    iemgui_size((void *)x, &x->x_gui);
}

static void toggle_delta(t_toggle *x, t_symbol *s, int ac, t_atom *av)
{iemgui_delta((void *)x, &x->x_gui, s, ac, av);}

static void toggle_pos(t_toggle *x, t_symbol *s, int ac, t_atom *av)
{iemgui_pos((void *)x, &x->x_gui, s, ac, av);}

static void toggle_color(t_toggle *x, t_symbol *s, int ac, t_atom *av)
{iemgui_color((void *)x, &x->x_gui, s, ac, av);}

static void toggle_send(t_toggle *x, t_symbol *s)
{iemgui_send(x, &x->x_gui, s);}

static void toggle_receive(t_toggle *x, t_symbol *s)
{iemgui_receive(x, &x->x_gui, s);}

static void toggle_label(t_toggle *x, t_symbol *s)
{iemgui_label((void *)x, &x->x_gui, s);}

static void toggle_label_font(t_toggle *x, t_symbol *s, int ac, t_atom *av)
{iemgui_label_font((void *)x, &x->x_gui, s, ac, av);}

static void toggle_label_pos(t_toggle *x, t_symbol *s, int ac, t_atom *av)
{iemgui_label_pos((void *)x, &x->x_gui, s, ac, av);}

static void toggle_init(t_toggle *x, t_floatarg f)
{
    x->x_gui.x_isa.x_loadinit = (f==0.0)?0:1;
}

static void toggle_nonzero(t_toggle *x, t_floatarg f)
{
    if(f != 0.0)
	x->x_nonzero = f;
}

static void toggle_list(t_toggle *x, t_symbol *s, int ac, t_atom *av)
{
    int l=iemgui_list((void *)x, &x->x_gui, s, ac, av);

    if(l < 0)
    {
	if(IS_A_FLOAT(av,0))
	    toggle_float(x, atom_getfloatarg(0, ac, av));
    }
    else if(l > 0)
    {
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
	canvas_fixlinesfor(glist_getcanvas(x->x_gui.x_glist), (t_text*)x);
    }
}

static void *toggle_new(t_symbol *s, int argc, t_atom *argv)
{
    t_toggle *x = (t_toggle *)pd_new(toggle_class);
    int bflcol[]={-262144, -1, -1};
    t_symbol *srl[3];
    int a=IEM_GUI_DEFAULTSIZE, f=0;
    int ldx=20, ldy=8;
    int fs=8, iinit=0, ifstyle=0;
    float on=0.0, nonzero=1.0;
    t_iem_init_symargs *init=(t_iem_init_symargs *)(&iinit);
    t_iem_fstyle_flags *fstyle=(t_iem_fstyle_flags *)(&ifstyle);
    char str[144];

    srl[0] = gensym("empty");
    srl[1] = gensym("empty");
    srl[2] = gensym("empty");

    if(((argc == 13)||(argc == 14))&&IS_A_FLOAT(argv,0)
       &&IS_A_FLOAT(argv,1)
       &&(IS_A_SYMBOL(argv,2)||IS_A_FLOAT(argv,2))
       &&(IS_A_SYMBOL(argv,3)||IS_A_FLOAT(argv,3))
       &&(IS_A_SYMBOL(argv,4)||IS_A_FLOAT(argv,4))
       &&IS_A_FLOAT(argv,5)&&IS_A_FLOAT(argv,6)
       &&IS_A_FLOAT(argv,7)&&IS_A_FLOAT(argv,8)&&IS_A_FLOAT(argv,9)
       &&IS_A_FLOAT(argv,10)&&IS_A_FLOAT(argv,11)&&IS_A_FLOAT(argv,12))
    {
	a = (int)atom_getintarg(0, argc, argv);
	iinit = (int)atom_getintarg(1, argc, argv);
	if(IS_A_SYMBOL(argv,2))
	    srl[0] = atom_getsymbolarg(2, argc, argv);
	else if(IS_A_FLOAT(argv,2))
	{
	    sprintf(str, "%d", (int)atom_getintarg(2, argc, argv));
	    srl[0] = gensym(str);
	}
	if(IS_A_SYMBOL(argv,3))
	    srl[1] = atom_getsymbolarg(3, argc, argv);
	else if(IS_A_FLOAT(argv,3))
	{
	    sprintf(str, "%d", (int)atom_getintarg(3, argc, argv));
	    srl[1] = gensym(str);
	}
	if(IS_A_SYMBOL(argv,4))
	    srl[2] = atom_getsymbolarg(4, argc, argv);
	else if(IS_A_FLOAT(argv,4))
	{
	    sprintf(str, "%d", (int)atom_getintarg(4, argc, argv));
	    srl[2] = gensym(str);
	}
	ldx = (int)atom_getintarg(5, argc, argv);
	ldy = (int)atom_getintarg(6, argc, argv);
	ifstyle = (int)atom_getintarg(7, argc, argv);
	fs = (int)atom_getintarg(8, argc, argv);
	bflcol[0] = (int)atom_getintarg(9, argc, argv);
	bflcol[1] = (int)atom_getintarg(10, argc, argv);
	bflcol[2] = (int)atom_getintarg(11, argc, argv);
	on = (float)atom_getfloatarg(12, argc, argv);
    }
    if((argc == 14)&&IS_A_FLOAT(argv,13))
	nonzero = (float)atom_getfloatarg(13, argc, argv);
    x->x_gui.x_draw = (t_iemfunptr)toggle_draw;
    iinit &= IEM_INIT_ARGS_ALL;
    ifstyle &= IEM_FSTYLE_FLAGS_ALL;

    fstyle->x_snd_able = 1;
    fstyle->x_rcv_able = 1;
    x->x_gui.x_glist = (t_glist *)canvas_getcurrent();
    x->x_gui.x_isa = *init;
    if(!strcmp(srl[0]->s_name, "empty")) fstyle->x_snd_able = 0;
    if(!strcmp(srl[1]->s_name, "empty")) fstyle->x_rcv_able = 0;
    x->x_gui.x_unique_num = 0;
    if(fstyle->x_font_style == 1) strcpy(x->x_gui.x_font, "helvetica");
    else if(fstyle->x_font_style == 2) strcpy(x->x_gui.x_font, "times");
    else { fstyle->x_font_style = 0;
	strcpy(x->x_gui.x_font, "courier"); }
    x->x_gui.x_fsf = *fstyle;
    x->x_nonzero = (nonzero!=0.0)?nonzero:1.0;
    if(x->x_gui.x_isa.x_loadinit)
	x->x_on = (on!=0.0)?nonzero:0.0;
    else
	x->x_on = 0.0;
    iemgui_first_dollararg2sym(&x->x_gui, srl);
    if(x->x_gui.x_fsf.x_rcv_able) pd_bind(&x->x_gui.x_obj.ob_pd, srl[1]);
    x->x_gui.x_snd = srl[0];
    x->x_gui.x_rcv = srl[1];
    x->x_gui.x_lab = srl[2];
    x->x_gui.x_ldx = ldx;
    x->x_gui.x_ldy = ldy;

    if(fs < 4)
	fs = 4;
    x->x_gui.x_fontsize = fs;
    x->x_gui.x_w = iemgui_clip_size(a);
    x->x_gui.x_h = x->x_gui.x_w;
    iemgui_all_colfromload(&x->x_gui, bflcol);
    iemgui_verify_snd_ne_rcv(&x->x_gui);
    outlet_new(&x->x_gui.x_obj, &s_float);
    return (x);
}

static void toggle_ff(t_toggle *x)
{
    if(x->x_gui.x_fsf.x_selected)
	pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
    if(x->x_gui.x_fsf.x_rcv_able)
	pd_unbind(&x->x_gui.x_obj.ob_pd, x->x_gui.x_rcv);
}

void toggle_setup(void)
{
    toggle_class = class_new(gensym("tgl"), (t_newmethod)toggle_new,
			     (t_method)toggle_ff, sizeof(t_toggle), 0, A_GIMME, 0);
    class_addcreator((t_newmethod)toggle_new, gensym("toggle"), A_GIMME, 0);
    class_addbang(toggle_class, toggle_bang);
    class_addfloat(toggle_class, toggle_float);
    class_addlist(toggle_class, toggle_list);
    class_addmethod(toggle_class, (t_method)toggle_click, gensym("click"),
		    A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
    class_addmethod(toggle_class, (t_method)toggle_dialog, gensym("dialog"),
		    A_GIMME, 0);
    class_addmethod(toggle_class, (t_method)toggle_loadbang, gensym("loadbang"), 0);
    class_addmethod(toggle_class, (t_method)toggle_set, gensym("set"), A_FLOAT, 0);
    class_addmethod(toggle_class, (t_method)toggle_size, gensym("size"), A_GIMME, 0);
    class_addmethod(toggle_class, (t_method)toggle_delta, gensym("delta"), A_GIMME, 0);
    class_addmethod(toggle_class, (t_method)toggle_pos, gensym("pos"), A_GIMME, 0);
    class_addmethod(toggle_class, (t_method)toggle_color, gensym("color"), A_GIMME, 0);
    class_addmethod(toggle_class, (t_method)toggle_send, gensym("send"), A_DEFSYM, 0);
    class_addmethod(toggle_class, (t_method)toggle_receive, gensym("receive"), A_DEFSYM, 0);
    class_addmethod(toggle_class, (t_method)toggle_label, gensym("label"), A_DEFSYM, 0);
    class_addmethod(toggle_class, (t_method)toggle_label_pos, gensym("label_pos"), A_GIMME, 0);
    class_addmethod(toggle_class, (t_method)toggle_label_font, gensym("label_font"), A_GIMME, 0);
    class_addmethod(toggle_class, (t_method)toggle_init, gensym("init"), A_FLOAT, 0);
    class_addmethod(toggle_class, (t_method)toggle_nonzero, gensym("nonzero"), A_FLOAT, 0);
    if(!iemgui_key_sym)
	iemgui_key_sym = gensym("#keyname");
    toggle_widgetbehavior.w_getrectfn = toggle_getrect;
    toggle_widgetbehavior.w_displacefn = iemgui_displace;
    toggle_widgetbehavior.w_selectfn = iemgui_select;
    toggle_widgetbehavior.w_activatefn = NULL;
    toggle_widgetbehavior.w_deletefn = iemgui_delete;
    toggle_widgetbehavior.w_visfn = iemgui_vis;
    toggle_widgetbehavior.w_clickfn = toggle_newclick;
    toggle_widgetbehavior.w_propertiesfn = toggle_properties;
    toggle_widgetbehavior.w_savefn = toggle_save;
    class_setwidget(toggle_class, &toggle_widgetbehavior);
    class_sethelpsymbol(toggle_class, gensym("toggle"));
}







/* ---------- cnv  my gui-canvas for a window ---------------- */

t_widgetbehavior my_canvas_widgetbehavior;
static t_class *my_canvas_class;

/* widget helper functions */

void my_canvas_draw_new(t_my_canvas *x, t_glist *glist)
{
    t_canvas *canvas=glist_getcanvas(glist);


    sys_vgui(".x%x.c create rectangle %d %d %d %d -fill #%6.6x -outline #%6.6x -tags %xRECT\n",
	     canvas, x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos,
	     x->x_gui.x_obj.te_xpos + x->x_vis_w, x->x_gui.x_obj.te_ypos + x->x_vis_h,
	     x->x_gui.x_bcol, x->x_gui.x_bcol, x);
    sys_vgui(".x%x.c create rectangle %d %d %d %d -outline #%6.6x -tags %xBASE\n",
	     canvas, x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos,
	     x->x_gui.x_obj.te_xpos + x->x_gui.x_w, x->x_gui.x_obj.te_ypos + x->x_gui.x_h,
	     x->x_gui.x_bcol, x);
    sys_vgui(".x%x.c create text %d %d -text {%s} -anchor w \
	     -font {%s %d bold} -fill #%6.6x -tags %xLABEL\n",
	     canvas, x->x_gui.x_obj.te_xpos+x->x_gui.x_ldx, x->x_gui.x_obj.te_ypos+x->x_gui.x_ldy,
	     strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"",
	     x->x_gui.x_font, x->x_gui.x_fontsize, x->x_gui.x_lcol, x);
}

void my_canvas_draw_move(t_my_canvas *x, t_glist *glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c coords %xRECT %d %d %d %d\n",
	     canvas, x, x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos, x->x_gui.x_obj.te_xpos + x->x_vis_w,
	     x->x_gui.x_obj.te_ypos + x->x_vis_h);
    sys_vgui(".x%x.c coords %xBASE %d %d %d %d\n",
	     canvas, x, x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos,
	     x->x_gui.x_obj.te_xpos + x->x_gui.x_w, x->x_gui.x_obj.te_ypos + x->x_gui.x_h);
    sys_vgui(".x%x.c coords %xLABEL %d %d\n",
	     canvas, x, x->x_gui.x_obj.te_xpos+x->x_gui.x_ldx,
	     x->x_gui.x_obj.te_ypos+x->x_gui.x_ldy);
}

void my_canvas_draw_erase(t_my_canvas* x, t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c delete %xBASE\n", canvas, x);
    sys_vgui(".x%x.c delete %xRECT\n", canvas, x);
    sys_vgui(".x%x.c delete %xLABEL\n", canvas, x);
}

void my_canvas_draw_config(t_my_canvas* x, t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c itemconfigure %xRECT -fill #%6.6x -outline #%6.6x\n", canvas, x,
	     x->x_gui.x_bcol, x->x_gui.x_bcol);
    sys_vgui(".x%x.c itemconfigure %xBASE -outline #%6.6x\n", canvas, x,
	     x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_bcol);
    sys_vgui(".x%x.c itemconfigure %xLABEL -font {%s %d bold} -fill #%6.6x -text {%s} \n",
	     canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize, x->x_gui.x_lcol,
	     strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"");
}

void my_canvas_draw_select(t_my_canvas* x, t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    if(x->x_gui.x_fsf.x_selected)
    {
	pd_bind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
	sys_vgui(".x%x.c itemconfigure %xBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
    }
    else
    {
	pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
	sys_vgui(".x%x.c itemconfigure %xBASE -outline #%6.6x\n", canvas, x, x->x_gui.x_bcol);
    }
}

void my_canvas_draw(t_my_canvas *x, t_glist *glist, int mode)
{
    if(mode == IEM_GUI_DRAW_MODE_MOVE)
	my_canvas_draw_move(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_NEW)
	my_canvas_draw_new(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_SELECT)
	my_canvas_draw_select(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_ERASE)
	my_canvas_draw_erase(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_CONFIG)
	my_canvas_draw_config(x, glist);
}

/* ------------------------ cnv widgetbehaviour----------------------------- */

static void my_canvas_getrect(t_gobj *z, t_glist *owner, int *xp1, int *yp1, int *xp2, int *yp2)
{
    t_my_canvas *x = (t_my_canvas *)z;

    *xp1 = x->x_gui.x_obj.te_xpos;
    *yp1 = x->x_gui.x_obj.te_ypos;
    *xp2 = x->x_gui.x_obj.te_xpos + x->x_gui.x_w;
    *yp2 = x->x_gui.x_obj.te_ypos + x->x_gui.x_h;
}

static void my_canvas_save(t_gobj *z, t_binbuf *b)
{
    t_my_canvas *x = (t_my_canvas *)z;
    int bflcol[3], *ip1, *ip2;
    t_symbol *srl[3];

    iemgui_save(&x->x_gui, srl, bflcol);
    ip1 = (int *)(&x->x_gui.x_isa);
    ip2 = (int *)(&x->x_gui.x_fsf);
    binbuf_addv(b, "ssiisiiisssiiiiiii", gensym("#X"),gensym("obj"),
		(t_int)x->x_gui.x_obj.te_xpos, (t_int)x->x_gui.x_obj.te_ypos,
		gensym("cnv"), x->x_gui.x_w, x->x_vis_w, x->x_vis_h,
		srl[0], srl[1], srl[2], x->x_gui.x_ldx, x->x_gui.x_ldy,
		(*ip2)&IEM_FSTYLE_FLAGS_ALL, x->x_gui.x_fontsize,
		bflcol[0], bflcol[2], (*ip1)&IEM_INIT_ARGS_ALL);
    binbuf_addv(b, ";");
}

static void my_canvas_properties(t_gobj *z, t_glist *owner)
{
    t_my_canvas *x = (t_my_canvas *)z;
    char buf[800];
    t_symbol *srl[3];

    iemgui_properties(&x->x_gui, srl);
    sprintf(buf, "pdtk_iemgui_dialog %%s MY_CANVAS \
	    ------selectable_dimensions(pix):------ %d %d size: 0.0 0.0 empty \
	    ------visible_rectangle(pix)(pix):------ %d width: %d height: %d \
	    %d empty empty %d %d \
	    %s %s \
	    %s %d %d \
	    %d %d \
	    %d %d %d\n",
	    x->x_gui.x_w, 1,
	    x->x_vis_w, x->x_vis_h, 0,/*no_schedule*/
	    -1, -1, -1,/*no linlog, no init, no multi*/
	    srl[0]->s_name, srl[1]->s_name,
	    srl[2]->s_name, x->x_gui.x_ldx, x->x_gui.x_ldy,
	    x->x_gui.x_fsf.x_font_style, x->x_gui.x_fontsize,
	    0xffffff & x->x_gui.x_bcol, -1/*no frontcolor*/, 0xffffff & x->x_gui.x_lcol);
    gfxstub_new(&x->x_gui.x_obj.ob_pd, x, buf);
}

static void my_canvas_get_pos(t_my_canvas *x)
{
    if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
    {
	x->x_at[0].a_w.w_float = x->x_gui.x_obj.te_xpos;
	x->x_at[1].a_w.w_float = x->x_gui.x_obj.te_ypos;
	pd_list(x->x_gui.x_snd->s_thing, &s_list, 2, x->x_at);
    }
}

static void my_canvas_dialog(t_my_canvas *x, t_symbol *s, int argc, t_atom *argv)
{
    t_symbol *srl[3];
    int a = (int)atom_getintarg(0, argc, argv);
    int w = (int)atom_getintarg(2, argc, argv);
    int h = (int)atom_getintarg(3, argc, argv);
    int sr_flags = iemgui_dialog(&x->x_gui, srl, argc, argv);

    x->x_gui.x_isa.x_loadinit = 0;
    if(a < 1)
	a = 1;
    x->x_gui.x_w = a;
    x->x_gui.x_h = x->x_gui.x_w;
    if(w < 1)
	w = 1;
    x->x_vis_w = w;
    if(h < 1)
	h = 1;
    x->x_vis_h = h;
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_CONFIG);
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
}

static void my_canvas_size(t_my_canvas *x, t_symbol *s, int ac, t_atom *av)
{
    int i = (int)atom_getintarg(0, ac, av);

    if(i < 1)
	i = 1;
    x->x_gui.x_w = i;
    x->x_gui.x_h = i;
    iemgui_size((void *)x, &x->x_gui);
}

static void my_canvas_delta(t_my_canvas *x, t_symbol *s, int ac, t_atom *av)
{iemgui_delta((void *)x, &x->x_gui, s, ac, av);}

static void my_canvas_pos(t_my_canvas *x, t_symbol *s, int ac, t_atom *av)
{iemgui_pos((void *)x, &x->x_gui, s, ac, av);}

static void my_canvas_vis_size(t_my_canvas *x, t_symbol *s, int ac, t_atom *av)
{
    int i;

    i = (int)atom_getintarg(0, ac, av);
    if(i < 1)
	i = 1;
    x->x_vis_w = i;
    if(ac > 1)
    {
        i = (int)atom_getintarg(1, ac, av);
        if(i < 1)
	    i = 1;
    }
    x->x_vis_h = i;
    if(glist_isvisible(x->x_gui.x_glist))
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
}

static void my_canvas_color(t_my_canvas *x, t_symbol *s, int ac, t_atom *av)
{iemgui_color((void *)x, &x->x_gui, s, ac, av);}

static void my_canvas_send(t_my_canvas *x, t_symbol *s)
{iemgui_send(x, &x->x_gui, s);}

static void my_canvas_receive(t_my_canvas *x, t_symbol *s)
{iemgui_receive(x, &x->x_gui, s);}

static void my_canvas_label(t_my_canvas *x, t_symbol *s)
{iemgui_label((void *)x, &x->x_gui, s);}

static void my_canvas_label_pos(t_my_canvas *x, t_symbol *s, int ac, t_atom *av)
{iemgui_label_pos((void *)x, &x->x_gui, s, ac, av);}

static void my_canvas_label_font(t_my_canvas *x, t_symbol *s, int ac, t_atom *av)
{iemgui_label_font((void *)x, &x->x_gui, s, ac, av);}

static void my_canvas_list(t_my_canvas *x, t_symbol *s, int ac, t_atom *av)
{
    int l=iemgui_list((void *)x, &x->x_gui, s, ac, av);

    /*if(l < 0)
    {
	post("error: my_canvas: no method for 'list'");
    }
    else */if(l > 0)
    {
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
	canvas_fixlinesfor(glist_getcanvas(x->x_gui.x_glist), (t_text*)x);
    }
}

static void *my_canvas_new(t_symbol *s, int argc, t_atom *argv)
{
    t_my_canvas *x = (t_my_canvas *)pd_new(my_canvas_class);
    int bflcol[]={-233017, -1, -66577};
    t_symbol *srl[3];
    int a=IEM_GUI_DEFAULTSIZE, w=100, h=60;
    int ldx=20, ldy=12, f=2, i=0;
    int fs=14, iinit=0, ifstyle=0;
    t_iem_init_symargs *init=(t_iem_init_symargs *)(&iinit);
    t_iem_fstyle_flags *fstyle=(t_iem_fstyle_flags *)(&ifstyle);
    char str[144];

    srl[0] = gensym("empty");
    srl[1] = gensym("empty");
    srl[2] = gensym("empty");

    if(((argc >= 10)&&(argc <= 13))
       &&IS_A_FLOAT(argv,0)&&IS_A_FLOAT(argv,1)&&IS_A_FLOAT(argv,2))
    {
	a = (int)atom_getintarg(0, argc, argv);
	w = (int)atom_getintarg(1, argc, argv);
	h = (int)atom_getintarg(2, argc, argv);
    }
    if((argc >= 12)&&(IS_A_SYMBOL(argv,3)||IS_A_FLOAT(argv,3))&&(IS_A_SYMBOL(argv,4)||IS_A_FLOAT(argv,4)))
    {
	i = 2;
	if(IS_A_SYMBOL(argv,3))
	    srl[0] = atom_getsymbolarg(3, argc, argv);
	else if(IS_A_FLOAT(argv,3))
	{
	    sprintf(str, "%d", (int)atom_getintarg(3, argc, argv));
	    srl[0] = gensym(str);
	}
	if(IS_A_SYMBOL(argv,4))
	    srl[1] = atom_getsymbolarg(4, argc, argv);
	else if(IS_A_FLOAT(argv,4))
	{
	    sprintf(str, "%d", (int)atom_getintarg(4, argc, argv));
	    srl[1] = gensym(str);
	}
    }
    else if((argc == 11)&&(IS_A_SYMBOL(argv,3)||IS_A_FLOAT(argv,3)))
    {
	i = 1;
	if(IS_A_SYMBOL(argv,3))
	    srl[1] = atom_getsymbolarg(3, argc, argv);
	else if(IS_A_FLOAT(argv,3))
	{
	    sprintf(str, "%d", (int)atom_getintarg(3, argc, argv));
	    srl[1] = gensym(str);
	}
    }

    if(((argc >= 10)&&(argc <= 13))
       &&(IS_A_SYMBOL(argv,i+3)||IS_A_FLOAT(argv,i+3))&&IS_A_FLOAT(argv,i+4)
       &&IS_A_FLOAT(argv,i+5)&&IS_A_FLOAT(argv,i+6)
       &&IS_A_FLOAT(argv,i+7)&&IS_A_FLOAT(argv,i+8)
       &&IS_A_FLOAT(argv,i+9))
    {
	if(IS_A_SYMBOL(argv,i+3))
	    srl[2] = atom_getsymbolarg(i+3, argc, argv);
	else if(IS_A_FLOAT(argv,i+3))
	{
	    sprintf(str, "%d", (int)atom_getintarg(i+3, argc, argv));
	    srl[2] = gensym(str);
	}
	ldx = (int)atom_getintarg(i+4, argc, argv);
	ldy = (int)atom_getintarg(i+5, argc, argv);
	ifstyle = (int)atom_getintarg(i+6, argc, argv);
	fs = (int)atom_getintarg(i+7, argc, argv);
	bflcol[0] = (int)atom_getintarg(i+8, argc, argv);
	bflcol[2] = (int)atom_getintarg(i+9, argc, argv);
    }
    if((argc == 13)&&IS_A_FLOAT(argv,i+10))
    {
	iinit = (int)(atom_getintarg(i+10, argc, argv));
    }
    x->x_gui.x_draw = (t_iemfunptr)my_canvas_draw;
    iinit &= IEM_INIT_ARGS_ALL;
    ifstyle &= IEM_FSTYLE_FLAGS_ALL;
    fstyle->x_snd_able = 1;
    fstyle->x_rcv_able = 1;
    x->x_gui.x_glist = (t_glist *)canvas_getcurrent();
    x->x_gui.x_isa = *init;
    if(!strcmp(srl[0]->s_name, "empty")) fstyle->x_snd_able = 0;
    if(!strcmp(srl[1]->s_name, "empty")) fstyle->x_rcv_able = 0;
    x->x_gui.x_unique_num = 0;
    if(a < 1)
	a = 1;
    x->x_gui.x_w = a;
    x->x_gui.x_h = x->x_gui.x_w;
    if(w < 1)
	w = 1;
    x->x_vis_w = w;
    if(h < 1)
	h = 1;
    x->x_vis_h = h;
    if(fstyle->x_font_style == 1) strcpy(x->x_gui.x_font, "helvetica");
    else if(fstyle->x_font_style == 2) strcpy(x->x_gui.x_font, "times");
    else { fstyle->x_font_style = 0;
	strcpy(x->x_gui.x_font, "courier"); }
    x->x_gui.x_fsf = *fstyle;
    iemgui_first_dollararg2sym(&x->x_gui, srl);
    if(x->x_gui.x_fsf.x_rcv_able) pd_bind(&x->x_gui.x_obj.ob_pd, srl[1]);
    x->x_gui.x_snd = srl[0];
    x->x_gui.x_rcv = srl[1];
    x->x_gui.x_lab = srl[2];
    x->x_gui.x_ldx = ldx;
    x->x_gui.x_ldy = ldy;
    if(fs < 4)
	fs = 4;
    x->x_gui.x_fontsize = fs;
    iemgui_all_colfromload(&x->x_gui, bflcol);
    x->x_at[0].a_type = A_FLOAT;
    x->x_at[1].a_type = A_FLOAT;
    iemgui_verify_snd_ne_rcv(&x->x_gui);
    return (x);
}

static void my_canvas_ff(t_my_canvas *x)
{
    if(x->x_gui.x_fsf.x_selected)
	pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
    if(x->x_gui.x_fsf.x_rcv_able)
	pd_unbind(&x->x_gui.x_obj.ob_pd, x->x_gui.x_rcv);
}

void my_canvas_setup(void)
{
    my_canvas_class = class_new(gensym("cnv"), (t_newmethod)my_canvas_new,
				(t_method)my_canvas_ff, sizeof(t_my_canvas), CLASS_NOINLET, A_GIMME, 0);
    class_addcreator((t_newmethod)my_canvas_new, gensym("my_canvas"), A_GIMME, 0);
    class_addlist(my_canvas_class, my_canvas_list);
    class_addmethod(my_canvas_class, (t_method)my_canvas_dialog, gensym("dialog"), A_GIMME, 0);
    class_addmethod(my_canvas_class, (t_method)my_canvas_size, gensym("size"), A_GIMME, 0);
    class_addmethod(my_canvas_class, (t_method)my_canvas_delta, gensym("delta"), A_GIMME, 0);
    class_addmethod(my_canvas_class, (t_method)my_canvas_pos, gensym("pos"), A_GIMME, 0);
    class_addmethod(my_canvas_class, (t_method)my_canvas_vis_size, gensym("vis_size"), A_GIMME, 0);
    class_addmethod(my_canvas_class, (t_method)my_canvas_color, gensym("color"), A_GIMME, 0);
    class_addmethod(my_canvas_class, (t_method)my_canvas_send, gensym("send"), A_DEFSYM, 0);
    class_addmethod(my_canvas_class, (t_method)my_canvas_receive, gensym("receive"), A_DEFSYM, 0);
    class_addmethod(my_canvas_class, (t_method)my_canvas_label, gensym("label"), A_DEFSYM, 0);
    class_addmethod(my_canvas_class, (t_method)my_canvas_label_pos, gensym("label_pos"), A_GIMME, 0);
    class_addmethod(my_canvas_class, (t_method)my_canvas_label_font, gensym("label_font"), A_GIMME, 0);
    class_addmethod(my_canvas_class, (t_method)my_canvas_get_pos, gensym("get_pos"), 0);

    if(!iemgui_key_sym)
	iemgui_key_sym = gensym("#keyname");
    my_canvas_widgetbehavior.w_getrectfn = my_canvas_getrect;
    my_canvas_widgetbehavior.w_displacefn = iemgui_displace;
    my_canvas_widgetbehavior.w_selectfn = iemgui_select;
    my_canvas_widgetbehavior.w_activatefn = NULL;
    my_canvas_widgetbehavior.w_deletefn = iemgui_delete;
    my_canvas_widgetbehavior.w_visfn = iemgui_vis;
    my_canvas_widgetbehavior.w_clickfn = NULL;
    my_canvas_widgetbehavior.w_propertiesfn = my_canvas_properties;
    my_canvas_widgetbehavior.w_savefn = my_canvas_save;
    class_setwidget(my_canvas_class, &my_canvas_widgetbehavior);
    class_sethelpsymbol(my_canvas_class, gensym("my_canvas"));
}








/* ------------ vsl gui-vertical  slider ----------------------- */

t_widgetbehavior vslider_widgetbehavior;
static t_class *vslider_class;

/* widget helper functions */

static void vslider_draw_update(t_vslider *x, t_glist *glist)
{
    if (glist_isvisible(glist))
    {
	int r = x->x_gui.x_obj.te_ypos + x->x_gui.x_h - (x->x_val + 50)/100;
	sys_vgui(".x%x.c coords %xKNOB %d %d %d %d\n",
		 glist_getcanvas(glist), x, x->x_gui.x_obj.te_xpos+1, r,
		 x->x_gui.x_obj.te_xpos + x->x_gui.x_w, r);
    }
}

static void vslider_draw_new(t_vslider *x, t_glist *glist)
{
    int r = x->x_gui.x_obj.te_ypos + x->x_gui.x_h - (x->x_val + 50)/100;
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c create rectangle %d %d %d %d -fill #%6.6x -tags %xBASE\n",
	     canvas, x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos-2,
	     x->x_gui.x_obj.te_xpos + x->x_gui.x_w, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+3,
	     x->x_gui.x_bcol, x);
    sys_vgui(".x%x.c create line %d %d %d %d -width 3 -fill #%6.6x -tags %xKNOB\n",
	     canvas, x->x_gui.x_obj.te_xpos+1, r,
	     x->x_gui.x_obj.te_xpos + x->x_gui.x_w, r, x->x_gui.x_fcol, x);
    sys_vgui(".x%x.c create text %d %d -text {%s} -anchor w \
	     -font {%s %d bold} -fill #%6.6x -tags %xLABEL\n",
	     canvas, x->x_gui.x_obj.te_xpos+x->x_gui.x_ldx, x->x_gui.x_obj.te_ypos+x->x_gui.x_ldy,
	     strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"",
	     x->x_gui.x_font, x->x_gui.x_fontsize, x->x_gui.x_lcol, x);
    if(!x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xOUT%d\n",
	     canvas,
	     x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+2,
	     x->x_gui.x_obj.te_xpos+7, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+3,
	     x, 0);
    if(!x->x_gui.x_fsf.x_rcv_able)
	sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xIN%d\n",
	     canvas,
	     x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos-2,
	     x->x_gui.x_obj.te_xpos+7, x->x_gui.x_obj.te_ypos-1,
	     x, 0);
}

static void vslider_draw_move(t_vslider *x, t_glist *glist)
{
    int r = x->x_gui.x_obj.te_ypos + x->x_gui.x_h - (x->x_val + 50)/100;
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c coords %xBASE %d %d %d %d\n",
	     canvas, x,
	     x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos-2,
	     x->x_gui.x_obj.te_xpos + x->x_gui.x_w, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+3);
    sys_vgui(".x%x.c coords %xKNOB %d %d %d %d\n",
	     canvas, x, x->x_gui.x_obj.te_xpos+1, r,
	     x->x_gui.x_obj.te_xpos + x->x_gui.x_w, r);
    sys_vgui(".x%x.c coords %xLABEL %d %d\n",
	     canvas, x, x->x_gui.x_obj.te_xpos+x->x_gui.x_ldx, x->x_gui.x_obj.te_ypos+x->x_gui.x_ldy);
    if(!x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c coords %xOUT%d %d %d %d %d\n",
	     canvas, x, 0,
	     x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+2,
	     x->x_gui.x_obj.te_xpos+7, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+3);
    if(!x->x_gui.x_fsf.x_rcv_able)
	sys_vgui(".x%x.c coords %xIN%d %d %d %d %d\n",
	     canvas, x, 0,
	     x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos-2,
	     x->x_gui.x_obj.te_xpos+7, x->x_gui.x_obj.te_ypos-1);
}

static void vslider_draw_erase(t_vslider* x,t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c delete %xBASE\n", canvas, x);
    sys_vgui(".x%x.c delete %xKNOB\n", canvas, x);
    sys_vgui(".x%x.c delete %xLABEL\n", canvas, x);
    if(!x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c delete %xOUT%d\n", canvas, x, 0);
    if(!x->x_gui.x_fsf.x_rcv_able)
	sys_vgui(".x%x.c delete %xIN%d\n", canvas, x, 0);
}

static void vslider_draw_config(t_vslider* x,t_glist* glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c itemconfigure %xLABEL -font {%s %d bold} -fill #%6.6x -text {%s} \n",
	     canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize,
	     x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
	     strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"");
    sys_vgui(".x%x.c itemconfigure %xKNOB -fill #%6.6x\n", canvas,
	     x, x->x_gui.x_fcol);
    sys_vgui(".x%x.c itemconfigure %xBASE -fill #%6.6x\n", canvas,
	     x, x->x_gui.x_bcol);
}

static void vslider_draw_io(t_vslider* x,t_glist* glist, int old_snd_rcv_flags)
{
    t_canvas *canvas=glist_getcanvas(glist);

    if((old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && !x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xOUT%d\n",
	     canvas,
	     x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+2,
	     x->x_gui.x_obj.te_xpos+7, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+3,
	     x, 0);
    if(!(old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && x->x_gui.x_fsf.x_snd_able)
        sys_vgui(".x%x.c delete %xOUT%d\n", canvas, x, 0);
    if((old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && !x->x_gui.x_fsf.x_rcv_able)
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xIN%d\n",
	     canvas,
	     x->x_gui.x_obj.te_xpos, x->x_gui.x_obj.te_ypos-2,
	     x->x_gui.x_obj.te_xpos+7, x->x_gui.x_obj.te_ypos-1,
	     x, 0);
    if(!(old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && x->x_gui.x_fsf.x_rcv_able)
        sys_vgui(".x%x.c delete %xIN%d\n", canvas, x, 0);
}

static void vslider_draw_select(t_vslider *x, t_glist *glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    if(x->x_gui.x_fsf.x_selected)
    {
	pd_bind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
	sys_vgui(".x%x.c itemconfigure %xBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
	sys_vgui(".x%x.c itemconfigure %xLABEL -fill #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
    }
    else
    {
	pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
	sys_vgui(".x%x.c itemconfigure %xBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
	sys_vgui(".x%x.c itemconfigure %xLABEL -fill #%6.6x\n", canvas, x, x->x_gui.x_lcol);
    }
}

void vslider_draw(t_vslider *x, t_glist *glist, int mode)
{
    if(mode == IEM_GUI_DRAW_MODE_UPDATE)
	vslider_draw_update(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_MOVE)
	vslider_draw_move(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_NEW)
	vslider_draw_new(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_SELECT)
	vslider_draw_select(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_ERASE)
	vslider_draw_erase(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_CONFIG)
	vslider_draw_config(x, glist);
    else if(mode >= IEM_GUI_DRAW_MODE_IO)
	vslider_draw_io(x, glist, mode - IEM_GUI_DRAW_MODE_IO);
}

/* ------------------------ vsl widgetbehaviour----------------------------- */


static void vslider_getrect(t_gobj *z, t_glist *owner,
			    int *xp1, int *yp1, int *xp2, int *yp2)
{
    t_vslider* x = (t_vslider*)z;

    *xp1 = x->x_gui.x_obj.te_xpos;
    *yp1 = x->x_gui.x_obj.te_ypos - 2;
    *xp2 = x->x_gui.x_obj.te_xpos + x->x_gui.x_w;
    *yp2 = x->x_gui.x_obj.te_ypos + x->x_gui.x_h + 3;
}

static void vslider_save(t_gobj *z, t_binbuf *b)
{
    t_vslider *x = (t_vslider *)z;
    int bflcol[3], *ip1, *ip2;
    t_symbol *srl[3];

    iemgui_save(&x->x_gui, srl, bflcol);
    ip1 = (int *)(&x->x_gui.x_isa);
    ip2 = (int *)(&x->x_gui.x_fsf);
    binbuf_addv(b, "ssiisiiffiisssiiiiiiiii", gensym("#X"),gensym("obj"),
		(t_int)x->x_gui.x_obj.te_xpos, (t_int)x->x_gui.x_obj.te_ypos,
		gensym("vsl"), x->x_gui.x_w, x->x_gui.x_h,
		(float)x->x_min, (float)x->x_max,
		x->x_lin0_log1, (*ip1)&IEM_INIT_ARGS_ALL,
		srl[0], srl[1], srl[2],
		x->x_gui.x_ldx, x->x_gui.x_ldy,
		(*ip2)&IEM_FSTYLE_FLAGS_ALL, x->x_gui.x_fontsize,
		bflcol[0], bflcol[1], bflcol[2],
		x->x_val, x->x_steady);
    binbuf_addv(b, ";");
}

void vslider_check_height(t_vslider *x, int h)
{
    if(h < IEM_SL_MINSIZE)
	h = IEM_SL_MINSIZE;
    x->x_gui.x_h = h;
    if(x->x_val > (x->x_gui.x_h*100 - 100))
    {
	x->x_pos = x->x_gui.x_h*100 - 100;
	x->x_val = x->x_pos;
    }
    if(x->x_lin0_log1)
	x->x_k = log(x->x_max/x->x_min)/(double)(x->x_gui.x_h - 1);
    else
	x->x_k = (x->x_max - x->x_min)/(double)(x->x_gui.x_h - 1);
}

void vslider_check_minmax(t_vslider *x, double min, double max)
{
    if(x->x_lin0_log1)
    {
	if((min == 0.0)&&(max == 0.0))
	    max = 1.0;
	if(max > 0.0)
	{
	    if(min <= 0.0)
		min = 0.01*max;
	}
	else
	{
	    if(min > 0.0)
		max = 0.01*min;
	}
    }
    x->x_min = min;
    x->x_max = max;
    if(x->x_lin0_log1)
	x->x_k = log(x->x_max/x->x_min)/(double)(x->x_gui.x_h - 1);
    else
	x->x_k = (x->x_max - x->x_min)/(double)(x->x_gui.x_h - 1);
}

static void vslider_properties(t_gobj *z, t_glist *owner)
{
    t_vslider *x = (t_vslider *)z;
    char buf[800];
    t_symbol *srl[3];
    int steady=-23;

    iemgui_properties(&x->x_gui, srl);
    if(x->x_steady)
	steady = -22;

    sprintf(buf, "pdtk_iemgui_dialog %%s VSLIDER \
	    --------dimensions(pix)(pix):-------- %d %d width: %d %d height: \
	    -----------output-range:----------- %g bottom: %g top: %d \
	    %d lin log %d %d \
	    %s %s \
	    %s %d %d \
	    %d %d \
	    %d %d %d\n",
	    x->x_gui.x_w, IEM_GUI_MINSIZE, x->x_gui.x_h, IEM_SL_MINSIZE,
	    x->x_min, x->x_max, 0,/*no_schedule*/
	    x->x_lin0_log1, x->x_gui.x_isa.x_loadinit, steady,/*no multi, but iem-characteristic*/
	    srl[0]->s_name, srl[1]->s_name,
	    srl[2]->s_name, x->x_gui.x_ldx, x->x_gui.x_ldy,
	    x->x_gui.x_fsf.x_font_style, x->x_gui.x_fontsize,
	    0xffffff & x->x_gui.x_bcol, 0xffffff & x->x_gui.x_fcol, 0xffffff & x->x_gui.x_lcol);
    gfxstub_new(&x->x_gui.x_obj.ob_pd, x, buf);
}

static void vslider_bang(t_vslider *x)
{
    double out;

    if(x->x_lin0_log1)
	out = x->x_min*exp(x->x_k*(double)(x->x_val)*0.01);
    else
	out = (double)(x->x_val)*0.01*x->x_k + x->x_min;
    if((out < 1.0e-10)&&(out > -1.0e-10))
	out = 0.0;

    outlet_float(x->x_gui.x_obj.ob_outlet, out);
    if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
	pd_float(x->x_gui.x_snd->s_thing, out);
}

static void vslider_dialog(t_vslider *x, t_symbol *s, int argc, t_atom *argv)
{
    t_symbol *srl[3];
    int w = (int)atom_getintarg(0, argc, argv);
    int h = (int)atom_getintarg(1, argc, argv);
    double min = (double)atom_getfloatarg(2, argc, argv);
    double max = (double)atom_getfloatarg(3, argc, argv);
    int lilo = (int)atom_getintarg(4, argc, argv);
    int steady = (int)atom_getintarg(6, argc, argv);
    int sr_flags;

    if(lilo != 0) lilo = 1;
    x->x_lin0_log1 = lilo;
    if(steady == 3)
	x->x_steady = 0;
    else
	x->x_steady = 1;
    sr_flags = iemgui_dialog(&x->x_gui, srl, argc, argv);
    x->x_gui.x_w = iemgui_clip_size(w);
    vslider_check_height(x, h);
    vslider_check_minmax(x, min, max);
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_CONFIG);
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_IO + sr_flags);
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
    canvas_fixlinesfor(glist_getcanvas(x->x_gui.x_glist), (t_text*)x);
}

static void vslider_motion(t_vslider *x, t_floatarg dx, t_floatarg dy)
{
    int old = x->x_val;

    if(x->x_gui.x_fsf.x_finemoved)
	x->x_pos -= (int)dy;
    else
	x->x_pos -= 100*(int)dy;
    x->x_val = x->x_pos;
    if(x->x_val > (100*x->x_gui.x_h - 100))
    {
	x->x_val = 100*x->x_gui.x_h - 100;
	x->x_pos += 50;
	x->x_pos -= x->x_pos%100;
    }
    if(x->x_val < 0)
    {
	x->x_val = 0;
	x->x_pos -= 50;
	x->x_pos -= x->x_pos%100;
    }
    if(old != x->x_val)
    {
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
	vslider_bang(x);
    }
}

static void vslider_click(t_vslider *x, t_floatarg xpos, t_floatarg ypos,
			  t_floatarg shift, t_floatarg ctrl, t_floatarg alt)
{
    if(!x->x_steady)
	x->x_val = (int)(100.0 * (x->x_gui.x_h + x->x_gui.x_obj.te_ypos - ypos));
    if(x->x_val > (100*x->x_gui.x_h - 100))
	x->x_val = 100*x->x_gui.x_h - 100;
    if(x->x_val < 0)
	x->x_val = 0;
    x->x_pos = x->x_val;
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
    vslider_bang(x);
    glist_grab(x->x_gui.x_glist, &x->x_gui.x_obj.te_g, (t_glistmotionfn)vslider_motion,
	       0, xpos, ypos);
}

static int vslider_newclick(t_gobj *z, struct _glist *glist,
			    int xpix, int ypix, int shift, int alt, int dbl, int doit)
{
    t_vslider* x = (t_vslider *)z;

    if(doit)
    {
	vslider_click( x, (t_floatarg)xpix, (t_floatarg)ypix, (t_floatarg)shift,
		       0, (t_floatarg)alt);
	if(shift)
	    x->x_gui.x_fsf.x_finemoved = 1;
	else
	    x->x_gui.x_fsf.x_finemoved = 0;
    }
    return (1);
}

static void vslider_set(t_vslider *x, t_floatarg f)
{
    double g;

    if(f > x->x_max)
	f = x->x_max;
    if(f < x->x_min)
	f = x->x_min;
    if(x->x_lin0_log1)
	g = log(f/x->x_min)/x->x_k;
    else
	g = (f - x->x_min) / x->x_k;
    x->x_val = (int)(100.0*g + 0.49999);
    x->x_pos = x->x_val;
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
}

static void vslider_float(t_vslider *x, t_floatarg f)
{
    vslider_set(x, f);
    if(x->x_gui.x_fsf.x_put_in2out)
	vslider_bang(x);
}

static void vslider_size(t_vslider *x, t_symbol *s, int ac, t_atom *av)
{
    x->x_gui.x_w = iemgui_clip_size((int)atom_getintarg(0, ac, av));
    if(ac > 1)
	vslider_check_height(x, (int)atom_getintarg(1, ac, av));
    iemgui_size((void *)x, &x->x_gui);
}

static void vslider_delta(t_vslider *x, t_symbol *s, int ac, t_atom *av)
{iemgui_delta((void *)x, &x->x_gui, s, ac, av);}

static void vslider_pos(t_vslider *x, t_symbol *s, int ac, t_atom *av)
{iemgui_pos((void *)x, &x->x_gui, s, ac, av);}

static void vslider_range(t_vslider *x, t_symbol *s, int ac, t_atom *av)
{
    vslider_check_minmax(x, (double)atom_getfloatarg(0, ac, av),
			 (double)atom_getfloatarg(1, ac, av));
}

static void vslider_color(t_vslider *x, t_symbol *s, int ac, t_atom *av)
{iemgui_color((void *)x, &x->x_gui, s, ac, av);}

static void vslider_send(t_vslider *x, t_symbol *s)
{iemgui_send(x, &x->x_gui, s);}

static void vslider_receive(t_vslider *x, t_symbol *s)
{iemgui_receive(x, &x->x_gui, s);}

static void vslider_label(t_vslider *x, t_symbol *s)
{iemgui_label((void *)x, &x->x_gui, s);}

static void vslider_label_pos(t_vslider *x, t_symbol *s, int ac, t_atom *av)
{iemgui_label_pos((void *)x, &x->x_gui, s, ac, av);}

static void vslider_label_font(t_vslider *x, t_symbol *s, int ac, t_atom *av)
{iemgui_label_font((void *)x, &x->x_gui, s, ac, av);}

static void vslider_log(t_vslider *x)
{
    x->x_lin0_log1 = 1;
    vslider_check_minmax(x, x->x_min, x->x_max);
}

static void vslider_lin(t_vslider *x)
{
    x->x_lin0_log1 = 0;
    x->x_k = (x->x_max - x->x_min)/(double)(x->x_gui.x_h - 1);
}

static void vslider_init(t_vslider *x, t_floatarg f)
{
    x->x_gui.x_isa.x_loadinit = (f==0.0)?0:1;
}

static void vslider_steady(t_vslider *x, t_floatarg f)
{
    x->x_steady = (f==0.0)?0:1;
}

static void vslider_loadbang(t_vslider *x)
{
    if(!sys_noloadbang && x->x_gui.x_isa.x_loadinit)
    {
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
	vslider_bang(x);
    }
}

static void vslider_list(t_vslider *x, t_symbol *s, int ac, t_atom *av)
{
    int l=iemgui_list((void *)x, &x->x_gui, s, ac, av);

    if(l < 0)
    {
	if(IS_A_FLOAT(av,0))
	    vslider_float(x, atom_getfloatarg(0, ac, av));
    }
    else if(l > 0)
    {
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
	canvas_fixlinesfor(glist_getcanvas(x->x_gui.x_glist), (t_text*)x);
    }
}

static void *vslider_new(t_symbol *s, int argc, t_atom *argv)
{
    t_vslider *x = (t_vslider *)pd_new(vslider_class);
    int bflcol[]={-262144, -1, -1};
    t_symbol *srl[3];
    int w=IEM_GUI_DEFAULTSIZE, h=IEM_SL_DEFAULTSIZE;
    int lilo=0, f=0, ldx=20, ldy=8;
    int fs=8, iinit=0, ifstyle=0, v=0, steady=1;
    double min=0.0, max=(double)(IEM_SL_DEFAULTSIZE-1);
    t_iem_init_symargs *init=(t_iem_init_symargs *)(&iinit);
    t_iem_fstyle_flags *fstyle=(t_iem_fstyle_flags *)(&ifstyle);
    char str[144];

    srl[0] = gensym("empty");
    srl[1] = gensym("empty");
    srl[2] = gensym("empty");


    if(((argc == 17)||(argc == 18))&&IS_A_FLOAT(argv,0)&&IS_A_FLOAT(argv,1)
       &&IS_A_FLOAT(argv,2)&&IS_A_FLOAT(argv,3)
       &&IS_A_FLOAT(argv,4)&&IS_A_FLOAT(argv,5)
       &&(IS_A_SYMBOL(argv,6)||IS_A_FLOAT(argv,6))
       &&(IS_A_SYMBOL(argv,7)||IS_A_FLOAT(argv,7))
       &&(IS_A_SYMBOL(argv,8)||IS_A_FLOAT(argv,8))
       &&IS_A_FLOAT(argv,9)&&IS_A_FLOAT(argv,10)
       &&IS_A_FLOAT(argv,11)&&IS_A_FLOAT(argv,12)&&IS_A_FLOAT(argv,13)
       &&IS_A_FLOAT(argv,14)&&IS_A_FLOAT(argv,15)&&IS_A_FLOAT(argv,16))
    {
	w = (int)atom_getintarg(0, argc, argv);
	h = (int)atom_getintarg(1, argc, argv);
	min = (double)atom_getfloatarg(2, argc, argv);
	max = (double)atom_getfloatarg(3, argc, argv);
	lilo = (int)atom_getintarg(4, argc, argv);
	iinit = (int)atom_getintarg(5, argc, argv);
	srl[0] = atom_getsymbolarg(6, argc, argv);
	srl[1] = atom_getsymbolarg(7, argc, argv);
	srl[2] = atom_getsymbolarg(8, argc, argv);
	if(IS_A_SYMBOL(argv,6))
	    srl[0] = atom_getsymbolarg(6, argc, argv);
	else if(IS_A_FLOAT(argv,6))
	{
	    sprintf(str, "%d", (int)atom_getintarg(6, argc, argv));
	    srl[0] = gensym(str);
	}
	if(IS_A_SYMBOL(argv,7))
	    srl[1] = atom_getsymbolarg(7, argc, argv);
	else if(IS_A_FLOAT(argv,7))
	{
	    sprintf(str, "%d", (int)atom_getintarg(7, argc, argv));
	    srl[1] = gensym(str);
	}
	if(IS_A_SYMBOL(argv,8))
	    srl[2] = atom_getsymbolarg(8, argc, argv);
	else if(IS_A_FLOAT(argv,8))
	{
	    sprintf(str, "%d", (int)atom_getintarg(8, argc, argv));
	    srl[2] = gensym(str);
	}
	ldx = (int)atom_getintarg(9, argc, argv);
	ldy = (int)atom_getintarg(10, argc, argv);
	ifstyle = (int)atom_getintarg(11, argc, argv);
	fs = (int)atom_getintarg(12, argc, argv);
	bflcol[0] = (int)atom_getintarg(13, argc, argv);
	bflcol[1] = (int)atom_getintarg(14, argc, argv);
	bflcol[2] = (int)atom_getintarg(15, argc, argv);
	v = (int)atom_getintarg(16, argc, argv);
    }
    if((argc == 18)&&IS_A_FLOAT(argv,17))
	steady = (int)atom_getintarg(17, argc, argv);
    x->x_gui.x_draw = (t_iemfunptr)vslider_draw;
    iinit &= IEM_INIT_ARGS_ALL;
    ifstyle &= IEM_FSTYLE_FLAGS_ALL;
    fstyle->x_snd_able = 1;
    fstyle->x_rcv_able = 1;
    x->x_gui.x_glist = (t_glist *)canvas_getcurrent();
    x->x_gui.x_isa = *init;
    if(x->x_gui.x_isa.x_loadinit)
	x->x_val = v;
    else
	x->x_val = 0;
    x->x_pos = x->x_val;
    if(lilo != 0) lilo = 1;
    x->x_lin0_log1 = lilo;
    if(steady != 0) steady = 1;
    x->x_steady = steady;
    if(!strcmp(srl[0]->s_name, "empty")) fstyle->x_snd_able = 0;
    if(!strcmp(srl[1]->s_name, "empty")) fstyle->x_rcv_able = 0;
    x->x_gui.x_unique_num = 0;
    if(fstyle->x_font_style == 1) strcpy(x->x_gui.x_font, "helvetica");
    else if(fstyle->x_font_style == 2) strcpy(x->x_gui.x_font, "times");
    else { fstyle->x_font_style = 0;
	strcpy(x->x_gui.x_font, "courier"); }
    x->x_gui.x_fsf = *fstyle;
    iemgui_first_dollararg2sym(&x->x_gui, srl);
    if(x->x_gui.x_fsf.x_rcv_able) pd_bind(&x->x_gui.x_obj.ob_pd, srl[1]);
    x->x_gui.x_snd = srl[0];
    x->x_gui.x_rcv = srl[1];
    x->x_gui.x_lab = srl[2];
    x->x_gui.x_ldx = ldx;
    x->x_gui.x_ldy = ldy;
    if(fs < 4)
	fs = 4;
    x->x_gui.x_fontsize = fs;
    x->x_gui.x_w = iemgui_clip_size(w);
    vslider_check_height(x, h);
    vslider_check_minmax(x, min, max);
    iemgui_all_colfromload(&x->x_gui, bflcol);
    iemgui_verify_snd_ne_rcv(&x->x_gui);
    outlet_new(&x->x_gui.x_obj, &s_float);
    return (x);
}

static void vslider_free(t_vslider *x)
{
    if(x->x_gui.x_fsf.x_selected)
	pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
    if(x->x_gui.x_fsf.x_rcv_able)
	pd_unbind(&x->x_gui.x_obj.ob_pd, x->x_gui.x_rcv);
}

void vslider_setup(void)
{
    vslider_class = class_new(gensym("vsl"), (t_newmethod)vslider_new,
			      (t_method)vslider_free, sizeof(t_vslider), 0, A_GIMME, 0);
    class_addcreator((t_newmethod)vslider_new, gensym("vslider"), A_GIMME, 0);
    class_addbang(vslider_class,vslider_bang);
    class_addfloat(vslider_class,vslider_float);
    class_addlist(vslider_class, vslider_list);
    class_addmethod(vslider_class, (t_method)vslider_click, gensym("click"),
		    A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
    class_addmethod(vslider_class, (t_method)vslider_motion, gensym("motion"),
		    A_FLOAT, A_FLOAT, 0);
    class_addmethod(vslider_class, (t_method)vslider_dialog, gensym("dialog"),
		    A_GIMME, 0);
    class_addmethod(vslider_class, (t_method)vslider_loadbang, gensym("loadbang"), 0);
    class_addmethod(vslider_class, (t_method)vslider_set, gensym("set"), A_FLOAT, 0);
    class_addmethod(vslider_class, (t_method)vslider_size, gensym("size"), A_GIMME, 0);
    class_addmethod(vslider_class, (t_method)vslider_delta, gensym("delta"), A_GIMME, 0);
    class_addmethod(vslider_class, (t_method)vslider_pos, gensym("pos"), A_GIMME, 0);
    class_addmethod(vslider_class, (t_method)vslider_range, gensym("range"), A_GIMME, 0);
    class_addmethod(vslider_class, (t_method)vslider_color, gensym("color"), A_GIMME, 0);
    class_addmethod(vslider_class, (t_method)vslider_send, gensym("send"), A_DEFSYM, 0);
    class_addmethod(vslider_class, (t_method)vslider_receive, gensym("receive"), A_DEFSYM, 0);
    class_addmethod(vslider_class, (t_method)vslider_label, gensym("label"), A_DEFSYM, 0);
    class_addmethod(vslider_class, (t_method)vslider_label_pos, gensym("label_pos"), A_GIMME, 0);
    class_addmethod(vslider_class, (t_method)vslider_label_font, gensym("label_font"), A_GIMME, 0);
    class_addmethod(vslider_class, (t_method)vslider_log, gensym("log"), 0);
    class_addmethod(vslider_class, (t_method)vslider_lin, gensym("lin"), 0);
    class_addmethod(vslider_class, (t_method)vslider_init, gensym("init"), A_FLOAT, 0);
    class_addmethod(vslider_class, (t_method)vslider_steady, gensym("steady"), A_FLOAT, 0);
    if(!iemgui_key_sym)
	iemgui_key_sym = gensym("#keyname");
    vslider_widgetbehavior.w_getrectfn =    vslider_getrect;
    vslider_widgetbehavior.w_displacefn =   iemgui_displace;
    vslider_widgetbehavior.w_selectfn =     iemgui_select;
    vslider_widgetbehavior.w_activatefn =   NULL;
    vslider_widgetbehavior.w_deletefn =     iemgui_delete;
    vslider_widgetbehavior.w_visfn =        iemgui_vis;
    vslider_widgetbehavior.w_clickfn =      vslider_newclick;
    vslider_widgetbehavior.w_propertiesfn = vslider_properties;;
    vslider_widgetbehavior.w_savefn =       vslider_save;
    class_setwidget(vslider_class, &vslider_widgetbehavior);
    class_sethelpsymbol(vslider_class, gensym("vslider"));
}







/* ----- vu  gui-peak- & rms- vu-meter-display ---------- */

t_widgetbehavior vu_widgetbehavior;
static t_class *vu_class;

/* widget helper functions */

static void vu_update_rms(t_vu *x, t_glist *glist)
{
    if(glist_isvisible(glist))
    {
	int w4=x->x_gui.x_w/4, off=x->x_gui.x_obj.te_ypos-1;
	int quad1=x->x_gui.x_obj.te_xpos+w4+1, quad3=x->x_gui.x_obj.te_xpos+x->x_gui.x_w-w4-1;

	sys_vgui(".x%x.c coords %xRCOVER %d %d %d %d\n",
		 glist_getcanvas(glist), x, quad1, off, quad3,
		 off + (x->x_led_size+1)*(IEM_VU_STEPS-x->x_rms));
    }
}

static void vu_update_peak(t_vu *x, t_glist *glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    if(glist_isvisible(glist))
    {
	if(x->x_peak)
	{
	    int i=iemgui_vu_col[x->x_peak];
	    int j=x->x_gui.x_obj.te_ypos + (x->x_led_size+1)*(IEM_VU_STEPS+1-x->x_peak)
		- (x->x_led_size+1)/2;

	    sys_vgui(".x%x.c coords %xPLED %d %d %d %d\n", canvas, x,
		     x->x_gui.x_obj.te_xpos, j,
		     x->x_gui.x_obj.te_xpos+x->x_gui.x_w+1, j);
	    sys_vgui(".x%x.c itemconfigure %xPLED -fill #%6.6x\n", canvas, x,
		     iemgui_color_hex[i]);
	}
	else
	{
	    int mid=x->x_gui.x_obj.te_xpos+x->x_gui.x_w/2;

	    sys_vgui(".x%x.c itemconfigure %xPLED -fill #%6.6x\n",
		     canvas, x, x->x_gui.x_bcol);
	    sys_vgui(".x%x.c coords %xPLED %d %d %d %d\n",
		     canvas, x, mid, x->x_gui.x_obj.te_ypos+20,
		     mid, x->x_gui.x_obj.te_ypos+20);
	}
    }
}

static void vu_draw_new(t_vu *x, t_glist *glist)
{
    t_canvas *canvas=glist_getcanvas(glist);


    int w4=x->x_gui.x_w/4, mid=x->x_gui.x_obj.te_xpos+x->x_gui.x_w/2, quad1=x->x_gui.x_obj.te_xpos+w4+1;
    int quad3=x->x_gui.x_obj.te_xpos+x->x_gui.x_w-w4, end=x->x_gui.x_obj.te_xpos+x->x_gui.x_w+4;
    int k1=x->x_led_size+1, k2=IEM_VU_STEPS+1, k3=k1/2;
    int led_col, yyy, i, k4=x->x_gui.x_obj.te_ypos-k3;

    sys_vgui(".x%x.c create rectangle %d %d %d %d -fill #%6.6x -tags %xBASE\n",
	     canvas, x->x_gui.x_obj.te_xpos-1, x->x_gui.x_obj.te_ypos-2,
	     x->x_gui.x_obj.te_xpos+x->x_gui.x_w+1,x->x_gui.x_obj.te_ypos+x->x_gui.x_h+2, x->x_gui.x_bcol, x);
    for(i=1; i<=IEM_VU_STEPS; i++)
    {
	led_col = iemgui_vu_col[i];
	yyy = k4 + k1*(k2-i);
	sys_vgui(".x%x.c create line %d %d %d %d -width %d -fill #%6.6x -tags %xRLED%d\n",
		 canvas, quad1, yyy, quad3, yyy, x->x_led_size, iemgui_color_hex[led_col], x, i);
	if(((i+2)&3) && (x->x_scale))
	    sys_vgui(".x%x.c create text %d %d -text {%s} -anchor w \
		     -font {%s %d bold} -fill #%6.6x -tags %xSCALE%d\n",
		     canvas, end, yyy+k3, iemgui_vu_scale_str[i], x->x_gui.x_font, x->x_gui.x_fontsize,
		     x->x_gui.x_lcol, x, i);
    }
    if(x->x_scale)
    {
	i=IEM_VU_STEPS+1;
	yyy = k4 + k1*(k2-i);
	sys_vgui(".x%x.c create text %d %d -text {%s} -anchor w \
		 -font {%s %d bold} -fill #%6.6x -tags %xSCALE%d\n",
		 canvas, end, yyy+k3, iemgui_vu_scale_str[i], x->x_gui.x_font, x->x_gui.x_fontsize,
		 x->x_gui.x_lcol, x, i);
    }
    sys_vgui(".x%x.c create rectangle %d %d %d %d -fill #%6.6x -outline #%6.6x -tags %xRCOVER\n",
	     canvas, quad1, x->x_gui.x_obj.te_ypos-1, quad3-1,
	     x->x_gui.x_obj.te_ypos-1 + k1*IEM_VU_STEPS, x->x_gui.x_bcol, x->x_gui.x_bcol, x);
    sys_vgui(".x%x.c create line %d %d %d %d -width %d -fill #%6.6x -tags %xPLED\n",
	     canvas, mid, x->x_gui.x_obj.te_ypos+10,
	     mid, x->x_gui.x_obj.te_ypos+10, x->x_led_size, x->x_gui.x_bcol, x);
    sys_vgui(".x%x.c create text %d %d -text {%s} -anchor w \
	     -font {%s %d bold} -fill #%6.6x -tags %xLABEL\n",
	     canvas, x->x_gui.x_obj.te_xpos+x->x_gui.x_ldx, x->x_gui.x_obj.te_ypos+x->x_gui.x_ldy,
	     strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"",
	     x->x_gui.x_font, x->x_gui.x_fontsize, x->x_gui.x_lcol, x);
    if(!x->x_gui.x_fsf.x_snd_able)
    {
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xOUT%d\n",
	     canvas,
	     x->x_gui.x_obj.te_xpos-1, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+1,
	     x->x_gui.x_obj.te_xpos + IOWIDTH-1, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+2,
	     x, 0);
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xOUT%d\n",
	     canvas,
	     x->x_gui.x_obj.te_xpos+x->x_gui.x_w+1-IOWIDTH, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+1,
	     x->x_gui.x_obj.te_xpos+x->x_gui.x_w+1, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+2,
	     x, 1);
    }
    if(!x->x_gui.x_fsf.x_rcv_able)
    {
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xIN%d\n",
	     canvas,
	     x->x_gui.x_obj.te_xpos-1, x->x_gui.x_obj.te_ypos-2,
	     x->x_gui.x_obj.te_xpos + IOWIDTH-1, x->x_gui.x_obj.te_ypos-1,
	     x, 0);
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xIN%d\n",
	     canvas,
	     x->x_gui.x_obj.te_xpos+x->x_gui.x_w+1-IOWIDTH, x->x_gui.x_obj.te_ypos-2,
	     x->x_gui.x_obj.te_xpos+x->x_gui.x_w+1, x->x_gui.x_obj.te_ypos-1,
	     x, 1);
    }
}


static void vu_draw_move(t_vu *x, t_glist *glist)
{
    t_canvas *canvas=glist_getcanvas(glist);

    int w4=x->x_gui.x_w/4, quad1=x->x_gui.x_obj.te_xpos+w4+1;
    int quad3=x->x_gui.x_obj.te_xpos+x->x_gui.x_w-w4, end=x->x_gui.x_obj.te_xpos+x->x_gui.x_w+4;
    int k1=x->x_led_size+1, k2=IEM_VU_STEPS+1, k3=k1/2;
    int yyy, i, k4=x->x_gui.x_obj.te_ypos-k3;

    sys_vgui(".x%x.c coords %xBASE %d %d %d %d\n",
	     canvas, x, x->x_gui.x_obj.te_xpos-1, x->x_gui.x_obj.te_ypos-2,
	     x->x_gui.x_obj.te_xpos+x->x_gui.x_w+1,x->x_gui.x_obj.te_ypos+x->x_gui.x_h+2);
    for(i=1; i<=IEM_VU_STEPS; i++)
    {
	yyy = k4 + k1*(k2-i);
	sys_vgui(".x%x.c coords %xRLED%d %d %d %d %d\n",
		 canvas, x, i, quad1, yyy, quad3, yyy);
	if(((i+2)&3) && (x->x_scale))
	    sys_vgui(".x%x.c coords %xSCALE%d %d %d\n",
		     canvas, x, i, end, yyy+k3);
    }
    if(x->x_scale)
    {
	i=IEM_VU_STEPS+1;
	yyy = k4 + k1*(k2-i);
	sys_vgui(".x%x.c coords %xSCALE%d %d %d\n",
		 canvas, x, i, end, yyy+k3);
    }
    vu_update_peak(x, glist);
    vu_update_rms(x, glist);
    sys_vgui(".x%x.c coords %xLABEL %d %d\n",
	     canvas, x, x->x_gui.x_obj.te_xpos+x->x_gui.x_ldx,
	     x->x_gui.x_obj.te_ypos+x->x_gui.x_ldy);
    if(!x->x_gui.x_fsf.x_snd_able)
    {
        sys_vgui(".x%x.c coords %xOUT%d %d %d %d %d\n",
	     canvas, x, 0,
	     x->x_gui.x_obj.te_xpos-1, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+1,
	     x->x_gui.x_obj.te_xpos + IOWIDTH-1, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+2);
        sys_vgui(".x%x.c coords %xOUT%d %d %d %d %d\n",
	     canvas, x, 1,
	     x->x_gui.x_obj.te_xpos+x->x_gui.x_w+1-IOWIDTH, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+1,
		 x->x_gui.x_obj.te_xpos+x->x_gui.x_w+1, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+2);
    }
    if(!x->x_gui.x_fsf.x_rcv_able)
    {
    sys_vgui(".x%x.c coords %xIN%d %d %d %d %d\n",
	     canvas, x, 0,
	     x->x_gui.x_obj.te_xpos-1, x->x_gui.x_obj.te_ypos-2,
	     x->x_gui.x_obj.te_xpos + IOWIDTH-1, x->x_gui.x_obj.te_ypos-1);
    sys_vgui(".x%x.c coords %xIN%d %d %d %d %d\n",
	     canvas, x, 1,
	     x->x_gui.x_obj.te_xpos+x->x_gui.x_w+1-IOWIDTH, x->x_gui.x_obj.te_ypos-2,
	     x->x_gui.x_obj.te_xpos+x->x_gui.x_w+1, x->x_gui.x_obj.te_ypos-1);
    }
}

static void vu_draw_erase(t_vu* x,t_glist* glist)
{
    int i;
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c delete %xBASE\n", canvas, x);
    for(i=1; i<=IEM_VU_STEPS; i++)
    {
	sys_vgui(".x%x.c delete %xRLED%d\n", canvas, x, i);
	if(((i+2)&3) && (x->x_scale))
	    sys_vgui(".x%x.c delete %xSCALE%d\n", canvas, x, i);
    }
    if(x->x_scale)
    {
	i=IEM_VU_STEPS+1;
	sys_vgui(".x%x.c delete %xSCALE%d\n", canvas, x, i);
    }
    sys_vgui(".x%x.c delete %xPLED\n", canvas, x);
    sys_vgui(".x%x.c delete %xRCOVER\n", canvas, x);
    sys_vgui(".x%x.c delete %xLABEL\n", canvas, x);
    if(!x->x_gui.x_fsf.x_snd_able)
    {
        sys_vgui(".x%x.c delete %xOUT%d\n", canvas, x, 0);
        sys_vgui(".x%x.c delete %xOUT%d\n", canvas, x, 1);
    }
    if(!x->x_gui.x_fsf.x_rcv_able)
    {
        sys_vgui(".x%x.c delete %xIN%d\n", canvas, x, 0);
        sys_vgui(".x%x.c delete %xIN%d\n", canvas, x, 1);
    }
}

static void vu_draw_config(t_vu* x, t_glist* glist)
{
    int i;
    t_canvas *canvas=glist_getcanvas(glist);

    sys_vgui(".x%x.c itemconfigure %xBASE -fill #%6.6x\n", canvas, x, x->x_gui.x_bcol);
    for(i=1; i<=IEM_VU_STEPS; i++)
    {
	sys_vgui(".x%x.c itemconfigure %xRLED%d -width %d\n", canvas, x, i,
		 x->x_led_size);
	if(((i+2)&3) && (x->x_scale))
	    sys_vgui(".x%x.c itemconfigure %xSCALE%d -text {%s} -font {%s %d bold} -fill #%6.6x\n",
		     canvas, x, i, iemgui_vu_scale_str[i], x->x_gui.x_font, x->x_gui.x_fontsize,
		     x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol);
    }
    if(x->x_scale)
    {
	i=IEM_VU_STEPS+1;
	sys_vgui(".x%x.c itemconfigure %xSCALE%d -text {%s} -font {%s %d bold} -fill #%6.6x\n",
		 canvas, x, i, iemgui_vu_scale_str[i], x->x_gui.x_font, x->x_gui.x_fontsize,
		 x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol);
    }
    sys_vgui(".x%x.c itemconfigure %xLABEL -font {%s %d bold} -fill #%6.6x -text {%s} \n",
	     canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize,
	     x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
	     strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"");

    sys_vgui(".x%x.c itemconfigure %xRCOVER -fill #%6.6x -outline #%6.6x\n", canvas,
	     x, x->x_gui.x_bcol, x->x_gui.x_bcol);
    sys_vgui(".x%x.c itemconfigure %xPLED -width %d\n", canvas, x,
	     x->x_led_size);
}

static void vu_draw_io(t_vu* x, t_glist* glist, int old_snd_rcv_flags)
{
    t_canvas *canvas=glist_getcanvas(glist);

    if((old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && !x->x_gui.x_fsf.x_snd_able)
    {
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xOUT%d\n",
	     canvas,
	     x->x_gui.x_obj.te_xpos-1, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+1,
	     x->x_gui.x_obj.te_xpos + IOWIDTH-1, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+2,
	     x, 0);
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xOUT%d\n",
	     canvas,
	     x->x_gui.x_obj.te_xpos+x->x_gui.x_w+1-IOWIDTH, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+1,
	     x->x_gui.x_obj.te_xpos+x->x_gui.x_w+1, x->x_gui.x_obj.te_ypos + x->x_gui.x_h+2,
	     x, 1);
    }
    if(!(old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && x->x_gui.x_fsf.x_snd_able)
    {
        sys_vgui(".x%x.c delete %xOUT%d\n", canvas, x, 0);
        sys_vgui(".x%x.c delete %xOUT%d\n", canvas, x, 1);
    }
    if((old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && !x->x_gui.x_fsf.x_rcv_able)
    {
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xIN%d\n",
	     canvas,
	     x->x_gui.x_obj.te_xpos-1, x->x_gui.x_obj.te_ypos-2,
	     x->x_gui.x_obj.te_xpos + IOWIDTH-1, x->x_gui.x_obj.te_ypos-1,
	     x, 0);
        sys_vgui(".x%x.c create rectangle %d %d %d %d -tags %xIN%d\n",
	     canvas,
	     x->x_gui.x_obj.te_xpos+x->x_gui.x_w+1-IOWIDTH, x->x_gui.x_obj.te_ypos-2,
	     x->x_gui.x_obj.te_xpos+x->x_gui.x_w+1, x->x_gui.x_obj.te_ypos-1,
	     x, 1);
    }
    if(!(old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && x->x_gui.x_fsf.x_rcv_able)
    {
        sys_vgui(".x%x.c delete %xIN%d\n", canvas, x, 0);
        sys_vgui(".x%x.c delete %xIN%d\n", canvas, x, 1);
    }
}

static void vu_draw_select(t_vu* x,t_glist* glist)
{
    int i;
    t_canvas *canvas=glist_getcanvas(glist);

    if(x->x_gui.x_fsf.x_selected)
    {
	pd_bind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
	sys_vgui(".x%x.c itemconfigure %xBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
	for(i=1; i<=IEM_VU_STEPS; i++)
	{
	    if(((i+2)&3) && (x->x_scale))
		sys_vgui(".x%x.c itemconfigure %xSCALE%d -fill #%6.6x\n",
			 canvas, x, i, IEM_GUI_COLOR_SELECTED);
	}
	if(x->x_scale)
	{
	    i=IEM_VU_STEPS+1;
	    sys_vgui(".x%x.c itemconfigure %xSCALE%d -fill #%6.6x\n",
		     canvas, x, i, IEM_GUI_COLOR_SELECTED);
	}
	sys_vgui(".x%x.c itemconfigure %xLABEL -fill #%6.6x\n", canvas, x, IEM_GUI_COLOR_SELECTED);
    }
    else
    {
	pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
	sys_vgui(".x%x.c itemconfigure %xBASE -outline #%6.6x\n", canvas, x, IEM_GUI_COLOR_NORMAL);
	for(i=1; i<=IEM_VU_STEPS; i++)
	{
	    if(((i+2)&3) && (x->x_scale))
		sys_vgui(".x%x.c itemconfigure %xSCALE%d -fill #%6.6x\n",
			 canvas, x, i, x->x_gui.x_lcol);
	}
	if(x->x_scale)
	{
	    i=IEM_VU_STEPS+1;
	    sys_vgui(".x%x.c itemconfigure %xSCALE%d -fill #%6.6x\n",
		     canvas, x, i, x->x_gui.x_lcol);
	}
	sys_vgui(".x%x.c itemconfigure %xLABEL -fill #%6.6x\n", canvas, x, x->x_gui.x_lcol);
    }
}

void vu_draw(t_vu *x, t_glist *glist, int mode)
{
    if(mode == IEM_GUI_DRAW_MODE_MOVE)
	vu_draw_move(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_NEW)
	vu_draw_new(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_SELECT)
	vu_draw_select(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_ERASE)
	vu_draw_erase(x, glist);
    else if(mode == IEM_GUI_DRAW_MODE_CONFIG)
	vu_draw_config(x, glist);
    else if(mode >= IEM_GUI_DRAW_MODE_IO)
	vu_draw_io(x, glist, mode - IEM_GUI_DRAW_MODE_IO);
}

/* ------------------------ vu widgetbehaviour----------------------------- */


static void vu_getrect(t_gobj *z, t_glist *owner,
		       int *xp1, int *yp1, int *xp2, int *yp2)
{
    t_vu* x = (t_vu*)z;

    *xp1 = x->x_gui.x_obj.te_xpos - 1;
    *yp1 = x->x_gui.x_obj.te_ypos - 2;
    *xp2 = x->x_gui.x_obj.te_xpos + x->x_gui.x_w + 1;
    *yp2 = x->x_gui.x_obj.te_ypos + x->x_gui.x_h + 2;
}

static void vu_save(t_gobj *z, t_binbuf *b)
{
    t_vu *x = (t_vu *)z;
    int bflcol[3], *ip1, *ip2;
    t_symbol *srl[3];

    iemgui_save(&x->x_gui, srl, bflcol);
    ip1 = (int *)(&x->x_gui.x_isa);
    ip2 = (int *)(&x->x_gui.x_fsf);
    binbuf_addv(b, "ssiisiissiiiiiiii", gensym("#X"),gensym("obj"),
		(t_int)x->x_gui.x_obj.te_xpos, (t_int)x->x_gui.x_obj.te_ypos,
		gensym("vu"), x->x_gui.x_w, x->x_gui.x_h,
		srl[1], srl[2],
		x->x_gui.x_ldx, x->x_gui.x_ldy,
		(*ip2)&IEM_FSTYLE_FLAGS_ALL, x->x_gui.x_fontsize,
		bflcol[0], bflcol[2], x->x_scale, (*ip1)&IEM_INIT_ARGS_ALL);
    binbuf_addv(b, ";");
}

void vu_check_height(t_vu *x, int h)
{
    int n;

    n = h / IEM_VU_STEPS;
    if(n < IEM_VU_MINSIZE)
	n = IEM_VU_MINSIZE;
    x->x_led_size = n-1;
    x->x_gui.x_h = IEM_VU_STEPS * n;
}

static void vu_scale(t_vu *x, t_floatarg fscale)
{
    int i, scale = (int)fscale;

    if(scale != 0) scale = 1;
    if(x->x_scale && !scale)
    {
	t_canvas *canvas=glist_getcanvas(x->x_gui.x_glist);

	x->x_scale = (int)scale;
	if(glist_isvisible(x->x_gui.x_glist))
	{
	    for(i=1; i<=IEM_VU_STEPS; i++)
	    {
		if((i+2)&3)
		    sys_vgui(".x%x.c delete %xSCALE%d\n", canvas, x, i);
	    }
	    i=IEM_VU_STEPS+1;
	    sys_vgui(".x%x.c delete %xSCALE%d\n", canvas, x, i);
	}
    }
    if(!x->x_scale && scale)
    {
	int w4=x->x_gui.x_w/4, end=x->x_gui.x_obj.te_xpos+x->x_gui.x_w+4;
	int k1=x->x_led_size+1, k2=IEM_VU_STEPS+1, k3=k1/2;
	int yyy, k4=x->x_gui.x_obj.te_ypos-k3;
	t_canvas *canvas=glist_getcanvas(x->x_gui.x_glist);

	x->x_scale = (int)scale;
	if(glist_isvisible(x->x_gui.x_glist))
	{
	    for(i=1; i<=IEM_VU_STEPS; i++)
	    {
		yyy = k4 + k1*(k2-i);
		if((i+2)&3)
		    sys_vgui(".x%x.c create text %d %d -text {%s} -anchor w \
			     -font {%s %d bold} -fill #%6.6x -tags %xSCALE%d\n",
			     canvas, end, yyy+k3, iemgui_vu_scale_str[i], x->x_gui.x_font, x->x_gui.x_fontsize,
			     x->x_gui.x_lcol, x, i);
	    }
	    i=IEM_VU_STEPS+1;
	    yyy = k4 + k1*(k2-i);
	    sys_vgui(".x%x.c create text %d %d -text {%s} -anchor w \
		     -font {%s %d bold} -fill #%6.6x -tags %xSCALE%d\n",
		     canvas, end, yyy+k3, iemgui_vu_scale_str[i], x->x_gui.x_font, x->x_gui.x_fontsize,
		     x->x_gui.x_lcol, x, i);
	}
    }
}

static void vu_properties(t_gobj *z, t_glist *owner)
{
    t_vu *x = (t_vu *)z;
    char buf[800];
    t_symbol *srl[3];

    iemgui_properties(&x->x_gui, srl);
    sprintf(buf, "pdtk_iemgui_dialog %%s VU-METER \
	    --------dimensions(pix)(pix):-------- %d %d width: %d %d height: \
	    empty 0.0 empty 0.0 empty %d \
	    %d no_scale scale %d %d \
	    %s %s \
	    %s %d %d \
	    %d %d \
	    %d %d %d\n",
	    x->x_gui.x_w, IEM_GUI_MINSIZE, x->x_gui.x_h, IEM_VU_STEPS*IEM_VU_MINSIZE,
	    0,/*no_schedule*/
	    x->x_scale, -1, -1,/*no linlog, no init, no multi*/
	    "nosndno", srl[1]->s_name,/*no send*/
	    srl[2]->s_name, x->x_gui.x_ldx, x->x_gui.x_ldy,
	    x->x_gui.x_fsf.x_font_style, x->x_gui.x_fontsize,
	    0xffffff & x->x_gui.x_bcol, -1/*no front-color*/, 0xffffff & x->x_gui.x_lcol);
    gfxstub_new(&x->x_gui.x_obj.ob_pd, x, buf);
}

static void vu_dialog(t_vu *x, t_symbol *s, int argc, t_atom *argv)
{
    t_symbol *srl[3];
    int w = (int)atom_getintarg(0, argc, argv);
    int h = (int)atom_getintarg(1, argc, argv);
    int scale = (int)atom_getintarg(4, argc, argv);
    int sr_flags;

    srl[0] = gensym("empty");
    sr_flags = iemgui_dialog(&x->x_gui, srl, argc, argv);
    x->x_gui.x_fsf.x_snd_able = 0;
    x->x_gui.x_isa.x_loadinit = 0;
    x->x_gui.x_w = iemgui_clip_size(w);
    vu_check_height(x, h);
    if(scale != 0)
	scale = 1;
    vu_scale(x, (float)scale);
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_CONFIG);
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_IO + sr_flags);
    (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
    canvas_fixlinesfor(glist_getcanvas(x->x_gui.x_glist), (t_text*)x);
}

static void vu_size(t_vu *x, t_symbol *s, int ac, t_atom *av)
{
    x->x_gui.x_w = iemgui_clip_size((int)atom_getintarg(0, ac, av));
    if(ac > 1)
        vu_check_height(x, (int)atom_getintarg(1, ac, av));
    if(glist_isvisible(x->x_gui.x_glist))
    {
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_CONFIG);
	canvas_fixlinesfor(glist_getcanvas(x->x_gui.x_glist), (t_text*)x);
    }
}

static void vu_delta(t_vu *x, t_symbol *s, int ac, t_atom *av)
{iemgui_delta((void *)x, &x->x_gui, s, ac, av);}

static void vu_pos(t_vu *x, t_symbol *s, int ac, t_atom *av)
{iemgui_pos((void *)x, &x->x_gui, s, ac, av);}

static void vu_color(t_vu *x, t_symbol *s, int ac, t_atom *av)
{iemgui_color((void *)x, &x->x_gui, s, ac, av);}

static void vu_receive(t_vu *x, t_symbol *s)
{iemgui_receive(x, &x->x_gui, s);}

static void vu_label(t_vu *x, t_symbol *s)
{iemgui_label((void *)x, &x->x_gui, s);}

static void vu_label_pos(t_vu *x, t_symbol *s, int ac, t_atom *av)
{iemgui_label_pos((void *)x, &x->x_gui, s, ac, av);}

static void vu_label_font(t_vu *x, t_symbol *s, int ac, t_atom *av)
{iemgui_label_font((void *)x, &x->x_gui, s, ac, av);}

static void vu_float(t_vu *x, t_floatarg rms)
{
    int i;

    if(rms <= IEM_VU_MINDB)
	x->x_rms = 0;
    else if(rms >= IEM_VU_MAXDB)
	x->x_rms = IEM_VU_STEPS;
    else
    {
	int i = (int)(2.0*(rms + IEM_VU_OFFSET));
	x->x_rms = iemgui_vu_db2i[i];
    }
    i = (int)(100.0*rms + 10000.5);
    rms = 0.01*(float)(i - 10000);
    x->x_fr = rms;
    outlet_float(x->x_out_rms, rms);
    vu_update_rms(x, x->x_gui.x_glist);
}

static void vu_ft1(t_vu *x, t_floatarg peak)
{
    int i;

    if(peak <= IEM_VU_MINDB)
	x->x_peak = 0;
    else if(peak >= IEM_VU_MAXDB)
	x->x_peak = IEM_VU_STEPS;
    else
    {
	int i = (int)(2.0*(peak + IEM_VU_OFFSET));
	x->x_peak = iemgui_vu_db2i[i];
    }
    i = (int)(100.0*peak + 10000.5);
    peak = 0.01*(float)(i - 10000);
    x->x_fp = peak;
    outlet_float(x->x_out_peak, peak);
    vu_update_peak(x, x->x_gui.x_glist);
}

static void vu_list(t_vu *x, t_symbol *s, int ac, t_atom *av)
{
    int l=iemgui_list((void *)x, &x->x_gui, s, ac, av);

    if(l < 0)
    {
	if((IS_A_FLOAT(av,0))&&(IS_A_FLOAT(av,1)))
	{
	    vu_ft1(x, atom_getfloatarg(1, ac, av));
	    vu_float(x, atom_getfloatarg(0, ac, av));
	}
    }
    else if(l > 0)
    {
	(*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_MOVE);
	canvas_fixlinesfor(glist_getcanvas(x->x_gui.x_glist), (t_text*)x);
    }
}

static void vu_bang(t_vu *x)
{
    outlet_float(x->x_out_peak, x->x_fp);
    outlet_float(x->x_out_rms, x->x_fr);
    vu_update_rms(x, x->x_gui.x_glist);
    vu_update_peak(x, x->x_gui.x_glist);
}

static void *vu_new(t_symbol *s, int argc, t_atom *argv)
{
    t_vu *x = (t_vu *)pd_new(vu_class);
    int bflcol[]={-66577, -1, -1};
    t_symbol *srl[3];
    int w=IEM_GUI_DEFAULTSIZE, h=IEM_VU_STEPS*IEM_VU_DEFAULTSIZE;
    int ldx=35, ldy=8, f=0, fs=8, scale=1;
    int iinit=0, ifstyle=0;
    int ftbreak=IEM_BNG_DEFAULTBREAKFLASHTIME, fthold=IEM_BNG_DEFAULTHOLDFLASHTIME;
    t_iem_init_symargs *init=(t_iem_init_symargs *)(&iinit);
    t_iem_fstyle_flags *fstyle=(t_iem_fstyle_flags *)(&ifstyle);
    char str[144];

    srl[0] = gensym("empty");
    srl[1] = gensym("empty");
    srl[2] = gensym("empty");

    if((argc >= 11)&&IS_A_FLOAT(argv,0)&&IS_A_FLOAT(argv,1)
       &&(IS_A_SYMBOL(argv,2)||IS_A_FLOAT(argv,2))
       &&(IS_A_SYMBOL(argv,3)||IS_A_FLOAT(argv,3))
       &&IS_A_FLOAT(argv,4)&&IS_A_FLOAT(argv,5)
       &&IS_A_FLOAT(argv,6)&&IS_A_FLOAT(argv,7)
       &&IS_A_FLOAT(argv,8)&&IS_A_FLOAT(argv,9)&&IS_A_FLOAT(argv,10))
    {
	w = (int)atom_getintarg(0, argc, argv);
	h = (int)atom_getintarg(1, argc, argv);
	if(IS_A_SYMBOL(argv,2))
	    srl[1] = atom_getsymbolarg(2, argc, argv);
	else if(IS_A_FLOAT(argv,2))
	{
	    sprintf(str, "%d", (int)atom_getintarg(2, argc, argv));
	    srl[1] = gensym(str);
	}
	if(IS_A_SYMBOL(argv,3))
	    srl[2] = atom_getsymbolarg(3, argc, argv);
	else if(IS_A_FLOAT(argv,3))
	{
	    sprintf(str, "%d", (int)atom_getintarg(3, argc, argv));
	    srl[2] = gensym(str);
	}
	ldx = (int)atom_getintarg(4, argc, argv);
	ldy = (int)atom_getintarg(5, argc, argv);
	ifstyle = (int)atom_getintarg(6, argc, argv);
	fs = (int)atom_getintarg(7, argc, argv);
	bflcol[0] = (int)atom_getintarg(8, argc, argv);
	bflcol[2] = (int)atom_getintarg(9, argc, argv);
	scale = (int)atom_getintarg(10, argc, argv);
    }
    if((argc == 12)&&IS_A_FLOAT(argv,11))
	iinit = (int)atom_getintarg(11, argc, argv);
    x->x_gui.x_draw = (t_iemfunptr)vu_draw;
    iinit &= IEM_INIT_ARGS_ALL;
    ifstyle &= IEM_FSTYLE_FLAGS_ALL;

    fstyle->x_snd_able = 0;
    fstyle->x_rcv_able = 1;
    x->x_gui.x_glist = (t_glist *)canvas_getcurrent();
    x->x_gui.x_isa = *init;
    if(!strcmp(srl[1]->s_name, "empty")) fstyle->x_rcv_able = 0;
    x->x_gui.x_unique_num = 0;
    if(fstyle->x_font_style == 1) strcpy(x->x_gui.x_font, "helvetica");
    else if(fstyle->x_font_style == 2) strcpy(x->x_gui.x_font, "times");
    else { fstyle->x_font_style = 0;
	strcpy(x->x_gui.x_font, "courier"); }
    x->x_gui.x_fsf = *fstyle;
    iemgui_first_dollararg2sym(&x->x_gui, srl);
    if(x->x_gui.x_fsf.x_rcv_able) pd_bind(&x->x_gui.x_obj.ob_pd, srl[1]);
    x->x_gui.x_snd = srl[0];
    x->x_gui.x_rcv = srl[1];
    x->x_gui.x_lab = srl[2];
    x->x_gui.x_ldx = ldx;
    x->x_gui.x_ldy = ldy;

    if(fs < 4)
	fs = 4;
    x->x_gui.x_fontsize = fs;
    x->x_gui.x_w = iemgui_clip_size(w);
    vu_check_height(x, h);
    iemgui_all_colfromload(&x->x_gui, bflcol);
    if(scale != 0)
	scale = 1;
    x->x_scale = scale;
    x->x_peak = 0;
    x->x_rms = 0;
    x->x_fp = -101.0;
    x->x_fr = -101.0;
    iemgui_verify_snd_ne_rcv(&x->x_gui);
    inlet_new(&x->x_gui.x_obj, &x->x_gui.x_obj.ob_pd, &s_float, gensym("ft1"));
    x->x_out_rms = outlet_new(&x->x_gui.x_obj, &s_float);
    x->x_out_peak = outlet_new(&x->x_gui.x_obj, &s_float);
    return (x);
}

static void vu_free(t_vu *x)
{
    if(x->x_gui.x_fsf.x_selected)
	pd_unbind(&x->x_gui.x_obj.ob_pd, iemgui_key_sym);
    if(x->x_gui.x_fsf.x_rcv_able)
	pd_unbind(&x->x_gui.x_obj.ob_pd, x->x_gui.x_rcv);
}

void vu_setup(void)
{
    vu_class = class_new(gensym("vu"), (t_newmethod)vu_new, (t_method)vu_free,
			 sizeof(t_vu), 0, A_GIMME, 0);
    class_addbang(vu_class,vu_bang);
    class_addfloat(vu_class,vu_float);
    class_addmethod(vu_class, (t_method)vu_ft1, gensym("ft1"), A_FLOAT, 0);
    class_addlist(vu_class, vu_list);
    class_addmethod(vu_class, (t_method)vu_dialog, gensym("dialog"),
		    A_GIMME, 0);
    class_addmethod(vu_class, (t_method)vu_size, gensym("size"), A_GIMME, 0);
    class_addmethod(vu_class, (t_method)vu_scale, gensym("scale"), A_DEFFLOAT, 0);
    class_addmethod(vu_class, (t_method)vu_delta, gensym("delta"), A_GIMME, 0);
    class_addmethod(vu_class, (t_method)vu_pos, gensym("pos"), A_GIMME, 0);
    class_addmethod(vu_class, (t_method)vu_color, gensym("color"), A_GIMME, 0);
    class_addmethod(vu_class, (t_method)vu_receive, gensym("receive"), A_DEFSYM, 0);
    class_addmethod(vu_class, (t_method)vu_label, gensym("label"), A_DEFSYM, 0);
    class_addmethod(vu_class, (t_method)vu_label_pos, gensym("label_pos"), A_GIMME, 0);
    class_addmethod(vu_class, (t_method)vu_label_font, gensym("label_font"), A_GIMME, 0);
    if(!iemgui_key_sym)
	iemgui_key_sym = gensym("#keyname");
    vu_widgetbehavior.w_getrectfn =    vu_getrect;
    vu_widgetbehavior.w_displacefn =   iemgui_displace;
    vu_widgetbehavior.w_selectfn =     iemgui_select;
    vu_widgetbehavior.w_activatefn =   NULL;
    vu_widgetbehavior.w_deletefn =     iemgui_delete;
    vu_widgetbehavior.w_visfn =        iemgui_vis;
    vu_widgetbehavior.w_clickfn =      NULL;
    vu_widgetbehavior.w_propertiesfn = vu_properties;
    vu_widgetbehavior.w_savefn =       vu_save;
    class_setwidget(vu_class,&vu_widgetbehavior);
    class_sethelpsymbol(vu_class, gensym("vu"));
}







/* ------------------------ setup routine ------------------------- */

void g_7_guis_setup(void)
{
    bng_setup();
    hslider_setup();
    hdial_setup();
    toggle_setup();
    my_canvas_setup();
    vslider_setup();
    vu_setup();
}
